UNIT GLFLexikon;

{$F+}
{$O+}

(*  (C) 1991, 92, 93 by Johannes Leckebusch
    Version 0.703
    Stand: 23. 11. 93

    Hinweis: Die Routinen "naechsterBegriff", "vorigerBegriff" usw.
    von einNetzString sollten nicht benutzt werden. Sie sind nur als
    interne Hilfroutinen fr die Implementierung von "ActualElement",
    "FirstElement", "NextElement", "PrevElement", "NtesElement" und
    "LastElement" zu betrachten, die von eineStruktur als abstrakte
    Methoden ererbt wurden.

    Fr "aktuellerBegriffsIndex" gilt vorlufig folgende Regelung:
    Der Index 0 deutet auf den lexikalisch ersten Begriff (nach dem
    Leerstring). Ein Index von -1 bedeutet, da der Index ungltig ist.
    Momentan wei ich keine Methode, um bei Methoden wie "sucheBegriff"
    oder "naechsterBegriff" ab einem vorgegebenen Pointer den aktuellen
    Index direkt zu ermitteln. Erfolgt eine Operation, die den Index
    verwendet, so mu sie eben die Begriffe vom lexikalisch ersten an
    durchzhlen.

    Der Pointer "aktuellerBegriff" wird unabhngig vom Index verwaltet,
    er ist NIL, wenn er ungltig ist, z. B. nach einer Suchoperation ohne
    positives Ergebnis.

    Man knnte eine Prozedur "Durchzaehlen" schreiben, welche die Begriffe
    in ihrer lexikalischen Reihenfolge anhand der Verpointerung durchluft
    und das Feld Referenz auf n = Ordnung des Begriffes setzt, ohne dabei
    irgendwelche lexikalischen Vergleiche durchzufhren. Dies knnte auch
    ein "DurchzaehlenBisBegriff" sein, das stoppt, wenn das Objekt mit
    @SELF = BegriffsPointer aufgesucht wurde. ->IndexVonBegriff.
*)

INTERFACE

USES  Drivers, Objects, Views, GLFTypes, GLFBase, GLFWordBase, GLFViews;

TYPE
     einLexikonPointer = ^einLexikon;
     einLexikon = OBJECT (einNetzString)
                        AnzahlBegriffe: LONGINT;
                        aktuellerBegriffsIndex: LONGINT;
                        aktuellerBegriff: GLP;
                        Environment: GLP;

                        StartBegr, PreSelectBegr: GLP;
                        NStart, NPreSelect:       LONGINT;
                        Abmess:                   TRect;
                        SpaltenBreite:            INTEGER;
                        FenOffset:                TPoint;

                        CONSTRUCTOR Init (CONST xflags: einFlagSet);
                        DESTRUCTOR Done; VIRTUAL;
                        PROCEDURE initAbmess (VAR a: TRect; VAR spbreite: INTEGER);
                          VIRTUAL;
                        FUNCTION erzeugeMich (CONST xflags: einFlagSet): GLP; VIRTUAL;

                        FUNCTION isSubclass (CONST class: einGenTyp): BOOLEAN; VIRTUAL;

                        PROCEDURE LoeschPfad (CONST Sig: LONGINT); VIRTUAL;
                        PROCEDURE nulleNetzFlag (CONST flag: einFlag; CONST Sig: LONGINT);
                          VIRTUAL;
                        PROCEDURE setzeNetzFlag (CONST flag: einFlag; CONST Sig: LONGINT);
                          VIRTUAL;
                        FUNCTION StringVonDaten: STRING; VIRTUAL;

                        FUNCTION Edit (CONST view: PBuffText; CONST x, y, b, h: WORD;
                                       CONST name: GLP): GLP;
                          VIRTUAL;

                        PROCEDURE setAbmess (CONST a: TRect; CONST spbreite: INTEGER);
                          VIRTUAL;
                        PROCEDURE getAbmess (VAR a: TRect; VAR spbreite: INTEGER);
                          VIRTUAL;
                        PROCEDURE setOffset (CONST o: TPoint); VIRTUAL;
                        PROCEDURE getOffset (VAR o: TPoint); VIRTUAL;
                        PROCEDURE setPos (CONST p: TPoint); VIRTUAL;
                        PROCEDURE getPos (VAR p: TPoint); VIRTUAL;

                        PROCEDURE resetBegriff; VIRTUAL;
                        FUNCTION ersterBegriff: GLP; VIRTUAL;

                        FUNCTION indiziereBegriffe (VAR Index: LONGINT;
                                                    ind: WORD;
                                                    CONST stop: GLP;
                                                    VAR gefunden: BOOLEAN): GLP; VIRTUAL;

                          (* siehe Vorbemerkung - liefert Anzahl der Begriffe
                             und setzt diese. In jedem Begriffsobjekt wird
                             das Feld "meineReferenz" auf den Index gesetzt. "Stop" ist
                             der Begriff, bei dem gestoppt werden soll, NIL fr
                             Ende des Netzes. Gibt immer den letzten Begriff zurck *)

                        FUNCTION zaehleBegriffe: LONGINT; VIRTUAL;
                          (* re-indiziert Begriffe und gibt Anzahl zurck *)
                        FUNCTION IndexVonBegriff (CONST Begriff: GLP): LONGINT;
                          VIRTUAL;
                          (* Durchzhlen bis zum ang. Begriff, setzt
                             aktuellenBegriff und Index *)
                        FUNCTION IndexVonObjekt (CONST Objekt: GLP): LONGINT;
                          VIRTUAL;
                          (* Durchsucht die Wortbases nach dem assoziierten
                             Objekt und liefert seinen Index *)

                        (* berschriebene virtuelle Methoden von einNetzString *)
                        (* - diese verwalten den aktuellenBegriff und die Indizes *)

                        FUNCTION sucheBegriff (CONST Param: ParamP;
                                               CONST ind: WORD;
                                               VAR Absender: GLP;
                                               VAR ElemIndex: LONGINT): GLP; VIRTUAL;
                           (* suche String im Netz,
                              NIL, wenn unbekannt, sonst WortHandle *)
                        FUNCTION insertBegriff (CONST Param: ParamP;
                                                CONST ind: WORD;
                                                CONST absender, rueckverweis: GLP;
                                                CONST absind: WORD): GLP; VIRTUAL;
                        FUNCTION letzterBegriff: GLP; VIRTUAL;
                        FUNCTION naechsterBegriff (ind: WORD): GLP; VIRTUAL;
                          (* nchster nach dem aktuellenBegriff *)
                        FUNCTION vorigerBegriff {(ind: WORD; n: LONGINT)}: GLP; VIRTUAL;
                          (* voriger nach dem aktuellen Begriff *)
                        FUNCTION aehnlichsterBegriff (CONST wort: StringPointer;
                                                      VAR verg: StringRelation;
                                                      VAR gleichbis: WORD;
                                                      CONST ind: WORD;
                                                      VAR abs: GLP): GLP; VIRTUAL;

                        FUNCTION findeBegriff (wort: STRING; CONST exakt: BOOLEAN): GLP;
                          VIRTUAL;
                          (*
                             Wenn exakt gesetzt, kommt NIL zurck, wenn der Begriff nicht
                             vorhanden ist, sonst ein Pointer auf den hnlichsten
                             Begriff. Im Unterschied zu "aehnlichster" braucht kein
                             Stringpointer bergeben zu werden (die Routine ruft
                             aber je nachdem "aehnlichster" oder "suche-Begriff" auf *)

                        FUNCTION entferneBegriff (CONST begr: GLP): GLP; VIRTUAL;
                        (* entfernt begr aus der WordBase und liefert dessen
                           Assoziation zurck *)

                        (* von "eineStruktur" ererbte Methoden: *)

                        FUNCTION IndexIst: LONGINT; VIRTUAL;
                        FUNCTION LaengeIst: LONGINT; VIRTUAL;

                        FUNCTION ActualElement: GLP; VIRTUAL;
                        FUNCTION getActual: GLP; VIRTUAL;
                          (* Ergibt das Objekt unter dem ListenCursor *)

                        (* Funktionen mit Seiteneffekt auf aktuellen Begriff *)
                        FUNCTION FirstElement: GLP; VIRTUAL;
                             (* alphabetisch ersten Begriff *)
                        FUNCTION NextElement: GLP; VIRTUAL;
                        FUNCTION PrevElement: GLP; VIRTUAL;
                        FUNCTION NtesElement (CONST N: LONGINT): GLP; VIRTUAL;
                        FUNCTION NOrLastElement (CONST N: LONGINT): GLP; VIRTUAL;
                        FUNCTION LastElement: GLP; VIRTUAL;
                        FUNCTION FindElement (CONST Objekt: GLP): GLP; VIRTUAL;
                        FUNCTION NtesObjekt (CONST Ind: LONGINT): GLP; VIRTUAL;

                        (* Funktionen ohne Seiteneffekt auf aktuellen Begriff *)
                        FUNCTION FirstElementIs: GLP; VIRTUAL;
                             (* alphabetisch ersten Begriff *)
                        FUNCTION NextElementIs: GLP; VIRTUAL;
                        FUNCTION PrevElementIs: GLP; VIRTUAL;
                        FUNCTION NtesElementIs (CONST N: LONGINT): GLP; VIRTUAL;
                        FUNCTION NOrLastElementIs (CONST N: LONGINT): GLP; VIRTUAL;
                        FUNCTION LastElementIs: GLP; VIRTUAL;
                        FUNCTION FindElementIs (CONST Objekt: GLP): GLP; VIRTUAL;
                        FUNCTION NtesObjektIs (CONST Ind: LONGINT): GLP; VIRTUAL;

                        FUNCTION NameVonIndexIs (CONST Ind: LONGINT): GLP; VIRTUAL;
                        FUNCTION NameVonIndex (CONST Ind: LONGINT): GLP; VIRTUAL;

                        FUNCTION insertString (CONST s: STRING;
                                               CONST eindeutig: BOOLEAN):
                          einNetzStringPointer; VIRTUAL;
                        PROCEDURE schreibBegriff (CONST view: PBuffText; CONST begr: GLP); VIRTUAL;
                        PROCEDURE schreibListe (CONST view: PBuffText; start: einNetzStringPointer);
                        PROCEDURE Monitor (CONST view: PBuffText); VIRTUAL;
                      END (* einLexikon *);

IMPLEMENTATION

USES DOS, App, EditStuff, GLFStructures, GLFWindows;

CONSTRUCTOR einLexikon.Init;
VAR   Dummy: einNetzStringPointer;
BEGIN
  einNetzString.Init (xflags);
  MeinTyp := Lexikon;
  Dummy := insertString ('', TRUE);
    (* Dies ist "der leere Begriff", der allen
       anderen lexikalisch vorangeht *)
  {AnzahlBegriffe := 0;}
  aktuellerBegriffsIndex := -1;
  {
  aktuellerBegriff := NIL;
  Environment := NIL;
  NStart := 0; NPreSelect := 0;
  StartBegr := NIL; PreSelectBegr := NIL;
  }
  initAbmess (Abmess, Spaltenbreite);
END (* Init *);

DESTRUCTOR einLexikon.Done;
BEGIN
  (* zeigMeldung ('einLexikon', 'Done'); *)
  einNetzString.Done;
  IF aktuellerBegriff <> NIL THEN DISPOSE (aktuellerBegriff, Done);
END;

PROCEDURE einLexikon.initAbmess (VAR a: TRect; VAR spbreite: INTEGER);
BEGIN
  a.Assign (0, 0, 70, 20);
  zentriere (a);
  spbreite := 30;
END (* initAbmess *);

FUNCTION einLexikon.erzeugeMich (CONST xflags: einFlagSet): GLP;
BEGIN
  erzeugeMich := NEW (einLexikonPointer, Init (xflags));
END (* erzeugeMich *);

FUNCTION einLexikon.isSubclass (CONST class: einGenTyp): BOOLEAN;
BEGIN
  isSubclass := (class = Lexikon) OR INHERITED isSubClass (class);
END (* Member *);

PROCEDURE einLexikon.LoeschPfad (CONST Sig: LONGINT);
BEGIN
  INHERITED LoeschPfad (Sig);
  IF aktuellerBegriff <> NIL THEN BEGIN
    IF Sig = aktuellerBegriff^.meineSignatur THEN aktuellerBegriff := NIL
    ELSE aktuellerBegriff^.LoeschPfad (Sig);
  END (* IF *);
END (* LoeschPfad *);

PROCEDURE einLexikon.nulleNetzFlag (CONST flag: einFlag; CONST Sig: LONGINT);
BEGIN
  IF (Sig <> meineSignatur) THEN BEGIN
    INHERITED nulleNetzFlag (flag, Sig);
    IF aktuellerBegriff <> NIL THEN aktuellerBegriff^.nulleNetzFlag (flag, Sig);
  END;
END (* nulleNetzFlag *);

PROCEDURE einLexikon.setzeNetzFlag (CONST flag: einFlag; CONST Sig: LONGINT);
BEGIN
  IF (Sig <> meineSignatur) THEN BEGIN
    INHERITED setzeNetzFlag (flag, Sig);
    IF aktuellerBegriff <> NIL THEN aktuellerBegriff^.setzeNetzFlag (flag, Sig);
  END;
END (* setzeNetzFlag *);

FUNCTION einLexikon.StringVonDaten: STRING;
BEGIN
  StringVonDaten := GLF.StringVonDaten;
END (* StringVonDaten *);

FUNCTION einLexikon.Edit (CONST view: PBuffText;
                          CONST x, y, b, h: WORD;
                          CONST name: GLP): GLP;
VAR   p:        GLP;
      str:      STRING;
      Param:    ParamP;
BEGIN
  str := '';
  p := EditName (name, @SELF, x + 1, y + 1, b, str);
  {strp := @str;}
  Param := NewParam;
  WITH Param^ DO BEGIN
    wort := @str; neu := TRUE;
  END (* WITH *);
  p := insertBegriff (Param, 0, NIL, NIL, 0);
  Edit := p;
END (* Edit *);

PROCEDURE einLexikon.setAbmess (CONST a: TRect;
                                CONST spbreite: INTEGER);
BEGIN
  Abmess := a;
END (* setAbmess *);

PROCEDURE einLexikon.getAbmess (VAR a: TRect; VAR spbreite: INTEGER);
BEGIN
  a := Abmess;
END (* getAbmess *);

PROCEDURE einLexikon.setPos (CONST p: TPoint);
VAR dummyp: POINTER;
BEGIN
  Abmess.Move (p.X - Abmess.A.X, p.Y - Abmess.A.Y);
  dummyp := Message (DeskTop, evBroadcast, cmGLFObjektUpdate, @SELF);
END (* setPos *);

PROCEDURE einLexikon.getPos (VAR p: TPoint);
BEGIN
  p := Abmess.A;
END (* getPos *);

PROCEDURE einLexikon.setOffset (CONST o: TPoint);
BEGIN
  FenOffset := o;
END (* setOffset *);

PROCEDURE einLexikon.getOffset (VAR o: TPoint);
BEGIN
  o := FenOffset;
END (* getOffset *);

(********************* Begriffsorientierte Funktionen *****************)

PROCEDURE einLexikon.resetBegriff;
BEGIN
  aktuellerBegriff := NIL; aktuellerBegriffsindex := -1;
END (* resetBegriff *);

FUNCTION einLexikon.ersterBegriff: GLP;
(* alphabetisch ersten Begriff *)
BEGIN
  IF SeitenVektor <> NIL THEN BEGIN
    aktuellerBegriff := SeitenVektor^[0];
  END
  ELSE BEGIN
    aktuellerBegriff := NIL;
  END (* ELSE *);
  IF aktuellerBegriff <> NIL THEN aktuellerBegriffsIndex := 0
  ELSE aktuellerBegriffsIndex := -1;
  ersterBegriff := aktuellerBegriff;
END (* ersterBegriff *);

FUNCTION einLexikon.indiziereBegriffe (VAR Index: LONGINT; ind: WORD;
                                           CONST stop: GLP;
                                           VAR gefunden: BOOLEAN): GLP;
(* siehe Vorbemerkung - liefert Anzahl der Begriffe
   und setzt diese *)
VAR p:                    GLP;
BEGIN
  p := ersterBegriff; Index := 0; gefunden := FALSE;
{zeigMeldung ('indiziereBegriffe', 'p = ' + atString (p) + ', ' + p^.StringVonTyp);}
  IF p <> NIL THEN BEGIN
    ind := einNetzStringPointer (p)^.LaengeIst;
    (* ind wird als erstes DECrementiert, wenn > 0 *)
    indiziereBegriffe :=
      einNetzStringPointer (p)^.indiziereBegriffe (Index, ind, stop, gefunden);
    AnzahlBegriffe := Index;
  END
  ELSE indiziereBegriffe := NIL;
END (* indiziereBegriffe *);

FUNCTION einLexikon.zaehleBegriffe: LONGINT;
VAR  Index:  LONGINT;
     gef:    BOOLEAN;
     dummy:  GLP;
BEGIN
  dummy := indiziereBegriffe (Index, 0, NIL, gef);
  zaehleBegriffe := Index;
END (* zaehleBegriffe *);

FUNCTION einLexikon.IndexVonBegriff (CONST Begriff: GLP): LONGINT;
(* Durchzhlen bis zum ang. Begriff, setzt
   aktuellenBegriff und Index *)
VAR  Index:  LONGINT;
     Anz:    LONGINT;
     gef:    BOOLEAN;
     dp:     GLP;
BEGIN
  Anz := AnzahlBegriffe;
  dp := indiziereBegriffe (Index, 0, Begriff, gef);
  IndexVonBegriff := Index;
  AnzahlBegriffe := Anz; (* weil das von indiziereBegriffe sonst auf den
    Index des gefundenen Begriffs verkrzt wird! *)
END (* IndexVonBegriff *);

(* berschriebene virtuelle Methoden von einNetzString *)
(* - diese verwalten den aktuellenBegriff und die Indizes *)

FUNCTION einLexikon.sucheBegriff (CONST Param: ParamP;
                                  CONST ind: WORD;
                                  VAR Absender: GLP;
                                  VAR ElemIndex: LONGINT): GLP;
(* suche String im Netz, NIL, wenn unbekannt, sonst WortHandle *)
VAR p: GLP;
BEGIN
  (* Das Suchen eines Begriffes erfolgt natrlich entlang der
     Zugriffspfade auf schnellstmgliche Weise. Wie kann man dabei
     den aktuellen Begriffsindex ermitteln? Siehe Vorbemerkung und
     "indiziereBegriffe" *)
  resetBegriff;
  WITH Param^ DO BEGIN
    normalisiereBezeichner (wort, PSchreib);
  END (* WITH *);
  Absender := NIL;
  aktuellerBegriff :=
    einNetzString.sucheBegriff (Param {wort}, ind,
                                {schreibExakt, PSchreib,} Absender,
                                aktuellerBegriffsIndex);
  IF aktuellerBegriff = NIL THEN resetBegriff;
  sucheBegriff := aktuellerBegriff; (* Hm - Index stimmt nicht! *)
END (* sucheBegriff *);

FUNCTION einLexikon.insertBegriff (CONST Param: ParamP;
                                   CONST ind: WORD;
                                   CONST absender, rueckverweis: GLP;
                                   CONST absind: WORD): GLP;
BEGIN
  resetBegriff;
  WITH Param^ DO BEGIN
    normalisiereBezeichner (wort, PSchreib);
    aktuellerBegriff :=
      einNetzString.insertBegriff (Param {wort, PSchreib}, ind, {neu,}
                                   absender, NIL, absind);
    IF (wort^.Laenge > 0) AND (aktuellerBegriff <> NIL) THEN BEGIN
      (* Hier fehlt noch ein Test, ob der Begriff schon vorhanden war!
         Vorlufig lasse ich vor dem Aufruf der Namensbrowser die Begriffe
         durchzhlen, um ggf. AnzahlBegriffe zu korrigieren!
         Das falsche Hochzhlen stammt von den Eintragungen der
         Dummy-Strings ' '. *)
      IF (AnzahlBegriffe <= 1) OR (STRING (wort^) <> ' ') THEN BEGIN
        INC (AnzahlBegriffe);
        setzeFlag (EditFlag);
      END (* IF *);
    END (* IF *);
  END (* WITH *);
  insertBegriff := aktuellerBegriff;
END (* insertBegriff *);

FUNCTION einLexikon.letzterBegriff: GLP;
(* Der wre eigentlich besonders einfach zu finden - man luft einfach
   unter dem Index 0 bis ans Ende der Verzeigerung und ermittelt dann
   den letzten Nachfolger der ersten Seitenkette. *)
VAR  Index:  LONGINT;
     gef:    BOOLEAN;
     dummy:  GLP;
BEGIN
  aktuellerBegriff := indiziereBegriffe (Index, 0, NIL, gef);
  letzterBegriff := aktuellerBegriff;
END (* letzterBegriff *);

FUNCTION einLexikon.naechsterBegriff (ind: WORD): GLP;
  (* nchster nach dem aktuellenBegriff *)
BEGIN
  IF aktuellerBegriff = NIL THEN BEGIN
    aktuellerBegriff := einNetzstring.naechsterBegriff (ind);
    aktuellerBegriffsIndex := 0;
  END
  ELSE BEGIN
    aktuellerBegriff :=
      einNetzStringPointer (aktuellerBegriff)^.naechsterBegriff (ind);
    IF (aktuellerBegriff <> NIL) AND (aktuellerBegriffsIndex >= 0) THEN
      INC (aktuellerBegriffsIndex)
    ELSE aktuellerBegriffsIndex := - 1;
  END (* ELSE *);
  naechsterBegriff := aktuellerBegriff;
END (* naechsterBegriff *);

FUNCTION einLexikon.vorigerBegriff {(ind: WORD; n: LONGINT)}: GLP;
  (* voriger nach dem aktuellen Begriff *)
VAR s:     STRING;
    abs:   GLP;
    begr:  GLP;
    dummy: LONGINT;
    Param: ParamP;
BEGIN
  IF aktuellerBegriff = NIL THEN BEGIN
     vorigerBegriff := NIL; EXIT;
  END (* IF aktuellerBegriff = NIL *);
  einNetzStringPointer (AktuellerBegriff)^.StringVonBegriff (s);
  abs := NIL;
  Param := NewParam;
  WITH Param^ DO BEGIN
    wort := @s; schreibExakt := TRUE;
  END (* WITH *);
  begr := sucheBegriff (Param {StringPointer (@s)}, 0, {TRUE, NIL,}
                        abs, dummy);
  IF abs = aktuellerBegriff THEN BEGIN
zeigMeldung ('einNetzString', 'vorigerBegriff = aktuellerBegriff');
    vorigerBegriff := NIL; EXIT;
  END;
  aktuellerBegriff := abs; DEC (aktuellerBegriffsIndex);
  vorigerBegriff := abs;
END (* vorigerBegriff *);

FUNCTION einLexikon.aehnlichsterBegriff (CONST wort: StringPointer;
                                         VAR verg: StringRelation;
                                         VAR gleichbis: WORD;
                                         CONST ind: WORD;
                                         VAR abs: GLP): GLP;
(* Das sollte auch recht effizient zu machen sein, dh. so da NICHT
   ALLE Begriffe durchlaufen werden! *)
VAR  PSchreib: PBITMAP;
BEGIN
  IF wort^.Laenge > 0 THEN BEGIN
    resetBegriff; (* vorlufig *)
    normalisiereBezeichner (wort, PSchreib);
(* VERDAMMT! normalisiereBezeichner ERZEUGT ja eine Bitmap! *)
    FreeMem (PSchreib, (wort^.Laenge - 1) DIV 8 + 1);
    aehnlichsterBegriff :=
      einNetzString.aehnlichsterBegriff (wort, verg, gleichbis, ind, abs);
  END
  ELSE aehnlichsterBegriff := NIL;
END (* aehnlichsterBegriff *);

FUNCTION einLexikon.findeBegriff (wort: STRING; CONST exakt: BOOLEAN): GLP;
VAR  strp:               StringPointer;
     p:                  GLP;
     ind, gleichbis:     WORD;
     verg:               StringRelation;
     abs:                GLP;
BEGIN
  IF wort <> '' THEN BEGIN
    strp := @wort;
    ind := 0; abs := NIL;
    p := aehnlichsterBegriff (strp, verg, gleichbis, ind, abs);
    IF (p <> NIL) AND (p <> @SELF) THEN BEGIN
      IF (verg = gleich) THEN BEGIN
        findeBegriff := p;
      END
      ELSE BEGIN
        IF exakt THEN BEGIN
          findeBegriff := NIL;
        END
        ELSE BEGIN
          findeBegriff := p;
        END (* ELSE *);
      END (* ELSE nicht gleich *);
    END (* IF p <> NIL *)
    ELSE BEGIN
      findeBegriff := NIL;
    END (* ELSE *);
  END (* wort <> '' *)
  ELSE BEGIN
    findeBegriff := NIL;
  END (* ELSE *);
END (* findeBegriff *);

FUNCTION einLexikon.entferneBegriff (CONST begr: GLP): GLP;
VAR
     wort:       STRING;
     strp:       StringPointer;
     p:                  GLP;
     ind, gleichbis:     WORD;
     verg:               StringRelation;
     abs:                GLP;
BEGIN
  wort := einNetzStringPointer (begr)^.StringVonDaten;
  strp := @wort;
  ind := 0; abs := NIL;
  p := aehnlichsterBegriff (strp, verg, gleichbis, ind, abs);
  entferneBegriff := einNetzStringPointer (begr)^.loescheBegriff (abs);
  resetBegriff;
END (* entferneBegriff *);

FUNCTION einLexikon.insertString (CONST s: STRING; CONST eindeutig: BOOLEAN):
  einNetzStringPointer;
VAR    Param:     ParamP;
BEGIN
  Param := NewParam;
  WITH Param^ DO BEGIN
    wort := @s; neu := eindeutig;
  END (* WITH *);
  insertString := einNetzStringPointer (insertBegriff
                     (Param, 0, NIL, NIL, 0)
                  );
END (* insertString *);

PROCEDURE einLexikon.schreibBegriff (CONST view: PBuffText; CONST begr: GLP);
VAR    str: STRING;
BEGIN
  IF begr = NIL THEN view^.WriteStrLn ('Begriff ist NIL')
  ELSE BEGIN
    einNetzStringPointer (begr)^.StringvonBegriff (str);
    view^.WriteChr ('"'); view^.WriteString (str, 0, 0); view^.WriteChr ('"');
  END;
END (* schreibBegriff *);

PROCEDURE einLexikon.schreibListe (CONST view: PBuffText;
                                         start: einNetzStringPointer);
BEGIN
  WHILE start <> NIL DO BEGIN
    schreibBegriff (view, start);
    start := einNetzStringPointer (start^.naechsterBegriff
                                    (start^.AnzahlSeiten));
  END (* WHILE *);
END (* schreibListe *);

(****************** von "eineStruktur" ererbte Methoden: **************)

FUNCTION einLexikon.IndexIst: LONGINT;
BEGIN
  IndexIst := aktuellerBegriffsIndex;
END (* IndexIst *);

FUNCTION einLexikon.LaengeIst: LONGINT;
BEGIN
  LaengeIst := AnzahlBegriffe;
END (* LaengeIst *);

FUNCTION einLexikon.ActualElement: GLP;
BEGIN
  ActualElement := aktuellerBegriff;
END (* ActualElement *);

FUNCTION einLexikon.getActual: GLP;
BEGIN
  IF aktuellerBegriff <> NIL THEN
    getActual := einStrukturPointer (aktuellerBegriff)^.getDataPointer
  ELSE getActual := NIL;
END (* getActual *);

FUNCTION einLexikon.FirstElement: GLP;
(* alphabetisch ersten Begriff *)
BEGIN
  FirstElement := ersterBegriff;
END (* FirstElement *);

FUNCTION einLexikon.NextElement: GLP;
VAR  ind:  WORD;
BEGIN
  ind := einNetzStringPointer (aktuellerBegriff)^.LaengeIst;
  NextElement := naechsterBegriff (ind);
END (* NextElement *);

FUNCTION einLexikon.IndexVonObjekt (CONST Objekt: GLP): LONGINT;
VAR Element: GLP;
BEGIN
  Element := ersterBegriff;
  WHILE (Element <> NIL) AND
        (einNetzStringPointer (Element)^.liesAssoziation <> Objekt) DO BEGIN
    Element := NextElement;
  END (* WHILE *);
  IF Element <> NIL THEN IndexVonObjekt := aktuellerBegriffsIndex
  ELSE IndexVonObjekt := -1;
END (* IndexVonObjekt *);

FUNCTION einLexikon.PrevElement: GLP;
BEGIN
  PrevElement := vorigerBegriff;
END (* PrevElement *);

FUNCTION einLexikon.NtesElement (CONST N: LONGINT): GLP;
VAR  p:    GLP;
     ind:  WORD;
BEGIN
  IF (aktuellerBegriff = NIL) OR (aktuellerBegriffsIndex < 0) OR
     (aktuellerBegriffsIndex > N) THEN BEGIN
    p := ersterBegriff;
  END;
  WHILE (aktuellerBegriffsIndex < N) AND (aktuellerBegriff <> NIL) DO BEGIN
    ind := einNetzStringPointer (aktuellerBegriff)^.LaengeIst;
    p := naechsterBegriff (ind);
  END (* WHILE *);
  NtesElement := aktuellerBegriff;
END (* NtesElement *);

FUNCTION einLexikon.NOrLastElement (CONST N: LONGINT): GLP;
BEGIN
  Abstract ('NOrLastElement');
END (* NOrLastElement *);

FUNCTION einLexikon.LastElement: GLP;
BEGIN
  LastElement := letzterBegriff;
END (* LastElement *);

FUNCTION einLexikon.FindElement (CONST Objekt: GLP): GLP;
BEGIN
zeigMeldung ('einLexikon (' + StringVonTyp,
             'FindElement NICHT IMPLEMENTIERT');
END (* FindElement *);

FUNCTION einLexikon.NtesObjekt (CONST Ind: LONGINT): GLP;
VAR  p:                  GLP;
     indx:               WORD;
BEGIN
  IF (aktuellerBegriff = NIL) OR (aktuellerBegriffsIndex < 0) OR
     (aktuellerBegriffsIndex > Ind) THEN BEGIN
    p := ersterBegriff;
  END;

  WHILE (aktuellerBegriffsIndex < Ind) AND (aktuellerBegriff <> NIL) DO BEGIN
    indx := einNetzStringPointer (aktuellerBegriff)^.LaengeIst;
    p := naechsterBegriff (indx);
  END (* WHILE *);
  IF aktuellerBegriff <> NIL THEN
    NtesObjekt := einNetzStringPointer (aktuellerBegriff)^.liesAssoziation
  ELSE NtesObjekt := NIL;
END (* NtesObjekt *);

(******************* Is-Funktionen ohne Seiteffekt auf Cursor ************)

FUNCTION einLexikon.FirstElementIs: GLP;
(* alphabetisch ersten Begriff *)
VAR  oldaktBegrIndex:    LONGINT;
     oldaktBegr:         GLP;
BEGIN
  oldaktBegrIndex := aktuellerBegriffsIndex;
  oldaktBegr := aktuellerBegriff;
  FirstElementIs := ersterBegriff;
  aktuellerBegriffsindex := oldaktBegrIndex;
  aktuellerBegriff := oldaktBegr;
END (* FirstElementIs *);

FUNCTION einLexikon.NextElementIs: GLP;
VAR  ind:  WORD;
VAR  oldaktBegrIndex:    LONGINT;
     oldaktBegr:         GLP;
BEGIN
  oldaktBegrIndex := aktuellerBegriffsIndex;
  oldaktBegr := aktuellerBegriff;
  ind := einNetzStringPointer (aktuellerBegriff)^.LaengeIst;
  NextElementIs := naechsterBegriff (ind);
  aktuellerBegriffsindex := oldaktBegrIndex;
  aktuellerBegriff := oldaktBegr;
END (* NextElementIs *);

FUNCTION einLexikon.PrevElementIs: GLP;
VAR  oldaktBegrIndex:    LONGINT;
     oldaktBegr:         GLP;
BEGIN
  oldaktBegrIndex := aktuellerBegriffsIndex;
  oldaktBegr := aktuellerBegriff;
zeigMeldung ('einLexikon (' + StringVonTyp,
             'PrevElementIs NICHT IMPLEMENTIERT');
  aktuellerBegriffsindex := oldaktBegrIndex;
  aktuellerBegriff := oldaktBegr;
END (* PrevElementIs *);

FUNCTION einLexikon.NtesElementIs (CONST N: LONGINT): GLP;
VAR  p:                  GLP;
     ind:                WORD;
     oldaktBegrIndex:    LONGINT;
     oldaktBegr:         GLP;
BEGIN
  oldaktBegrIndex := aktuellerBegriffsIndex;
  oldaktBegr := aktuellerBegriff;
  IF (aktuellerBegriff = NIL) OR (aktuellerBegriffsIndex < 0) OR
     (aktuellerBegriffsIndex > N) THEN BEGIN
    p := ersterBegriff;
  END;

  WHILE (aktuellerBegriffsIndex < N) AND (aktuellerBegriff <> NIL) DO BEGIN
    ind := einNetzStringPointer (aktuellerBegriff)^.LaengeIst;
    p := naechsterBegriff (ind);
  END (* WHILE *);
  NtesElementIs := aktuellerBegriff;
  aktuellerBegriffsindex := oldaktBegrIndex;
  aktuellerBegriff := oldaktBegr;
END (* NtesElementIs *);

FUNCTION einLexikon.NOrLastElementIs (CONST N: LONGINT): GLP;
VAR  oldaktBegrIndex:    LONGINT;
     oldaktBegr:         GLP;
BEGIN
  oldaktBegrIndex := aktuellerBegriffsIndex;
  oldaktBegr := aktuellerBegriff;
zeigMeldung ('einLexikon (' + StringVonTyp,
             'NOrLastElementIs NICHT IMPLEMENTIERT');
  aktuellerBegriffsindex := oldaktBegrIndex;
  aktuellerBegriff := oldaktBegr;
END (* NOrLastElementIs *);

FUNCTION einLexikon.LastElementIs: GLP;
VAR  oldaktBegrIndex:    LONGINT;
     oldaktBegr:         GLP;
BEGIN
  oldaktBegrIndex := aktuellerBegriffsIndex;
  oldaktBegr := aktuellerBegriff;
zeigMeldung ('einLexikon (' + StringVonTyp,
             'LastElementIs NICHT IMPLEMENTIERT');
  aktuellerBegriffsindex := oldaktBegrIndex;
  aktuellerBegriff := oldaktBegr;
END (* LastElementIs *);

FUNCTION einLexikon.FindElementIs (CONST Objekt: GLP): GLP;
VAR  oldaktBegrIndex:    LONGINT;
     oldaktBegr:         GLP;
BEGIN
  oldaktBegrIndex := aktuellerBegriffsIndex;
  oldaktBegr := aktuellerBegriff;
zeigMeldung ('einLexikon (' + StringVonTyp,
             'FindElementIs NICHT IMPLEMENTIERT');
  aktuellerBegriffsindex := oldaktBegrIndex;
  aktuellerBegriff := oldaktBegr;
END (* FindElementIs *);

FUNCTION einLexikon.NtesObjektIs (CONST Ind: LONGINT): GLP;
VAR  oldaktBegrIndex:    LONGINT;
     oldaktBegr:         GLP;
BEGIN
  oldaktBegrIndex := aktuellerBegriffsIndex;
  oldaktBegr := aktuellerBegriff;

  NtesObjektIs := NtesObjekt (Ind);

  aktuellerBegriffsindex := oldaktBegrIndex;
  aktuellerBegriff := oldaktBegr;
END (* NtesObjektIs *);

FUNCTION einLexikon.NameVonIndexIs (CONST Ind: LONGINT): GLP;
BEGIN
  NameVonIndexIs := NtesElementIs (Ind);
END (* NameVonIndexIs *);

FUNCTION einLexikon.NameVonIndex (CONST Ind: LONGINT): GLP;
BEGIN
  NameVonIndex := NtesElement (Ind);
END (* NameVonIndex *);

PROCEDURE einLexikon.Monitor;
BEGIN
  einNetzString.Monitor (view);
  view^.WriteString ('AnzahlBegriffe:   ', 0, 0); view^.WriteInt (AnzahlBegriffe); view^.WriteLn;
  view^.WriteString ('akt. Begr. Index: ', 0, 0); view^.WriteInt (aktuellerBegriffsIndex); view^.WriteLn;
  view^.WriteString ('aktuellerB. Poin: ', 0, 0); view^.WriteGLFPointer (aktuellerBegriff); view^.WriteLn;
  view^.WriteString ('Environment:      ', 0, 0); view^.WriteGLFPointer (Environment); view^.WriteLn;
END (* Monitor *);

END (* GLFLexikon *).