UNIT GLFLists;

{$F+}
{$O+}

(* (C) 1990-95 by Johannes Leckebusch
   Version 0.902
   Stand 07. 02. 1995
*)

(* Dieses Modul implementiert, aufbauend auf GLFNodes, allgemeine Operationen
   zur Verwaltung zeigerverketteter Listen wie Einfgen eines Elementes in
   eine Liste, Entfernen, Suchen des nten Elementes usw.

   VORSICHT! Am 1. 6. 91 NextElement und PrevElement so gendert, da sie
   NIL liefern, wenn kein nchstes/voriges Element mehr vorhanden ist.
   Haben vorher das letzte/erste Element zurckgegeben!

   Am 29. 7. 92 SetEnvironment und GetEnvironment von eineTabelle nach
   eineListe vorgezogen und dadurch Namensfehler in SimpleText-Windows
   nach Erzeugen ber das Men "Unterformulare" behoben.
*)

INTERFACE

USES Drivers, Objects, Views, GLFTypes, GLFBase, GLFQuadruple{, GLFLexikon};

TYPE

  einListenPointer = ^eineListe;
  eineListe = OBJECT (einQuadruple)
    (* Liste ist eine Folge von Tripeln, die mit NIL endet.
       Die Liste handhabt die Flle leere Liste, erstes u. letztes Element
       automatisch.
       Listenzeiger:
         Links:  Next-Zeiger, z. B. nchste Liste.
         Rechts: Die Knoten- oder Triple-Liste (Attribute)
         Rueck:  Liste, in der die Liste ein Element ist
         Verweis: Der aktuelle Knoten (Triple) = ListenCursor
       Knotenzeiger:
         Links: Der nchste Knoten oder NIL (bei einem Ring statt NIL
                wieder der erste Knoten)
         Rechts: Das Objekt oder NIL
         Rueck: Der vorige Knoten oder der Listenkopf
    *)
		ElementIndex:	LONGINT;
		AnzahlElemente: LONGINT;

		CONSTRUCTOR Init (CONST xflags: einFlagSet);
                {DESTRUCTOR Done; VIRTUAL;}
                FUNCTION erzeugeMich (CONST xflags: einFlagSet): GLP; VIRTUAL;

                FUNCTION isSubclass (CONST class: einGenTyp): BOOLEAN; VIRTUAL;
                FUNCTION isTerminal: BOOLEAN; VIRTUAL;
                FUNCTION meineAttribute: GLP; VIRTUAL;

                FUNCTION StringVonDaten: STRING; VIRTUAL;

    (********** Eigentliche Listenverwaltende Methoden ***********)

		FUNCTION IndexIst: LONGINT; VIRTUAL;
		FUNCTION LaengeIst: LONGINT; VIRTUAL;
                FUNCTION CursorIst: GLP; VIRTUAL;
                PROCEDURE ResetListe; VIRTUAL;
                PROCEDURE SetAlias (CONST Liste: GLP); VIRTUAL;
                PROCEDURE LoeschListe; VIRTUAL;

		FUNCTION get (CONST Element: GLP): GLP; VIRTUAL;
		  (* Unquote, dh. liefert das Objekt an dem Knoten *)
                FUNCTION getActual: GLP; VIRTUAL;
                  (* Ergibt das Objekt unter dem ListenCursor *)
                FUNCTION TypeOfActual: einGenTyp; VIRTUAL;
                FUNCTION isActualSubclass (CONST class: einGenTyp): BOOLEAN;
                  VIRTUAL;
		PROCEDURE put (CONST Element, Objekt: GLP); VIRTUAL;
		  (* Weist dem Knoten Element das Objekt zu. Beispiel:
		     put (Liste1.FirstElement, Liste2.NtesElement (n));
		     Weist dem ersten Element der Liste 1 den Inhalt des
		     Nten Elementes der Liste 2 zu.
		  *)
                PROCEDURE putActual (CONST Objekt: GLP); VIRTUAL;
                  (* Setzt das Objekt unter dem ListenCursor *)
                FUNCTION findNextObjekt (CONST Objekt: GLP): GLP;
                  VIRTUAL;
                  (* Durchsucht die Liste nach einem Knoten, der auf Objekt
                     deutet und stellt aktuelles Objekt und Index darauf
                     ein. Startet beim aktuellen Objekt und gibt NIL oder
                     den Knoten zurck, an dem das Objekt hngt. *)
                FUNCTION findPrevObjekt (CONST Objekt: GLP): GLP;
                  VIRTUAL;
                  (* Durchsucht die Liste nach einem Knoten, der auf Objekt
                     deutet und stellt aktuelles Objekt und Index darauf
                     ein. Startet beim aktuellen Objekt rckwrts *)

	      (* Die folgenden Operationen etablieren einen Knoten/Triple und
		 setzen seinen Rechts-Pointer auf das bergebene Objekt. Es
                 wird der Pointer auf den Knoten zurckgegeben und das
                 aktuelle Element eingestellt! *)
		FUNCTION insertFirstElement (CONST Objekt: GLP): GLP;
                  VIRTUAL;
		  (* Fgt am Anfang der Liste ein Element ein und gibt
                     Pointer auf den Knoten zurck, an den Objekt angehngt
                     wird *)
		FUNCTION insertBefore (CONST Objekt: GLP): GLP;
                  VIRTUAL;
		  (* Fgt vor dem aktuellen Element ein neues ein *)
		FUNCTION insertAfter (CONST Objekt: GLP): GLP;
                  VIRTUAL;
		  (* Fgt nach dem aktuellen Element ein Element ein *)
		FUNCTION appendElement (CONST Objekt: GLP): GLP;
                  VIRTUAL;
		  (* Hngt ein Element am Ende der Liste an, bei einem
		     Ring vor dem aktuellen Element *)

               (* Elemente in Mengen setzen/lschen/toggeln *)
                FUNCTION addElement (CONST Objekt: GLP): GLP; VIRTUAL;
                  (* Hngt Element an die Liste an, wenn es noch nicht
                     enthalten ist, liefert sonst vorhandenes Element *)
                FUNCTION subElement (CONST Objekt: GLP): GLP; VIRTUAL;
                  (* Entfernt Element aus der Liste und gibt es zurck,
                     NIL, falls es nicht enthalten ist *)
                FUNCTION toggleElement (CONST Objekt: GLP): GLP;
                  VIRTUAL;
                  (* Funktioniert, je nachdem, ob das Element enthalten
                     ist, wie Add oder Sub *)

	      (* Die folgenden Operationen liefern den Knoten, nicht das
		 Objekt selbst. Um dieses zu erhalten, verwende z. B.:
		 get (FirstElement);
	      *)
		FUNCTION actualElement: GLP; VIRTUAL;
                (* liefert aktuelles Element *)
                (* ... und diese haben einen Seiteneffekt auf den Listencursor
                   und ElementIndex: *)
		FUNCTION firstElement: GLP; VIRTUAL; (* impliziert Reset *)
		FUNCTION nextElement: GLP; VIRTUAL; (* verstellt aktuelles Element *)
		FUNCTION prevElement: GLP; VIRTUAL; (* verstellt aktuelles Element *)
		FUNCTION NtesElement (CONST N: LONGINT): GLP; VIRTUAL;
			(* stellt aktuelles auf das nte Element ein *)
                FUNCTION NOrLastElement (CONST N: LONGINT): GLP;
                  VIRTUAL;
		FUNCTION lastElement: GLP; VIRTUAL; (* verstellt aktuelles Element *)

                (* ... diese haben keinen Seiteneffekt: *)
		FUNCTION firstElementIs: GLP; VIRTUAL;
		FUNCTION nextElementIs: GLP; VIRTUAL;
		FUNCTION prevElementIs: GLP; VIRTUAL;
		FUNCTION ntesElementIs (CONST N: LONGINT): GLP;
                  VIRTUAL;
                FUNCTION NOrLastElementIs (CONST N: LONGINT): GLP;
                  VIRTUAL;
		FUNCTION lastElementIs: GLP; VIRTUAL;
                FUNCTION hatNextElement: BOOLEAN; VIRTUAL;
                FUNCTION hatPrevElement: BOOLEAN; VIRTUAL;
		FUNCTION findElement (CONST Objekt: GLP): GLP; VIRTUAL;
		  (* Durchsucht die Liste nach dem Objekt und liefert
		     seinen Knoten oder NIL *)
                FUNCTION IndexVonObjekt (CONST Objekt: GLP): LONGINT;
                  VIRTUAL;
                  (* Durchsucht die Liste nach dem Objekt und liefert
                     seinen Index *)

		FUNCTION DeleteFirst: GLP; VIRTUAL;
		  (* Liefert das erste Element und entfernt es aus der Liste *)
		FUNCTION DeleteLast: GLP; VIRTUAL;
		  (* Liefert das letzte Element und entfernt es *)
		FUNCTION Delete: GLP; VIRTUAL;
		  (* Liefert das aktuelle Element und entfernt es *)
		FUNCTION DeleteNext: GLP; VIRTUAL;
		  (* Liefert das nchste Element und entfernt es *)

	      (* Die folgenden Operationen arbeiten wie in LISP: *)
		FUNCTION Rest: GLP; VIRTUAL;
		  (* Liefert zweiten Listenknoten oder NIL, ohne aktuelles
		     Element zu beeinflussen. *)

		PROCEDURE SetObject (CONST ziel, quelle: GLP); VIRTUAL;
		  (* setzt nicht die Pointer gleich, sondern den Inhalt,
		     dh. kopiert die Verweispointer, also wie Set *)
                PROCEDURE sende (CONST view: PBuffText; VAR Ereignis: TEvent;
                                 CONST Sig: LONGINT);
                   VIRTUAL;
		PROCEDURE Monitor (CONST view: PBuffText); VIRTUAL;

                FUNCTION neuesElement (CONST Objekt: GLP): GLP; VIRTUAL;

	      END (* eineListe *);

    einTabellenPointer = ^eineTabelle;
    eineTabelle =  OBJECT (eineListe)
                     StartBegr, PreSelectBegr: GLP;
                     NStart, NPreSelect:       LONGINT;
                     Abmess:                   TRect;
                     Spaltenbreite:            INTEGER;
                     FenOffset:                TPoint;
                     CONSTRUCTOR Init (CONST xflags: einFlagSet);
                     FUNCTION erzeugeMich (CONST xflags: einFlagSet): GLP; VIRTUAL;
                     FUNCTION isSubclass (CONST class: einGenTyp): BOOLEAN; VIRTUAL;

                     PROCEDURE setAbmess (CONST a: TRect; CONST spbreite: INTEGER);
                       VIRTUAL;
                     PROCEDURE getAbmess (VAR a: TRect; VAR spbreite: INTEGER);
                       VIRTUAL;
                     PROCEDURE setOffset (CONST o: TPoint); VIRTUAL;
                     PROCEDURE getOffset (VAR o: TPoint); VIRTUAL;
                     PROCEDURE setPos (CONST p: TPoint); VIRTUAL;
                     PROCEDURE getPos (VAR p: TPoint); VIRTUAL;
                     PROCEDURE Monitor (CONST view: PBuffText); VIRTUAL;

                     FUNCTION neuesElement (CONST Objekt: GLP): GLP; VIRTUAL;
                   END (* eineTabelle *);
IMPLEMENTATION

USES App, EditStuff, CRT, GLFStructures, GLFNodes, {GLFWordBase,
     GLFFormulare,} GLFWindows;

(***************************** Methoden zu eineListe *******************)

CONSTRUCTOR eineListe.Init;
BEGIN
  INHERITED Init (xflags);
  MeinTyp := Liste;
  {ElementIndex := 0; AnzahlElemente := 0;}
END;

FUNCTION eineListe.erzeugeMich (CONST xflags: einFlagSet): GLP;
BEGIN
  erzeugeMich := NEW (einListenPointer, Init (xflags));
END (* erzeugeMich *);

FUNCTION eineListe.neuesElement (CONST Objekt: GLP): GLP;
VAR  neu: GLP;
BEGIN
  neu := neuesObjekt (Triple, []);
  einTriplePointer (neu)^.Rechts := Objekt;
  neuesElement := neu;
END (* neuesElement *);

FUNCTION eineListe.isSubclass (CONST class: einGenTyp): BOOLEAN;
BEGIN
  isSubclass := (class = Liste) OR INHERITED isSubClass (class);
END (* isSubclass *);

FUNCTION eineListe.isTerminal: BOOLEAN;
BEGIN
  isTerminal := FALSE;
END (* isTerminals *);

(********** Operationen zur Abfrage u. zum Setzen von Attributen *****)

FUNCTION eineListe.meineAttribute: GLP;
BEGIN
  meineAttribute := Rechts;
END;

FUNCTION eineListe.StringVonDaten: STRING;
VAR s, s1:  STRING;
    p:      einKnotenPointer;
BEGIN
  s := '[';
  GLP (p) := Rechts;
  WHILE (p <> NIL) DO BEGIN
    IF p^.Rechts <> NIL THEN BEGIN
      s1 := p^.Rechts^.StringVonDaten;
      IF p^.Links <> NIL THEN s1 := s1 + ', ';
      IF LENGTH (s) + LENGTH (s1) < 75
      (* angenommene Bildschirmbreite oder willkrliche Beschrnkung *) THEN
        s := s + s1
      ELSE BEGIN
        s := s + copy (s1, 1, 75 - LENGTH (s) - LENGTH (s1));
        p := NIL;
      END;
    END;
    IF p <> NIL THEN GLP (p) := p^.Links;
  END (* WHILE *);
  s := s + ']';
  StringVonDaten := s;
END (* StringVonDaten *);

FUNCTION eineListe.IndexIst: LONGINT;
BEGIN
  IndexIst := ElementIndex;
END (* IndexIst *);

FUNCTION eineListe.CursorIst: GLP;
BEGIN
  CursorIst := Verweis;
END (* CursorIst *);

FUNCTION eineListe.LaengeIst: LONGINT;
BEGIN
(* zeigMeldung (StringVonTyp (meinTyp), 'Liste.LaengeIst'); *)
  LaengeIst := AnzahlElemente;
END (* LaengeIst *);

PROCEDURE eineListe.ResetListe;
BEGIN
  Verweis := Rechts; ElementIndex := 0;
END (* ResetListe *);

PROCEDURE eineListe.SetAlias (CONST Liste: GLP);
(* Setze das Listenobjekt als Alias (z. B. fr Pointer-Zwecke) der
   bergebenen Liste gleich. Dies scheint mir jedoch auf Dauer eine
   fragwrdige Lsung, da durch Operationen auf einem Alias die Inte-
   gritt der ersten Liste zerstrt werden kann. *)
BEGIN
  IF (Liste <> NIL) AND Liste^.isSubclass (GLFTypes.Liste) THEN BEGIN
    INHERITED SetAlias (Liste);
    AnzahlElemente := einListenPointer (Liste)^.AnzahlElemente;
    ElementIndex := einListenPointer (Liste)^.ElementIndex;
  END;
END (* SetAlias *);

PROCEDURE eineListe.LoeschListe;
VAR  elem:  einTriplePointer;
BEGIN
  GLP (elem) := Rechts;
  WHILE elem <> NIL DO BEGIN
    elem^.Rechts := NIL;
    GLP (elem) := elem^.Links;
  END (* WHILE *);
END (* LoeschListe *);

FUNCTION eineListe.get (CONST Element: GLP): GLP;
(* Unquote, dh. liefert das Objekt an dem Knoten. Diese und die folgende
   Methode prfen ggw. nicht, ob das Element berhaupt zu dieser Liste
   gehrt und beeinflussen daher auch das aktuelle Element nicht. *)
BEGIN
  IF (Element <> NIL) AND (Element^.isSubclass (Knoten)) THEN BEGIN
    get := einKnotenPointer (Element)^.Rechts;
  END
  ELSE get := NIL;
END (* get *);

FUNCTION eineListe.getActual: GLP;
(* Effizienter Zugriff auf das aktuelle Objekt, dh. unter dem Cursor *)
BEGIN
  IF (Verweis <> NIL) AND (Verweis^.isSubclass (Knoten)) THEN BEGIN
    getActual := einKnotenPointer (Verweis)^.Rechts;
  END
  ELSE getActual := NIL;
END (* getActual *);

FUNCTION eineListe.TypeOfActual: einGenTyp;
BEGIN
  IF (Verweis <> NIL) AND (einTriplePointer (Verweis)^.Rechts <> NIL)
  THEN BEGIN
    TypeOfActual := einTriplePointer (Verweis)^.Rechts^.MeinTypIst;
  END
  ELSE TypeOfActual := FehlerTypX;
END (* TypeOfActual *);

FUNCTION eineListe.isActualSubclass (CONST class: einGenTyp): BOOLEAN;
BEGIN
  IF (Verweis <> NIL) AND (einTriplePointer (Verweis)^.Rechts <> NIL)
  THEN BEGIN
    isActualSubclass := einTriplePointer (Verweis)^.Rechts^.isSubclass (class);
  END
  ELSE isActualSubclass := FALSE;
END (* ActualMember *);

PROCEDURE eineListe.put (CONST Element, Objekt: GLP);
(* Weist dem Knoten Element das Objekt zu. Beispiel:
   put (Liste1.FirstElement, Liste2.NtesElement (n));
   Weist dem ersten Element der Liste 1 den Inhalt des
   Nten Elementes der Liste 2 zu.
*)
BEGIN
  IF (Element <> NIL) AND (Element^.isSubclass (Knoten)) THEN BEGIN
    einKnotenPointer (Element)^.Rechts := Objekt;
  END;
END (* put *);

PROCEDURE eineListe.putActual (CONST Objekt: GLP);
(* Effizientes Setzen des Objektes unter dem ListenCursor *)
BEGIN
  IF (Verweis <> NIL) AND (Verweis^.isSubclass (Knoten)) THEN BEGIN
    einKnotenPointer (Verweis)^.Rechts := Objekt;
  END;
END (* putActual *);

(************** Operationen zur Suche eines Elementes *************)

FUNCTION eineListe.hatNextElement: BOOLEAN;
BEGIN
  hatNextElement := (Verweis <> NIL) AND
                    (einTriplePointer (Verweis)^.Links <> NIL);
END (* hatNextElement *);

FUNCTION eineListe.hatPrevElement: BOOLEAN;
BEGIN
  hatPrevElement := (Verweis <> NIL) AND (einTriplePointer (Verweis)^.Rueck <> @SELF)
END (* hatPrevElement *);

FUNCTION eineListe.FindElement (CONST Objekt: GLP): GLP;
(* Durchsucht die Liste nach dem Objekt und liefert
   seinen Knoten oder NIL. Nur wenn das Objekt gefunden wurde,
   wird das aktuelle Element verstellt. *)
VAR   h:   GLP;
      ind: LONGINT;
BEGIN
  h := Rechts; ind := 0;
  WHILE (h <> NIL) AND (h^.isSubclass (Knoten)) AND
        (einTriplePointer (h)^.Rechts <> Objekt) DO BEGIN
    h := einKnotenPointer (h)^.Links; INC (ind);
  END (* WHILE *);
  IF (h <> NIL) AND (h^.isSubclass (Knoten)) AND
     (einKnotenPointer (h)^.Rechts = Objekt) THEN BEGIN
    Verweis := h; ElementIndex := ind;
  END
  ELSE h := NIL;
  FindElement := h;
END (* FindElement *);

FUNCTION eineListe.IndexVonObjekt (CONST Objekt: GLP): LONGINT;
VAR  element:     einListGluePointer;
     lokalIndex:  LONGINT;
BEGIN
  GLP (element) := FirstElementIs; lokalIndex := 0;
                                          (* Kann doch nicht richtig sein! *)
  WHILE (element <> NIL) AND (GLP(element){^.getDataPointer} <> Objekt) DO BEGIN
(*
zeigMeldung (StringVonTyp + '.IndexVonObjekt:' + atString (Objekt),
'lokalIndex: ' + StringVonNum (lokalIndex) + atString (element{^.getDataPointer}));
*)
    GLP (element) := einTriplePointer (element)^.Links;
    INC (lokalIndex);
  END (* WHILE *);
  IF element <> NIL THEN IndexVonObjekt := lokalIndex
  ELSE IndexVonObjekt := 0; (* unbefriedigend - sollte -1 sein!!! *)
END (* IndexVonObjekt *);

FUNCTION eineListe.FindNextObjekt (CONST Objekt: GLP): GLP;
(* Durchsucht die Liste ab dem aktuellen Objekt nach einem Knoten,
   der auf Objekt deutet. Wre um FindFirst, FindBack, FindNth zu
   ergnzen, auerdem sollte die Vergleichsmethode whlbar sein.
   Diese und die folgende Methode mssen wegen der End-Bedingung fr
   die Suche in einRing berschrieben werden! *)
VAR h:   GLP;
    ind: LONGINT;
BEGIN
  h := Verweis; ind := ElementIndex;
  WHILE (h <> NIL) AND (h^.isSubclass (Knoten)) AND
        (einKnotenPointer (h)^.Rechts <> Objekt) DO BEGIN
    h := einKnotenPointer (h)^.Links; INC (ind);
  END (* WHILE *);
  IF (h <> NIL) AND (h^.isSubclass (Knoten)) AND
     (einKnotenPointer (h)^.Rechts = Objekt) THEN BEGIN
    Verweis := h; ElementIndex := ind;
  END (* IF *)
  ELSE h := NIL;
  FindNextObjekt := h;
END (* FindNextObjekt *);

FUNCTION eineListe.FindPrevObjekt (CONST Objekt: GLP): GLP;
(* Durchsucht die Liste ab dem aktuellen Objekt nach einem Knoten,
   der auf Objekt deutet. Wre um FindFirst, FindBack, FindNth zu
   ergnzen, auerdem sollte die Vergleichsmethode whlbar sein. *)
VAR h:   GLP;
    ind: LONGINT;
BEGIN
  h := Verweis; ind := ElementIndex;
  WHILE (h <> @SELF) AND (h^.isSubclass (Triple)) AND
        (einTriplePointer (h)^.Rechts <> Objekt) DO BEGIN
    h := einTriplePointer (h)^.Rueck; INC (ind);
  END (* WHILE *);
  IF (h <> @SELF) AND (h^.isSubclass (Knoten)) AND
     (einKnotenPointer (h)^.Rechts = Objekt) THEN BEGIN
    Verweis := h; ElementIndex := ind;
  END (* IF *)
  ELSE h := NIL;
  FindPrevObjekt := h;
END (* FindPrevObjekt *);

(************** Operationen, die Elemente in die Liste einfgen **********)

(* Die folgenden Operationen etablieren einen Knoten/Triple und
   setzen seinen Rechts-Pointer auf das bergebene Objekt *)

FUNCTION eineListe.InsertFirstElement (CONST Objekt: GLP): GLP;
		  (* Fgt am Anfang der Liste ein Element ein *)
VAR   h:    GLP;
BEGIN
  h := Rechts;
  Rechts := neuesElement (Objekt);
  einTriplePointer (Rechts)^.Links := h;
  einTriplePointer (Rechts)^.Rueck := @SELF;
  IF h <> NIL THEN
    einTriplePointer (h)^.Rueck := Rechts;
  Verweis := Rechts;
  INC (AnzahlElemente); ElementIndex := 0;
  InsertFirstElement := Rechts;
END (* InsertFirstElement *);

FUNCTION eineListe.InsertBefore (CONST Objekt: GLP): GLP;
(* Fgt vor dem aktuellen Element ein neues ein *)
VAR   h:    GLP;
BEGIN
  (* Es gibt noch kein Element *) (* aktuell ist das erste Element *)
  IF (Verweis = NIL) OR (einTriplePointer (Verweis)^.Rueck = @SELF) THEN BEGIN
    InsertBefore := InsertFirstElement (Objekt);
  END
  ELSE BEGIN
    (* Es gibt ein Element _VOR_ dem aktuellen und ListCursor deutet auf
       das aktuelle Triple *)
    h := neuesElement (Objekt);
    einTriplePointer (Verweis)^.InsertRueckLinks (h); Verweis := h;
    INC (AnzahlElemente);
    InsertBefore := h;
  END (* ELSE *);
END (* InsertBefore *);

FUNCTION eineListe.InsertAfter (CONST Objekt: GLP): GLP;
(* Fgt nach dem aktuellen Element ein Element ein neues ein. *)
VAR   h:    GLP;
BEGIN
  IF (Verweis = NIL) THEN BEGIN
    InsertAfter := InsertFirstElement (Objekt);
  END
  ELSE BEGIN
    h := neuesElement (Objekt);
    einTriplePointer (Verweis)^.InsertLinks (h); Verweis := h;
    INC (AnzahlElemente); INC (ElementIndex);
    InsertAfter := h;
  END;
END (* InsertAfter *);

FUNCTION eineListe.AppendElement (CONST Objekt: GLP): GLP;
(* Hngt ein Element am Ende der Liste an, bei einem
   Ring vor dem aktuellen Element *)
VAR   h:    GLP;
BEGIN
  IF (Verweis = NIL) THEN AppendElement := InsertFirstElement (Objekt)
  ELSE BEGIN
    Verweis := LastElement;
    AppendElement := InsertAfter (Objekt);
  END;
END (* AppendElement *);

FUNCTION eineListe.AddElement (CONST Objekt: GLP): GLP;
(* Fgt ein Element in die Liste ein, falls es noch nicht vorhanen ist,
   gibt sonst das vorhandene zurck *)
VAR  elem: GLP;
BEGIN
  elem := FindElement (Objekt);
  IF elem = NIL THEN
    AddElement := AppendElement (Objekt)
  ELSE AddElement := NIL;
END (* AddElement *);

FUNCTION eineListe.SubElement (CONST Objekt: GLP): GLP;
(* Entfernt ein Element aus der Liste und gibt es zurck, oder
   NIL, wenn es nicht vorhanden ist *)
VAR  elem: GLP;
BEGIN
  elem := FindElement (Objekt);
  IF elem <> NIL THEN
    SubElement := Delete
  ELSE SubElement := NIL;
END (* SubElement *);

FUNCTION eineListe.ToggleElement (CONST Objekt: GLP): GLP;
(* Funktioniert, je nachdem, ob das Element enthalten ist,
   wie Add oder Sub *)
VAR  elem: GLP;
BEGIN
  elem := FindElement (Objekt);
  IF elem = NIL THEN BEGIN
    (* zeigMeldung ('Toggle', 'Append'); *)
    ToggleElement := AppendElement (Objekt)
  END
  ELSE BEGIN
    (* zeigMeldung ('Toggle', 'Delete'); *)
    ToggleElement := Delete;
  END;
END (* ToggleElement *);


(************* Operationen zum Verstellen des ListCursors **********)

(* Die folgenden Operationen liefern den Knoten, nicht das
   Objekt selbst. Um dieses zu erhalten, verwende z. B.:
   get (FirstElement);
*)
FUNCTION eineListe.ActualElement: GLP; (* liefert aktuelles Element *)
BEGIN
  ActualElement := Verweis;
END;

(* Mit Seiteneffekt: *)

FUNCTION eineListe.FirstElement: GLP; (* impliziert Reset *)
BEGIN
  Verweis := Rechts;
  ElementIndex := 0;
  FirstElement := Verweis;
END (* FirstElement *);

FUNCTION eineListe.NextElement: GLP; (* verstellt aktuelles Element *)
BEGIN
     (* Es gibt berhaupt ein Element und es gibt ein nchstes El. *)
{
zeigMeldung ('Liste: ' + StringVonTyp + ', Anzahl: ' + StringVonNum (AnzahlElemente),
'NextElement Index: ' + StringVonNum (ElementIndex));
}
  IF (Verweis <> NIL) AND (einTriplePointer (Verweis)^.Links <> NIL) THEN BEGIN
    Verweis := einTriplePointer (Verweis)^.Links;
    INC (ElementIndex);
    NextElement := Verweis;
  END
  ELSE NextElement := NIL;
END (* NextElement *);

FUNCTION eineListe.PrevElement: GLP; (* verstellt aktuelles Element *)
BEGIN
  IF (Verweis <> NIL) AND (einTriplePointer (Verweis)^.Rueck <> @SELF) THEN BEGIN
    Verweis := einTriplePointer (Verweis)^.Rueck;
    DEC (ElementIndex);
    PrevElement := Verweis;
  END
  ELSE PrevElement := NIL;
END (* PrevElement *);

FUNCTION eineListe.NtesElement (CONST N: LONGINT): GLP;
(* stellt aktuelles auf das nte Element ein, NIL wenn dieses nicht
   existiert. *)
VAR   h:   GLP;
      ind: LONGINT;
BEGIN
  IF N = 0 THEN BEGIN
    Verweis := Rechts;
    ElementIndex := 0;
    NtesElement := FirstElement;
  END
  ELSE
  IF (Verweis <> NIL) AND (N < AnzahlElemente) THEN BEGIN
    ind := ElementIndex; h := Verweis;
    WHILE (ind < N) AND (h <> NIL) DO BEGIN
      h := einTriplePointer (h)^.Links; INC (ind);
    END (* WHILE *);
    WHILE (ind > N) AND (einTriplePointer (h)^.Rueck <> @SELF) DO BEGIN
      h := einTriplePointer (h)^.Rueck; DEC (ind);
    END (* WHILE *);
    IF h <> NIL THEN BEGIN
      Verweis := h; ElementIndex := ind;
    END (* IF *);
    NtesElement := h;
  END
  ELSE NtesElement := NIL;
{
zeigMeldung (StringVonTyp (meinTyp) + '.NtesElement',
             'ElementIndex: ' + StringVonNum (ElementIndex) +
             '; Verweis: ' + atString (Verweis));
}
END (* NtesElement *);

FUNCTION eineListe.NOrLastElement (CONST N: LONGINT): GLP;
(* stellt aktuelles auf das Nte Element oder das letzte ein *)
BEGIN
  IF (N < AnzahlElemente) THEN NOrLastElement := NtesElement (N)
  ELSE NOrLastElement := LastElement;
{
zeigMeldung (StringVonTyp (meinTyp) + '.NOrLastElement',
             'ElementIndex: ' + StringVonNum (ElementIndex) +
             '; Verweis: ' + atString (Verweis));
}
END (* NOrLastElement *);

FUNCTION eineListe.LastElement: GLP; (* verstellt aktuelles Element *)
BEGIN
  IF Verweis <> NIL THEN BEGIN
    WHILE einTriplePointer (Verweis)^.Links <> NIL DO BEGIN
      Verweis := einTriplePointer (Verweis)^.Links; INC (ElementIndex);
    END (* WHILE *);
  END (* IF *);
  LastElement := Verweis;
END (* LastElement *);

(* Ohne Seiteneffekt: *)

FUNCTION eineListe.FirstElementIs: GLP;
BEGIN
  FirstElementIs := Rechts;
END (* FirstElementIs *);

FUNCTION eineListe.NextElementIs: GLP;
BEGIN
     (* Es gibt berhaupt ein Element und es gibt ein nchstes El. *)
  IF (Verweis <> NIL) AND (einTriplePointer (Verweis)^.Links <> NIL) THEN BEGIN
    NextElementIs := einTriplePointer (Verweis)^.Links;
  END
  ELSE NextElementIs := NIL;
END (* NextElement *);

FUNCTION eineListe.PrevElementIs: GLP;
BEGIN
  IF (Verweis <> NIL) AND (einTriplePointer (Verweis)^.Rueck <> @SELF) THEN BEGIN
    PrevElementIs := einTriplePointer (Verweis)^.Rueck;
  END
  ELSE PrevElementIs := NIL;
END (* PrevElementIs *);

FUNCTION eineListe.NtesElementIs (CONST N: LONGINT): GLP;
VAR   h:   GLP;
      ind: LONGINT;
BEGIN
  h := NIL;
  IF N = 0 THEN BEGIN
    h := Rechts;
  END
  ELSE
  IF (Verweis <> NIL) AND (N < AnzahlElemente) THEN BEGIN
    ind := ElementIndex; h := Verweis;
    WHILE (ind < N) AND (h <> NIL) DO BEGIN
      h := einTriplePointer (h)^.Links; INC (ind);
    END (* WHILE *);
    WHILE (ind > N) AND (einTriplePointer (h)^.Rueck <> @SELF) DO BEGIN
      h := einTriplePointer (h)^.Rueck; DEC (ind);
    END (* WHILE *);
  END;
{
zeigMeldung (StringVonTyp (meinTyp) + '.NtesElementIs',
             'Index: ' + StringVonNum (N) +
             '; Pointer: ' + atString (h));
}
  NtesElementIs := h;
END (* NtesElementIs *);

FUNCTION eineListe.NOrLastElementIs (CONST N: LONGINT): GLP;
(* ergibt das Nte Element oder das letzte *)
BEGIN
  IF (N < AnzahlElemente) THEN NOrLastElementIs := NtesElementIs (N)
  ELSE NOrLastElementIs := LastElementIs;
END (* NOrLastElement *);

FUNCTION eineListe.LastElementIs: GLP;
VAR  h: GLP;
BEGIN
  IF Verweis <> NIL THEN BEGIN
    h := Verweis;
    WHILE einTriplePointer (h)^.Links <> NIL DO BEGIN
      h := einTriplePointer (h)^.Links;
    END (* WHILE *);
  END (* IF *);
  LastElementIs := h;
END (* LastElementIs *);

(********** Operationen, die Elemente lschen *******)

FUNCTION eineListe.DeleteFirst: GLP;
(* Liefert das erste Element und entfernt es aus der Liste. *)
VAR  h:  GLP;
BEGIN
  IF Rechts <> NIL THEN BEGIN
    h := Rechts;
    IF Verweis = Rechts THEN Verweis := einTriplePointer (h)^.Links;
    DEC (AnzahlElemente);
    Rechts := einTriplePointer (h)^.Links;
    IF Rechts <> NIL THEN BEGIN
      einTriplePointer (Rechts)^.Rueck := @SELF;
    END;
    DeleteFirst := h;
  END
  ELSE DeleteFirst := NIL;
END (* DeleteFirst *);

FUNCTION eineListe.DeleteLast: GLP;
(* Liefert das letzte Element und entfernt es *)
VAR  h:  GLP;
BEGIN
  IF Rechts <> NIL THEN BEGIN
    IF AnzahlElemente > 1 THEN BEGIN
      h := Verweis; (* Chance, da Verweis mehr oder weniger auf das
                          Listenende deutet *)
      WHILE (einTriplePointer (h)^.Links <> NIL) DO BEGIN
        h := einTriplePointer (h)^.Links;
      END;
      IF (einTriplePointer (h)^.Links = NIL) THEN BEGIN
        (* ListCursor stand auf dem letzten Element, das entfernt wird *)
        h := einTriplePointer (h)^.Rueck;
        Verweis := h; DEC (ElementIndex);
      END;
      DEC (AnzahlElemente);
      DeleteLast := einTriplePointer (h)^.DeleteLinks;
    END
    ELSE BEGIN
      Verweis := NIL; AnzahlElemente := 0; ElementIndex := 0;
      DeleteLast := Rechts; Rechts := NIL;
    END (* IF AnzahlElemente *);
  END
  ELSE DeleteLast := NIL;
END (* DeleteLast *);

FUNCTION eineListe.Delete: GLP;
(* Liefert das aktuelle Element und entfernt es *)
VAR  h:  GLP;
BEGIN
  IF Rechts <> NIL THEN BEGIN
    IF AnzahlElemente > 1 THEN BEGIN
      DEC (AnzahlElemente);
      (* Der Verweispointer ist der Cursor auf das aktuelle Element *)
      IF einTriplePointer (Verweis)^.Rueck = @SELF THEN BEGIN
        (* Das aktuelle ist das ersten Element *)
        Verweis := einTriplePointer (Verweis)^.Links;
        (* Verweis deutet jetzt auf das nchste Element *)
        Delete := Rechts;
        Rechts := Verweis;
        einTriplePointer (Rechts)^.Rueck := @SELF;
      END
      ELSE BEGIN
        Verweis := einTriplePointer (Verweis)^.Rueck;
        (* Das aktuelle Element ist jetzt das vorige *)
        Delete := einTriplePointer (Verweis)^.DeleteLinks;
        IF einTriplePointer (Verweis)^.Links <> NIL THEN
          Verweis := einTriplePointer (Verweis)^.Links
          (* Das aktuelle ist jetzt das Element nach dem ursprnglich
             aktuellen *)
        ELSE DEC (ElementIndex);
      END (* IF Rueck = @SELF *);
    END (* IF AnzahlElemente > 1 THEN *)
    ELSE BEGIN
      Verweis := NIL; AnzahlElemente := 0; ElementIndex := 0;
      Delete := Rechts; Rechts := NIL;
    END (* IF AnzahlElemente *);
  END
  ELSE Delete := NIL;
END (* Delete *);

FUNCTION eineListe.DeleteNext: GLP;
(* Liefert das nchste Element und entfernt es *)
VAR  h:  GLP;
BEGIN
     (* Es gibt berhaupt ein Element und es gibt ein nchstes El. *)
  IF (Verweis <> NIL) AND (einTriplePointer (Verweis)^.Links <> NIL) THEN BEGIN
    DeleteNext := einTriplePointer (Verweis)^.DeleteLinks;
    DEC (AnzahlElemente);
  END
  ELSE DeleteNext := NIL;
END (* DeleteNext *);


(************** Die folgenden Operationen arbeiten wie in LISP: *********)

FUNCTION eineListe.Rest: GLP;
(* Liefert zweiten Listenknoten oder NIL, ohne aktuelles
   Element zu beeinflussen. *)
BEGIN
  IF Verweis <> NIL THEN BEGIN
    Rest := einTriplePointer (Verweis)^.Links;
  END
  ELSE Rest := NIL;
END (* Rest *);


PROCEDURE eineListe.SetObject (CONST ziel, quelle: GLP);
(* setzt nicht die Pointer gleich, sondern den Inhalt,
   dh. kopiert die Verweispointer, also wie Set *)
BEGIN
  IF (ziel <> NIL) AND (quelle <> NIL) AND
    ziel^.isSubclass (Knoten) AND quelle^.isSubclass (Knoten) THEN BEGIN
      einKnotenPointer (ziel)^.Rechts := einKnotenPointer (quelle)^.Rechts;
  END;
END (* SetObject *);

PROCEDURE eineListe.sende (CONST view: PBuffText; VAR ereignis: TEvent;
                           CONST Sig: LONGINT);
BEGIN
  eineStruktur.sende (view, ereignis, Sig);
  IF (Rechts <> NIL) AND (Rechts^.meineSignatur <> Sig) AND (ereignis.what <> evNothing) THEN
    Rechts^.sende (view, ereignis, Sig);
  IF (Links <> NIL) AND (Links^.meineSignatur <> Sig) AND (ereignis.what <> evNothing) THEN
    Links^.sende (view, ereignis, Sig);
END (* sende *);

PROCEDURE eineListe.Monitor;
BEGIN
  INHERITED Monitor (view);
  view^.WriteString ('AnzahlElemente:   ', 0, 0); view^.WriteInt (AnzahlElemente);
    view^.WriteLn;
  view^.WriteString ('ElementIndex:     ', 0, 0); view^.WriteInt (ElementIndex);
    view^.WriteLn;
END;

(*************************** Methoden zu eineTabelle *******************)

CONSTRUCTOR eineTabelle.Init;
BEGIN
  INHERITED Init (xflags);
  MeinTyp := Tabelle;
  {NStart := 0; NPreSelect := 0;}
  StartBegr := NIL; PreSelectBegr := NIL;
  initAbmess (Abmess, Spaltenbreite);
  {FenOffset.X := 0; FenOffset.Y := 0;}
  {meinKnoten := NIL;}
END (* Init *);

FUNCTION eineTabelle.erzeugeMich (CONST xflags: einFlagSet): GLP;
BEGIN
  erzeugeMich := NEW (einTabellenPointer, Init (xflags));
END (* erzeugeMich *);

FUNCTION eineTabelle.isSubclass (CONST class: einGenTyp): BOOLEAN;
BEGIN
  isSubclass := (class = Tabelle) OR INHERITED isSubClass (class);
END (* Member *);

FUNCTION eineTabelle.neuesElement (CONST Objekt: GLP): GLP;
VAR  neu: GLP;
BEGIN
  neu := neuesObjekt (Glue, [setFormular]);
  einTriplePointer (neu)^.Rechts := Objekt;
  neuesElement := neu;
END (* NeuesElement *);

PROCEDURE eineTabelle.setAbmess (CONST a: TRect; CONST spbreite: INTEGER);
BEGIN
  Abmess := a; Spaltenbreite := spbreite;
END (* setAbmess *);

PROCEDURE eineTabelle.getAbmess (VAR a: TRect; VAR spbreite: INTEGER);
BEGIN
  a := Abmess; spbreite := Spaltenbreite;
END (* getAbmess *);

PROCEDURE eineTabelle.setPos (CONST p: TPoint);
VAR dummyp: POINTER;
BEGIN
  Abmess.Move (p.X - Abmess.A.X, p.Y - Abmess.A.Y);
  dummyp := Message (DeskTop, evBroadcast, cmGLFObjektUpdate, @SELF);
END (* setPos *);

PROCEDURE eineTabelle.getPos (VAR p: TPoint);
BEGIN
  p := Abmess.A;
END (* getPos *);

PROCEDURE eineTabelle.setOffset (CONST o: TPoint);
BEGIN
{zeigMeldung ('eineTabelle.setOffset',
'X: ' + StringVonNum (o.x) + ', Y: ' + StringVonNum (o.y));}
  FenOffset := o;
END (* setOffset *);

PROCEDURE eineTabelle.getOffset (VAR o: TPoint);
BEGIN
  o := FenOffset;
END (* getOffset *);

PROCEDURE eineTabelle.Monitor (CONST view: PBuffText);
BEGIN
  INHERITED Monitor (view);
  view^.WriteString ('StartBegr:        ', 0, 0); view^.WriteGLFPointer (StartBegr);
    view^.WriteLn;
  view^.WriteString ('PreSelectBegr:    ', 0, 0); view^.WriteGLFPointer (PreSelectBegr);
    view^.WriteLn;
  view^.WriteString ('NStart:           ', 0, 0); view^.WriteInt (NStart);
    view^.WriteLn;
  view^.WriteString ('NPreSelect:       ', 0, 0); view^.WriteInt (NPreSelect);
    view^.WriteLn;
  view^.WriteString ('Abmessungen:      ', 0, 0); view^.WriteRect (Abmess);
    view^.WriteLn;
  view^.WriteString ('FenOffset:        ', 0, 0); view^.WritePoint (FenOffset);
    view^.WriteLn;
END (* Monitor *);

END (* GLFLists *).