UNIT GLFNodes;

{$F+}
{$O+}

(* (C) 1990-1995 by Johannes Leckebusch
   Version: 0.902
   Anpassung an TP ab: 6. 12. 90
   Stand: 07. 02. 95
*)

INTERFACE

USES GLFTypes, GLFBase, GLFStructures, Drivers, Views;

(* Diese Unit implementiert binre Knoten (Knoten) und dreifache Knoten
   (Triple) sowie Operationen zum Manipulieren der Zeiger, Einfgen und
   Lschen von Knoten in Bume und Listen.
   Der rechte und linke Zeiger sind generell zum Aufbau von Binrbumen
   gedacht, der dritte Zeiger (Rueck) bei Tripeln fr eine Rckwrts-
   verkettung von Listen oder von Bumen in Richtung der Wurzel.

   Bei Listen gilt die Konvention, da der linke Zeiger auf den nchsten
   Knoten und der rechte Zeiger auf das gespeicherte Objekt deutet.
*)

TYPE

  einKnotenPointer = ^einKnoten;
  einKnoten = OBJECT (eineStruktur)
		Links:  GLP; (* nchster Knoten *)
		Rechts: GLP; (* Datenzeiger *)

		CONSTRUCTOR Init (xflags: einFlagSet);
                DESTRUCTOR Done; VIRTUAL;
                {}
                DESTRUCTOR DoneCluster; VIRTUAL;
                PROCEDURE LoeschCluster (CONST Sig: LONGINT); VIRTUAL;
                {}
                FUNCTION erzeugeMich (CONST xflags: einFlagSet): GLP; VIRTUAL;
                FUNCTION IsAbstract: BOOLEAN; VIRTUAL;
                FUNCTION isSubclass (CONST class: einGenTyp): BOOLEAN; VIRTUAL;
		PROCEDURE nulleNetzFlag (CONST flag: einFlag; CONST Sig: LONGINT);
                  VIRTUAL;
                PROCEDURE setzeNetzFlag (CONST flag: einFlag; CONST Sig: LONGINT);
                  VIRTUAL;
                PROCEDURE LoeschPfad (CONST Sig: LONGINT); VIRTUAL;
                PROCEDURE sende (CONST view: PBuffText; VAR ereignis: TEvent;
                                 CONST Sig: LONGINT);
                  VIRTUAL;

		PROCEDURE zeigFenster (CONST view: PBuffText; CONST x, y, b, h: WORD;
                                       CONST zeichneFen: BOOLEAN);
		  VIRTUAL;
		FUNCTION Edit (CONST view: PBuffText; CONST x, y, b, h: WORD;
                               CONST name: GLP): GLP;
		  VIRTUAL;

(* Diese Methoden, die ich schon mal in eineStruktur "aufgeschoben" habe,
   mssen doch wohl nicht wirklich virtuell sein?! *)

		PROCEDURE setLinks (CONST p: GLP); {VIRTUAL dSetLinks;}
		PROCEDURE setRechts (CONST p: GLP); {VIRTUAL dSetRechts;}
		PROCEDURE LinksNeu (CONST typ: einGenTyp; CONST xflags: einFlagSet);
                  {VIRTUAL dLinksNeu;}
		PROCEDURE RechtsNeu (CONST typ: einGenTyp; CONST xflags: einFlagSet);
                  {VIRTUAL dRechtsNeu;}
		PROCEDURE InsertLinks (CONST p: GLP); {VIRTUAL dInsertLinks;}
		PROCEDURE InsertRechts (CONST p: GLP); {VIRTUAL dInsertRechts;}
		FUNCTION DeleteLinks: GLP; {VIRTUAL dDeleteLinks;}
		FUNCTION DeleteRechts: GLP; {VIRTUAL dDeleteRechts;}
		FUNCTION LinksIst: GLP; {VIRTUAL dLinksIst;}
		FUNCTION RechtsIst: GLP; {VIRTUAL dRechtsIst;}
		FUNCTION LinksNter (VAR n: LONGINT): GLP; {VIRTUAL dLinksNter;}
		FUNCTION RechtsNter (VAR n: LONGINT): GLP; {VIRTUAL dRechtsNter;}
		FUNCTION AnzahlLinks: LONGINT; {VIRTUAL dAnzahlLinks;}
		FUNCTION AnzahlRechts: LONGINT; {VIRTUAL dAnzahlRechts;}
		FUNCTION LinksLetzter: GLP; {VIRTUAL dLinksLetzter;}
		FUNCTION RechtsLetzter: GLP; {VIRTUAL dRechtsLetzter;}

		PROCEDURE Monitor (CONST view: PBuffText); VIRTUAL;
	      END (* einKnoten *);

  einTriplePointer = ^einTriple;
  einTriple = OBJECT (einKnoten)
		Rueck:  GLP; (* Rckwrtsverzeigerung in Listen *)

		CONSTRUCTOR Init (CONST xflags: einFlagSet);
                {DESTRUCTOR Done; VIRTUAL;}
                FUNCTION erzeugeMich (CONST xflags: einFlagSet): GLP; VIRTUAL;
                FUNCTION isSubclass (CONST class: einGenTyp): BOOLEAN; VIRTUAL;

		PROCEDURE RueckNeu (CONST typ: einGenTyp; CONST xflags: einFlagSet);
                  {VIRTUAL;}
		PROCEDURE InsertRueckLinks (CONST p: GLP); {VIRTUAL;}
             	PROCEDURE InsertRueckRechts (CONST p: GLP); {VIRTUAL;}
		FUNCTION RueckIst: GLP; VIRTUAL; (* Das ist schon in GLFBase *)
		FUNCTION RueckNter (VAR n: LONGINT): GLP; {VIRTUAL;}
		FUNCTION AnzahlRueck: LONGINT; {VIRTUAL;}
		FUNCTION RueckLetzter: GLP; {VIRTUAL;}
		FUNCTION DeleteRueck: GLP; {VIRTUAL;}
		PROCEDURE setRueck (CONST p: GLP); {VIRTUAL;}

		PROCEDURE Monitor (CONST view: PBuffText); VIRTUAL;
	      END (* einTriple *);

IMPLEMENTATION

USES EditStuff, Objects;

(*************************** Methoden zu einKnoten *************************)

CONSTRUCTOR einKnoten.Init;
BEGIN
  GLF.Init (xflags);
  MeinTyp := Knoten;
  {
  Links := NIL;
  Rechts := NIL;
  }
END (* Init *);

FUNCTION einKnoten.isAbstract: BOOLEAN;
BEGIN
  isAbstract := FALSE;
END (* isAbstract *);

FUNCTION einKnoten.erzeugeMich (CONST xflags: einFlagSet): GLP;
BEGIN
  erzeugeMich := NEW (einKnotenPointer, Init (xflags));
END (* erzeugeMich *);

DESTRUCTOR einKnoten.Done;
BEGIN
  INHERITED Done;
  IF Links <> NIL THEN DISPOSE (Links, Done);
  IF Rechts <> NIL THEN DISPOSE (Rechts, Done);
END;

{}
DESTRUCTOR einKnoten.DoneCluster;
BEGIN
  INHERITED DoneCluster;
  IF Links <> NIL THEN DISPOSE (Links, DoneCluster);
  IF Rechts <> NIL THEN DISPOSE (Rechts, DoneCluster);
END;

PROCEDURE einKnoten.LoeschCluster;
BEGIN
  INHERITED LoeschCluster (Sig);

  IF Links <> NIL THEN BEGIN
    IF Sig = Links^.meineSignatur THEN Links := NIL
    ELSE Links^.LoeschCluster (Sig);
  END (* IF *);

  IF Rechts <> NIL THEN BEGIN
    IF Sig = Rechts^.meineSignatur THEN Rechts := NIL
    ELSE Rechts^.LoeschCluster (Sig);
  END;
END (* LoeschCluster *);
{}

FUNCTION einKnoten.isSubclass (CONST class: einGenTyp): BOOLEAN;
BEGIN
  isSubclass := (class = Knoten) OR INHERITED isSubClass (class);
END (* Member *);

PROCEDURE einKnoten.nulleNetzFlag (CONST flag: einFlag; CONST Sig: LONGINT);
BEGIN
  {IF (flag IN meineFlags) THEN BEGIN}
  IF Sig <> meineSignatur THEN BEGIN
    INHERITED nulleNetzFlag (flag, Sig);
    IF Links <> NIL THEN BEGIN
      Links^.nulleNetzFlag (flag, Sig);
    END;
    IF Rechts <> NIL THEN BEGIN
      Rechts^.nulleNetzFlag (flag, Sig);
    END;
  END (* IF *);
END (* nulleNetzFlag *);

PROCEDURE einKnoten.setzeNetzFlag (CONST flag: einFlag; CONST Sig: LONGINT);
BEGIN
  {IF NOT (flag IN meineFlags) THEN BEGIN}
  IF Sig <> meineSignatur THEN BEGIN
    INHERITED setzeNetzFlag (flag, Sig);
    IF Links <> NIL THEN BEGIN
      Links^.setzeNetzFlag (flag, Sig);
    END;
    IF Rechts <> NIL THEN BEGIN
      Rechts^.setzeNetzFlag (flag, Sig);
    END;
  END (* IF *);
END (* setzeNetzFlag *);

PROCEDURE einKnoten.LoeschPfad;
BEGIN
  INHERITED LoeschPfad (Sig);
  IF Links <> NIL THEN BEGIN
    IF Sig = Links^.meineSignatur THEN Links := NIL
    ELSE Links^.LoeschPfad (Sig);
  END (* IF *);
  IF Rechts <> NIL THEN BEGIN
    IF Sig = Rechts^.meineSignatur THEN Rechts := NIL
    ELSE Rechts^.LoeschPfad (Sig);
  END (* IF *);
END (* LoeschPfad *);

PROCEDURE einKnoten.sende (CONST view: PBuffText; VAR ereignis: TEvent;
                           CONST Sig: LONGINT);
BEGIN
  INHERITED sende (view, ereignis, Sig);
  IF (Links <> NIL) AND (Links^.meineSignatur <> Sig) AND (ereignis.what <> evNothing) THEN
    Links^.sende (view, ereignis, Sig);
  IF (Rechts <> NIL) AND (Rechts^.meineSignatur <> Sig) AND (ereignis.what <> evNothing) THEN
    Rechts^.sende (view, ereignis, Sig);
END (* sende *);

PROCEDURE einKnoten.zeigFenster (CONST view: PBuffText;
                                 CONST x, y, b, h: WORD; CONST zeichneFen: BOOLEAN);
BEGIN Abstract ('zeigFenster'); END;

FUNCTION einKnoten.Edit (CONST view: PBuffText; CONST x, y, b, h: WORD; CONST name: GLP): GLP;
BEGIN
  Edit := @SELF;
END;

PROCEDURE einKnoten.setLinks (CONST p: GLP);
BEGIN Links := p;
END;

PROCEDURE einKnoten.setRechts (CONST p: GLP);
BEGIN Rechts := p;
END;

PROCEDURE einKnoten.LinksNeu (CONST typ: einGenTyp; CONST xflags: einFlagSet);
BEGIN Links := neuesObjekt (typ, xflags);
END;

PROCEDURE einKnoten.RechtsNeu (CONST typ: einGenTyp; CONST xflags: einFlagSet);
BEGIN Rechts := neuesObjekt (typ, xflags);
END;

PROCEDURE einKnoten.InsertLinks (CONST p: GLP);
VAR     h:      GLP;
BEGIN
  IF p <> NIL THEN BEGIN
    h := Links; Links := p;
    IF p^.isSubclass (Knoten) THEN BEGIN
      einKnotenPointer (p)^.SetLinks (h);
      IF p^.isSubclass (Triple) THEN BEGIN
        einTriplePointer (p)^.SetRueck (@SELF);
      END (* IF Triple *);
    END (* IF *);
    IF h <> NIL THEN BEGIN
      IF h^.isSubclass (Triple) THEN BEGIN
        einTriplePointer (h)^.setRueck (p);
      END;
    END (* IF h <> NIL *);
  END (* IF p <> NIL *);
END (* einKnoten.InsertLinks *);

PROCEDURE einKnoten.InsertRechts (CONST p: GLP);
VAR     h:      GLP;
BEGIN
  IF p <> NIL THEN BEGIN
    h := Rechts; Rechts := p;
    IF p^.isSubclass (Knoten) THEN BEGIN
      einKnotenPointer (p)^.SetRechts (h);
      IF p^.isSubclass (Triple) THEN BEGIN
        einTriplePointer (p)^.SetRueck (@SELF);
      END (* IF Triple *);
    END (* IF *);
    IF h <> NIL THEN BEGIN
      IF h^.isSubclass (Triple) THEN BEGIN
        einTriplePointer (h)^.setRueck (p);
      END (* IF h^.isSubclass *);
    END (* IF h <> NIL *);
  END (* IF p <> NIL *);
END (* einKnoten.InsertRechts *);

FUNCTION einKnoten.DeleteLinks: GLP;
VAR     h, d:   GLP;
BEGIN
  d := Links;
  IF (d <> NIL) AND (d^.isSubclass (Knoten)) THEN
    h := einKnotenPointer (d)^.LinksIst
  ELSE h := NIL;
  Links := h;
  IF h <> NIL THEN BEGIN
    IF h^.isSubclass (Triple) THEN BEGIN
      IF einTriplePointer (h)^.Rueck = d THEN
        einTriplePointer (h)^.Rueck := @SELF;
    END (* IF isSubclass *);
  END (* IF h <> NIL *);
  DeleteLinks := d;
END (* einKnoten.DeleteLinks *);

FUNCTION einKnoten.DeleteRechts: GLP;
VAR     h, d:   GLP;
BEGIN
  d := Rechts;
  IF (d <> NIL) AND (d^.isSubclass (Knoten)) THEN
    h := einKnotenPointer (d)^.RechtsIst
  ELSE h := NIL;
  Rechts := h;
  IF h <> NIL THEN BEGIN
    IF h^.isSubclass (Triple) THEN BEGIN
      IF einTriplePointer (h)^.Rueck = d THEN
        einTriplePointer (h)^.Rueck := @SELF;
    END (* IF Member h, einTriple *);
  END (* IF h <> NIL *);
  DeleteRechts := d;
END (* einKnoten.DeleteRechts *);

FUNCTION einKnoten.LinksIst: GLP;
BEGIN LinksIst := Links;
END;

FUNCTION einKnoten.RechtsIst: GLP;
BEGIN RechtsIst := SELF.Rechts;
END;

FUNCTION einKnoten.LinksLetzter: GLP;
VAR	h:	GLP;
	stop:	BOOLEAN;
BEGIN
  h := Links; stop := FALSE;
  REPEAT
    IF h = NIL THEN BEGIN
      stop := TRUE; IF h = NIL THEN h := @SELF;
    END
    ELSE BEGIN
      IF h^.isSubclass (Knoten) THEN BEGIN
        IF einKnotenPointer (h)^.Links <> NIL
	  THEN h := einKnotenPointer (h)^.Links ELSE stop := TRUE;
       END
      ELSE stop := TRUE;
    END (* ELSE *);
  UNTIL stop;
  LinksLetzter := h;
END (* LinksLetzter *);

FUNCTION einKnoten.LinksNter (VAR n: LONGINT): GLP;
VAR	h:	GLP;
	stop:	BOOLEAN;
	z:	LONGINT;
BEGIN
    h := @SELF; stop := FALSE; z := 0;
    REPEAT
      IF (z = n) THEN BEGIN
	stop := TRUE;
      END
      ELSE BEGIN
	IF h^.isSubclass (Knoten) THEN BEGIN
	  IF einKnotenPointer (h)^.Links <> NIL THEN BEGIN
	    h := einKnotenPointer (h)^.Links; INC (z);
	  END
	  ELSE stop := TRUE;
	END
	ELSE stop := TRUE;
      END (* ELSE *);
    UNTIL stop;
    n := z; LinksNter := h;
END (* LinksNter *);

FUNCTION einKnoten.AnzahlLinks: LONGINT;
VAR	zaehler:	LONGINT;
	p:		GLP;
BEGIN
    zaehler := 0;
    p := Links;
    WHILE (p <> NIL) AND (p^.isSubclass (Knoten)) DO BEGIN
      p := einKnotenPointer (p)^.Links; INC (zaehler);
    END (* WHILE *);
    IF p <> NIL THEN INC (zaehler);
    AnzahlLinks := zaehler;
END (* AnzahlLinks *);

FUNCTION einKnoten.RechtsLetzter: GLP;
VAR	h:	GLP;
	stop:	BOOLEAN;
BEGIN
    h := Rechts; stop := FALSE;
    REPEAT
      IF h = NIL THEN BEGIN
	stop := TRUE; IF h = NIL THEN h := @SELF;
      END
      ELSE BEGIN
	IF h^.isSubclass (Knoten) THEN BEGIN
	  IF einKnotenPointer (h)^.Rechts <> NIL
	  THEN h := einKnotenPointer (h)^.Rechts ELSE stop := TRUE;
	END
	ELSE stop := TRUE;
      END (* ELSE *);
    UNTIL stop;
    RechtsLetzter := h;
END (* RechtsLetzter *);

FUNCTION einKnoten.RechtsNter (VAR n: LONGINT): GLP;
VAR	h:	GLP;
	stop:	BOOLEAN;
	z:	LONGINT;
BEGIN
    h := @SELF; stop := FALSE; z := 0;
    REPEAT
      IF (z = n) THEN BEGIN
	stop := TRUE;
      END
      ELSE BEGIN
	IF h^.isSubclass (Knoten) THEN BEGIN
	  IF einKnotenPointer (h)^.Rechts <> NIL THEN BEGIN
	    h := einKnotenPointer (h)^.Rechts; INC (z);
	  END
	  ELSE stop := TRUE;
	END
	ELSE stop := TRUE;
      END (* ELSE *);
    UNTIL stop;
    n := z; RechtsNter := h;
END (* RechtsNter *);

FUNCTION einKnoten.AnzahlRechts: LONGINT;
VAR	zaehler:	LONGINT;
	p:		GLP;
BEGIN
    zaehler := 0;
    p := Rechts;
    WHILE (p <> NIL) AND (p^.isSubclass (Knoten)) DO BEGIN
      p := einKnotenPointer (p)^.Rechts; INC (zaehler);
    END (* WHILE *);
    IF p <> NIL THEN INC (zaehler);
    AnzahlRechts := zaehler;
END (* AnzahlRechts *);

PROCEDURE einKnoten.Monitor;
BEGIN
  INHERITED Monitor (view);
  view^.WriteString ('Links:            ', 0, 0); view^.WriteGLFPointer (Links);
  view^.WriteString  (', verfolgbare Pointer: ', 0, 0); view^.WriteInt (AnzahlLinks);
    view^.WriteLn;
  view^.WriteString  ('Rechts:           ', 0, 0); view^.WriteGLFPointer (Rechts);
  view^.WriteString  (', verfolgbare Pointer: ', 0, 0); view^.WriteInt (AnzahlRechts);
    view^.WriteLn;
END (* Monitor *);

(**************************** Methoden zu einTriple **********************)

CONSTRUCTOR einTriple.Init;
BEGIN
  INHERITED Init (xflags);
  MeinTyp := Triple;
  {Rueck := NIL;}
END (* einTriple.Init *);

FUNCTION einTriple.erzeugeMich (CONST xflags: einFlagSet): GLP;
VAR  gen:  einTriplePointer;
BEGIN
  NEW (gen, Init (xflags));
  erzeugeMich := GLP (gen);
END (* erzeugeMich *);

{
DESTRUCTOR einTriple.Done;
BEGIN
  INHERITED Done;
  IF Rueck <> NIL THEN DISPOSE (Rueck, Done);
END (* Done *);
}

FUNCTION einTriple.isSubclass (CONST class: einGenTyp): BOOLEAN;
BEGIN
  isSubclass := (class = Triple) OR INHERITED isSubClass (class);
END (* Member *);

PROCEDURE einTriple.SetRueck (CONST p: GLP);
BEGIN Rueck := p;
END;

FUNCTION einTriple.RueckIst: GLP;
BEGIN RueckIst := Rueck;
END;

FUNCTION einTriple.RueckLetzter: GLP;
VAR	h:	GLP;
	stop:	BOOLEAN;
(* Achtung bei zirkulren Referenzen: Flags setzen u. testen!!! *)
BEGIN
    h := Rueck; stop := FALSE;
    REPEAT
      IF h = NIL THEN BEGIN
	stop := TRUE; h := @SELF;
      END
      ELSE BEGIN
	IF h^.isSubclass (Triple) THEN BEGIN
	  IF einTriplePointer (h)^.Rueck <> NIL
	  THEN h := einTriplePointer (h)^.Rueck ELSE stop := TRUE;
	END
	ELSE stop := TRUE;
      END (* ELSE *);
    UNTIL stop;
    RueckLetzter := h;
END (* RueckLetzter *);

FUNCTION einTriple.RueckNter (VAR n: LONGINT): GLP;
VAR	h:	GLP;
	stop:	BOOLEAN;
	z:	LONGINT;
BEGIN
    h := @SELF; stop := FALSE; z := 0;
    REPEAT
      IF (z = n) THEN BEGIN
	stop := TRUE;
      END
      ELSE BEGIN
	IF h^.isSubclass (Triple) THEN BEGIN
	  IF einTriplePointer (h)^.Rueck <> NIL THEN BEGIN
	    h := einTriplePointer (h)^.Rueck; INC (z);
	  END
	  ELSE stop := TRUE;
	END
	ELSE stop := TRUE;
      END (* ELSE *);
    UNTIL stop;
    n := z; RueckNter := h;
END (* RueckNter *);

FUNCTION einTriple.AnzahlRueck: LONGINT;
VAR	zaehler:	LONGINT;
	p:		GLP;
BEGIN
    zaehler := 0;
    p := Rueck;
    WHILE (p <> NIL) AND (p^.isSubclass (Triple)) DO BEGIN
      p := einTriplePointer (p)^.Rueck; INC (zaehler);
    END (* WHILE *);
    IF p <> NIL THEN INC (zaehler);
    AnzahlRueck := zaehler;
END (* AnzahlRueck *);

PROCEDURE einTriple.RueckNeu (CONST typ: einGenTyp; CONST xflags: einFlagSet);
BEGIN Rueck := neuesObjekt (typ, xflags);
END;

PROCEDURE einTriple.InsertRueckLinks (CONST p: GLP);
(* Einfgen in doppelt verzeigerte Liste links/rueck *)
VAR     h:      GLP;
BEGIN
   IF p <> NIL THEN BEGIN
    h := Rueck; Rueck := p;
    IF p^.isSubclass (Knoten) THEN BEGIN
      einKnotenPointer (p)^.SetLinks (@SELF);
      IF p^.isSubclass (Triple) THEN BEGIN
	einTriplePointer (p)^.SetRueck (h);
      END (* IF Triple *);
    END (* IF *);
    IF h <> NIL THEN BEGIN
      IF h^.isSubclass (Knoten) THEN BEGIN
	einKnotenPointer (h)^.setLinks (p);
      END (* IF *);
    END (* IF h <> NIL *);
   END (* IF p <> NIL *);
END (* einTriple.InsertRueckLinks *);

PROCEDURE einTriple.InsertRueckRechts (CONST p: GLP);
(* Einfgen in doppelt verzeigerte Liste rechts/rueck *)
VAR     h:      GLP;
BEGIN
   IF p <> NIL THEN BEGIN
    h := Rueck; Rueck := p;
    IF p^.isSubclass (Knoten) THEN BEGIN
      einKnotenPointer (p)^.SetRechts (@SELF);
      IF p^.isSubclass (Triple) THEN BEGIN
	einTriplePointer (p)^.SetRueck (h);
      END (* IF Triple *);
    END (* IF *);
    IF h <> NIL THEN BEGIN
      IF h^.isSubclass (Knoten) THEN BEGIN
	einKnotenPointer (h)^.setRechts (p);
      END (* IF *);
    END (* IF h <> NIL *);
   END (* IF p <> NIL *);
END (* einTriple.InsertRueckRechts *);

FUNCTION einTriple.DeleteRueck: GLP;
VAR     h, d:   GLP;
BEGIN
    d := Rueck;
    IF (d <> NIL) AND (d^.isSubclass (Triple)) THEN
      h := einTriplePointer (d)^.RueckIst
    ELSE h := NIL;
    Rueck := h;
    IF h <> NIL THEN BEGIN
      IF h^.isSubclass (Knoten) THEN BEGIN
	IF einKnotenPointer (h)^.Links = d THEN einKnotenPointer (h)^.Links := @SELF;
	IF einKnotenPointer (h)^.Rechts = d THEN einKnotenPointer (h)^.Rechts := @SELF;
      END (* IF Member h, Knoten *);
    END (* IF h <> NIL *);
    DeleteRueck := d;
END (* einTriple.DeleteRueck *);

PROCEDURE einTriple.Monitor;
BEGIN
  INHERITED Monitor (view);
  view^.WriteString ('Rueck:            ', 0, 0); view^.WriteGLFPointer (Rueck);
  view^.WriteString (', verfolgbare Pointer: ', 0, 0); view^.WriteInt (AnzahlRueck);
    view^.WriteLn;
END (* Monitor *);

END (* GLFNodes *).