UNIT GLFQuadruple;

{$F+}
{$O+}

(* (C) 1990-1995 by Johannes Leckebusch
   Version 0.902
   Stand 07. 02. 1995
*)

INTERFACE

USES Objects, Drivers, Views, GLFTypes, GLFBase, GLFNodes;

TYPE

  einQuadruplePointer = ^einQuadruple;
  einQuadruple = OBJECT (einTriple)

                Verweis:     GLP;

		CONSTRUCTOR Init (CONST xflags: einFlagSet);
                DESTRUCTOR Done; VIRTUAL;
                FUNCTION erzeugeMich (CONST xflags: einFlagSet): GLP; VIRTUAL;
                FUNCTION isSubclass (CONST class: einGenTyp): BOOLEAN; VIRTUAL;
                FUNCTION VerweisIst: GLP;
                FUNCTION meineAttribute: GLP; VIRTUAL;
                PROCEDURE SetAlias (CONST Liste: GLP); VIRTUAL;
                FUNCTION getDataPointer: GLP; VIRTUAL;
                FUNCTION getDataStr: STRING; VIRTUAL;
                PROCEDURE LoeschPfad (CONST Sig: LONGINT); VIRTUAL;

		PROCEDURE Monitor (CONST view: PBuffText); VIRTUAL;
	      END (* einQuadruple *);

  einGluePointer = ^einGlue;
  einGlue = OBJECT (einQuadruple)
		CONSTRUCTOR Init (CONST xflags: einFlagSet);
                {DESTRUCTOR Done; VIRTUAL;}
                FUNCTION erzeugeMich (CONST xflags: einFlagSet): GLP; VIRTUAL;
                FUNCTION isSubclass (CONST class: einGenTyp): BOOLEAN; VIRTUAL;
                PROCEDURE setzAssoziation (CONST Assoz: GLP);
                           VIRTUAL;
                           (* Verknpfe Objekt mit einem GLF-Pointer.
                              Benutzt Verweis-Pointer des ListGlue.
                              Bemerkung: Wenn das Objekt ein NetzString ist,
                              entsteht eine lokale Wortbasis, die ber den
                              ersten Namen ansprechbar ist. *)
                FUNCTION liesAssoziation: GLP; VIRTUAL;
                           (* Lies den mit einem Objekt verknpften GLP-
                              Pointer (Objekt), kann NIL sein wenn nie
                              gesetzt. *)
            END (* einGlue *);

  einListGluePointer = ^einListGlue;
  einListGlue = OBJECT (einGlue)

                  CONSTRUCTOR Init (CONST xflags: einFlagSet);
                  FUNCTION erzeugeMich (CONST xflags: einFlagSet): GLP; VIRTUAL;

                  FUNCTION isSubClass (CONST class: einGenTyp): BOOLEAN; VIRTUAL;
                END (* einListGlue *);

IMPLEMENTATION

USES App, EditStuff,
     GLFViews, GLFStructures, GLFLists, GLFWindows;

(************************* Methoden zu einQuadruple *******************)

CONSTRUCTOR einQuadruple.Init;
BEGIN
  einTriple.Init (xflags);
  MeinTyp := Quadruple;
  {Verweis := NIL;}
END;

PROCEDURE einQuadruple.LoeschPfad (CONST Sig: LONGINT);
BEGIN
  INHERITED LoeschPfad (Sig);
  IF Verweis <> NIL THEN BEGIN
    IF Sig = Verweis^.meineSignatur THEN Verweis := NIL
    ELSE Verweis^.LoeschPfad (Sig);
  END (* IF *);
END (* LoeschPfad *);

DESTRUCTOR einQuadruple.Done;
BEGIN
  INHERITED Done;
  IF Verweis <> NIL THEN DISPOSE (Verweis, Done);
{zeigMeldung (StringVonTyp (meinTyp), 'Done ausgefhrt');}
END (* Done *);

FUNCTION einQuadruple.erzeugeMich (CONST xflags: einFlagSet): GLP;
BEGIN
  erzeugeMich := NEW (einQuadruplePointer, Init (xflags));
END (* erzeugeMich *);

FUNCTION einQuadruple.isSubclass (CONST class: einGenTyp): BOOLEAN;
BEGIN
  isSubclass := (class = Quadruple) OR INHERITED isSubClass (class);
END (* Member *);

(********** Operationen zur Abfrage u. zum Setzen von Attributen *****)

FUNCTION einQuadruple.VerweisIst: GLP;
BEGIN
  VerweisIst := Verweis;
END (* VerweisIst *);

FUNCTION einQuadruple.meineAttribute: GLP;
BEGIN
  meineAttribute := Verweis;
END;

PROCEDURE einQuadruple.SetAlias (CONST Liste: GLP);
(* Setze das Listenobjekt als Alias (z. B. fr Pointer-Zwecke) der
   bergebenen Liste gleich. Dies scheint mir jedoch auf Dauer eine
   fragwrdige Lsung, da durch Operationen auf einem Alias die Inte-
   gritt der ersten Liste zerstrt werden kann. *)
BEGIN
  IF (Liste <> NIL) AND Liste^.isSubclass (Quadruple) THEN BEGIN
    Links := einQuadruplePointer (Liste)^.Links;
    Rechts := einQuadruplePointer (Liste)^.Rechts;
    Rueck := einQuadruplePointer (Liste)^.Rueck;
    Verweis := einQuadruplePointer (Liste)^.Verweis;
  END;
END (* SetAlias *);

FUNCTION einQuadruple.getDataPointer: GLP;
BEGIN
  getDataPointer := Rechts;
END (* getDataPointer *);

FUNCTION einQuadruple.getDataStr: STRING;
BEGIN
  IF Rechts <> NIL THEN getDataStr := Rechts^.StringVonDaten
  ELSE getDataStr := '';
END (* getDataStr *);

PROCEDURE einQuadruple.Monitor;
BEGIN
  INHERITED Monitor (view);
  view^.WriteString ('Verweis:          ', 0, 0); view^.WriteGLFPointer (Verweis);
  view^.WriteLn;
END;

(************************* Methoden zu einGlue *******************)

CONSTRUCTOR einGlue.Init;
BEGIN
  INHERITED Init (xflags);
  MeinTyp := Glue;
END;

FUNCTION einGlue.erzeugeMich (CONST xflags: einFlagSet): GLP;
BEGIN
  erzeugeMich := NEW (einGluePointer, Init (xflags));
END (* erzeugeMich *);

FUNCTION einGlue.isSubclass (CONST class: einGenTyp): BOOLEAN;
BEGIN
  isSubclass := (class = Glue) OR INHERITED isSubClass (class);
END (* Member *);

PROCEDURE einGlue.setzAssoziation (CONST Assoz: GLP);
VAR s: STRING [40];
BEGIN
  IF Verweis <> NIL THEN s := Verweis^.StringVonTyp
  ELSE s := '';
  IF (Verweis = NIL) OR (zeigFrage ('Assoziation: ' + s,
                        'Vorhandene Assoziation berschreiben?')) THEN BEGIN
    Verweis := Assoz;
  END (* IF Rechts *);
END (* setzAssoziation *);

FUNCTION einGlue.LiesAssoziation: GLP;
BEGIN
  LiesAssoziation := Verweis;
END (* LiesAssoziation *);

(************************* Methoden zu einListGlue *******************)

CONSTRUCTOR einListGlue.Init;
BEGIN
  INHERITED Init (xflags);
  MeinTyp := ListGlue;
END;

FUNCTION einListGlue.erzeugeMich (CONST xflags: einFlagSet): GLP;
BEGIN
  erzeugeMich := NEW (einListGluePointer, Init (xflags));
END (* erzeugeMich *);

FUNCTION einListGlue.isSubclass (CONST class: einGenTyp): BOOLEAN;
BEGIN
  isSubclass := (class = ListGlue) OR INHERITED isSubClass (class);
END (* isSubclass *);

END (* GLFQuadruple *).