UNIT GLFStructures;

{$F+}
{$O+}

(* (C) 1990-1995 by Johannes Leckebusch
   Version 0.902
   Stand 07. 02. 1992
*)

(* Dies ist eine abstrakte Klasse (ohne Datenfelder, auer denen, die
   von GLF ererbt werden) und mit Dummy-Methoden, die ber alle struktur-
   bildenden Klassen von GLF gestlpt ist. Strukturbildende Klassen sind
   solche, die ber Zeiger verkettet sind (Knoten, Triple, Listen, Bume,
   Netze usw.)
   oder die aus Vektoren bestehen (Vektor16 aus Pointern, aber auch String-
   Vektoren, die Bytes oder Zeichen speichern).
   GLFStructures vereinigt in sich die abstrakten Methoden, von denen alle
   strukturverndernden Methoden, wie InsertElement etc., der effektiven
   Strukturklassen abgeleitet werden.
*)

INTERFACE

USES Drivers, Objects, GLFTypes, GLFBase, {My}Views;

TYPE

        einStrukturPointer = ^eineStruktur;
        eineStruktur = OBJECT (GLF)

                CONSTRUCTOR Init (CONST xflags: einFlagSet);
                FUNCTION erzeugeMich (CONST xflags: einFlagSet): GLP; VIRTUAL;
                PROCEDURE InitMenu (VAR dieMenuLeiste: GLP);
                  VIRTUAL;
                FUNCTION isSubclass (CONST class: einGenTyp): BOOLEAN; VIRTUAL;
{
                PROCEDURE sende (CONST view: PBuffText; VAR ereignis: TEvent;
                                 CONST Sig: LONGINT);
                  VIRTUAL;
}

                PROCEDURE setzPuffer (CONST Groesse: WORD); VIRTUAL;
                PROCEDURE initAbmess (VAR a: TRect; VAR spbreite: INTEGER);
                  VIRTUAL;
                PROCEDURE setzeFensterAbmess (a: TRect);
                  VIRTUAL;
                PROCEDURE setAbmess (CONST a: TRect; CONST spbreite: INTEGER);
                  VIRTUAL;
                PROCEDURE getAbmess (VAR a: TRect; VAR spbreite: INTEGER);
                  VIRTUAL;
                PROCEDURE setOffset (CONST o: TPoint); VIRTUAL;
                PROCEDURE getOffset (VAR o: TPoint); VIRTUAL;
                PROCEDURE setPos (CONST p: TPoint); VIRTUAL;
                PROCEDURE getPos (VAR p: TPoint); VIRTUAL;

                (* spezielle Methoden zur Listenverwaltung: *)

                FUNCTION IndexIst: LONGINT; VIRTUAL;
                FUNCTION LaengeIst: LONGINT; VIRTUAL;
                  (* Liefert auch AnzahlElemente bei Vektor *)
                FUNCTION CursorIst: GLP; VIRTUAL;

                PROCEDURE ResetListe; VIRTUAL; (* Aktuelles = erstes Element *)
                PROCEDURE SetAlias (CONST Liste: GLP); VIRTUAL;
                PROCEDURE LoeschListe; VIRTUAL;
                  (* Setzt die Datenzeiger einer Liste bzw. sonstigen
                     Struktur auf NIL, ohne die referierten Objekte
                     zu disposen! *)
                  (* Setz eine Liste einer anderen gleich *)
                FUNCTION get (CONST Element: GLP): GLP; VIRTUAL;
                  (* Unquote, dh. liefert das Objekt an dem Knoten *)
                FUNCTION getActual: GLP; VIRTUAL;
                FUNCTION TypeOfActual: einGenTyp; VIRTUAL;
                (* FUNCTION ActualMember (p: GLP): BOOLEAN; VIRTUAL; *)
                FUNCTION isActualSubclass (CONST class: einGenTyp): BOOLEAN;
                PROCEDURE put (CONST Element, Objekt: GLP); VIRTUAL;
                  (* Weist dem Knoten Element das Objekt zu. Beispiel:
                     put (Liste1.FirstElement, Liste2.NtesElement (n));
                     Weist dem ersten Element der Liste 1 den Inhalt des
                     Nten Elementes der Liste 2 zu.
                  *)
                PROCEDURE putActual (CONST Objekt: GLP); VIRTUAL;
                FUNCTION FindNextObjekt (CONST Objekt: GLP): GLP;
                  VIRTUAL;
                  (* Durchsucht die Liste nach einem Knoten, der auf Objekt
                     deutet *)
                FUNCTION FindPrevObjekt (CONST Objekt: GLP): GLP; VIRTUAL;
                  (* Durchsucht die Liste nach einem Knoten, der auf Objekt
                     deutet *)

              (* Einfgen in Listen:
                 Die folgenden Operationen etablieren einen Knoten/Triple und
                 setzen seinen Rechts-Pointer auf das bergebene Objekt *)
                FUNCTION InsertFirstElement (CONST Objekt: GLP): GLP;
                  VIRTUAL;
                  (* Fgt am Anfang der Liste ein Element ein *)
                FUNCTION InsertBefore (CONST Objekt: GLP): GLP;
                  VIRTUAL;
                  (* Fgt vor dem aktuellen Element ein neues ein *)
                FUNCTION InsertAfter (CONST Objekt: GLP): GLP; VIRTUAL;
                  (* Fgt nach dem aktuellen Element ein Element ein *)
                FUNCTION AppendElement (CONST Objekt: GLP): GLP;
                  VIRTUAL;
                  (* Hngt ein Element am Ende der Liste an, bei einem
                     Ring vor dem aktuellen Element *)
               (* Elemente in Mengen setzen/lschen/toggeln *)
                FUNCTION AddElement (CONST Objekt: GLP): GLP; VIRTUAL;
                  (* Hngt Element an die Liste an, wenn es noch nicht
                     enthalten ist, liefert sonst vorhandenes Element *)
                FUNCTION SubElement (CONST Objekt: GLP): GLP; VIRTUAL;
                  (* Entfernt Element aus der Liste und gibt es zurck,
                     NIL, falls es nicht enthalten ist *)
                FUNCTION ToggleElement (CONST Objekt: GLP): GLP;
                  VIRTUAL;
                  (* Funktioniert, je nachdem, ob das Element enthalten
                     ist, wie Add oder Sub *)

                (* Die folgenden Operationen haben alle den Seiteneffekt,
                   den Listencursor und ElementIndex zu verstellen! *)
                FUNCTION ActualElement: GLP; VIRTUAL; (* liefert aktuelles Element *)
                FUNCTION FirstElement: GLP; VIRTUAL; (* impliziert Reset *)
                FUNCTION NextElement: GLP; VIRTUAL; (* verstellt aktuelles Element *)
                FUNCTION PrevElement: GLP; VIRTUAL; (* verstellt aktuelles Element *)
                FUNCTION NtesElement (CONST N: LONGINT): GLP; VIRTUAL;
                        (* stellt aktuelles auf das nte Element ein *)
                FUNCTION NOrLastElement (CONST N: LONGINT): GLP;
                  VIRTUAL;
                FUNCTION LastElement: GLP; VIRTUAL; (* verstellt aktuelles Element *)
                FUNCTION FindElement (CONST Objekt: GLP): GLP; VIRTUAL;
                  (* Durchsucht die Liste nach dem Objekt und liefert
                     seinen Knoten oder NIL *)
                FUNCTION NtesObjekt (CONST Ind: LONGINT): GLP; VIRTUAL;
                  (* quivalent zu getDataPointer (NtesObjekt (Ind)) *)

                (* Die folgenden Operationen sollen KEINEN SeitenEffekt haben *)
                FUNCTION FirstElementIs: GLP; VIRTUAL;
                FUNCTION NextElementIs: GLP; VIRTUAL;
                FUNCTION PrevElementIs: GLP; VIRTUAL;
                FUNCTION NtesElementIs (CONST N: LONGINT): GLP; VIRTUAL;
                FUNCTION NOrLastElementIs (CONST N: LONGINT): GLP; VIRTUAL;
                FUNCTION LastElementIs: GLP; VIRTUAL;
                FUNCTION FindElementIs (CONST Objekt: GLP): GLP; VIRTUAL;
                  (* Durchsucht die Liste nach dem Objekt und liefert
                     seinen Knoten oder NIL *)
                FUNCTION NtesObjektIs (CONST Ind: LONGINT): GLP; VIRTUAL;
                  (* quivalent zu getDataPointer (NtesObjekt (Ind)) *)
                FUNCTION IndexVonObjekt (CONST Objekt: GLP): LONGINT;
                  VIRTUAL;
                  (* Durchzhlen bis zum berg. Objekt, setzt
                     aktuelles Element und Index *)

                FUNCTION DeleteFirst: GLP; VIRTUAL;
                  (* Liefert das erste Element und entfernt es aus der Liste *)
                FUNCTION DeleteLast: GLP; VIRTUAL;
                  (* Liefert das letzte Element und entfernt es *)
                FUNCTION Delete: GLP; VIRTUAL;
                  (* Liefert das aktuelle Element und entfernt es *)
                FUNCTION DeleteNext: GLP; VIRTUAL;
                  (* Liefert das nchste Element und entfernt es *)

              (* Die folgenden Operationen arbeiten wie in LISP: *)
                FUNCTION Rest: GLP; VIRTUAL;
                  (* Liefert zweiten Listenknoten oder NIL, ohne aktuelles
                     Element zu beeinflussen. *)

                PROCEDURE SetObject (CONST ziel, quelle: GLP); VIRTUAL;
                  (* setzt nicht die Pointer gleich, sondern den Inhalt,
                     dh. kopiert die Verweispointer, also wie Set *)

                PROCEDURE setzAssoziation (CONST Assoz: GLP);
                             VIRTUAL;
                             (* Verknpfe Objekt mit einem GLF-Pointer.
                                Bemerkung: Wenn das Objekt ein NetzString ist,
                                entsteht eine lokale Wortbasis, die ber den
                                ersten Namen ansprechbar ist. *)
                FUNCTION liesAssoziation: GLP; VIRTUAL;
                             (* Lies den mit einem Objekt verknpften GLP-
                                Pointer (Objekt), kann NIL sein wenn nie
                                gesetzt. *)
                FUNCTION getDataPointer: GLP; VIRTUAL;

                (* spezielle Vektormethoden: *)

                PROCEDURE allocVektor (CONST anzahl); VIRTUAL;
                PROCEDURE deallocVektor; VIRTUAL;
                PROCEDURE resizeVektor (CONST anzahl); VIRTUAL;
                PROCEDURE assertVekSize (CONST anzahl); VIRTUAL;
                PROCEDURE appendWert (CONST wert); VIRTUAL;
                PROCEDURE appendTyp (CONST typ: einGenTyp); VIRTUAL;
                PROCEDURE einfuege (CONST start, anzahl; CONST elem);  VIRTUAL;
                PROCEDURE ausfuege (CONST start, anzahl); VIRTUAL;
                PROCEDURE setze (CONST index; CONST elem); VIRTUAL;
                (*PROCEDURE hole (VAR index; VAR elem);*)
                (* hole ist in GLFBase fr GLF definiert *)
                PROCEDURE neuPointer (CONST index: LONGINT; CONST typ: einGenTyp;
                                      CONST xflags: einFlagSet); VIRTUAL;
                PROCEDURE setPointer (CONST index: LONGINT; CONST p: GLP); VIRTUAL;
                FUNCTION getPointer (CONST index: LONGINT): GLP; VIRTUAL;
                PROCEDURE insertAtIndex (CONST index: LONGINT; CONST p: GLP); VIRTUAL;
                FUNCTION deleteAtIndex (CONST index: LONGINT): GLP; VIRTUAL;

                PROCEDURE TextSpeichern; VIRTUAL;
                PROCEDURE TextLaden (CONST psname: PString; CONST StartZeile: LONGINT;
                                     CONST Block: BOOLEAN);
                  VIRTUAL;
END (* eineStruktur *);

IMPLEMENTATION

USES EditStuff;

CONSTRUCTOR eineStruktur.Init;
BEGIN
  INHERITED Init (xflags); meinTyp := Struktur;
END (* Init *);

PROCEDURE eineStruktur.InitMenu (VAR dieMenuLeiste: GLP);
BEGIN
  Abstract ('InitMenu');
  dieMenuLeiste := NIL;
END (* InitMenu *);

FUNCTION eineStruktur.erzeugeMich (CONST xflags: einFlagSet): GLP;
BEGIN
  erzeugeMich := NEW (einStrukturPointer, Init (xflags));
END (* erzeugeMich *);

FUNCTION eineStruktur.isSubclass (CONST class: einGenTyp): BOOLEAN;
BEGIN
  isSubclass := (class = Struktur) OR INHERITED isSubClass (class);
END (* Member *);

PROCEDURE eineStruktur.initAbmess (VAR a: TRect; VAR spbreite: INTEGER);
BEGIN
  a.Assign (0, 0, 50, 10);
  zentriere (a);
  spbreite := 30;
END (* initAbmess *);

PROCEDURE eineStruktur.setzPuffer (CONST Groesse: WORD);
BEGIN
  Abstract ('setzPuffer');
END (* setzPuffer *);

PROCEDURE eineStruktur.setAbmess (CONST a: TRect; CONST spbreite: INTEGER);
BEGIN Abstract ('setAbmess'); END;
PROCEDURE eineStruktur.setzeFensterAbmess (a: TRect);
BEGIN Abstract ('setzeFensterAbmess'); END;
PROCEDURE eineStruktur.getAbmess (VAR a: TRect; VAR spbreite: INTEGER);
BEGIN Abstract ('getAbmess'); END;
PROCEDURE eineStruktur.setOffset (CONST o: TPoint);
BEGIN Abstract ('setOffset'); END;
PROCEDURE eineStruktur.getOffset (VAR o: TPoint);
BEGIN Abstract ('getOffset'); END;
PROCEDURE eineStruktur.setPos (CONST p: TPoint);
BEGIN Abstract ('setPos'); END;
PROCEDURE eineStruktur.getPos (VAR p: TPoint);
BEGIN Abstract ('getPos'); END;

(* spezielle Methoden zur Listenverwaltung: *)

FUNCTION eineStruktur.IndexIst: LONGINT;
BEGIN
  IndexIst := 0; Abstract ('IndexIst');
END (* IndexIst *);

FUNCTION eineStruktur.LaengeIst: LONGINT;
BEGIN
(* zeigMeldung (StringVonTyp, 'LaengeIst nicht implementiert'); *)
  LaengeIst := 0; Abstract ('LaengeIst');
END (* LaengeIst *);

FUNCTION eineStruktur.CursorIst: GLP;
BEGIN
  CursorIst := NIL; Abstract ('CursorIst');
END (* CursorIst *);

PROCEDURE eineStruktur.ResetListe;
BEGIN Abstract ('ResetListe');
END;

PROCEDURE eineStruktur.SetAlias (CONST Liste: GLP);
BEGIN
  Abstract ('SetAlias');
END;

PROCEDURE eineStruktur.LoeschListe;
BEGIN
  Abstract ('LoeschListe');
END;

FUNCTION eineStruktur.get (CONST Element: GLP): GLP;
(* Unquote, dh. liefert das Objekt an dem Knoten *)
BEGIN
  get := Element; Abstract ('get');
END (* get *);

FUNCTION eineStruktur.getActual: GLP;
BEGIN
  getActual := {@SELF} NIL {27. 9. 94}; Abstract ('getActual');
END (* getActual *);

FUNCTION eineStruktur.TypeOfActual: einGenTyp;
BEGIN
  TypeOfActual := FehlerTypX; Abstract ('TypeOfActual');
END (* TypeOfActual *);

FUNCTION eineStruktur.isActualSubclass (CONST class: einGenTyp): BOOLEAN;
VAR  g:  GLP;
BEGIN
  g := getActual;
  isActualSubclass := (g <> NIL) AND g^.isSubclass (class);
END (* isSubclass *);

PROCEDURE eineStruktur.put (CONST Element, Objekt: GLP);
BEGIN
(* Weist dem Knoten Element das Objekt zu. Beispiel:
   put (Liste1.FirstElement, Liste2.NtesElement (n));
   Weist dem ersten Element der Liste 1 den Inhalt des
   Nten Elementes der Liste 2 zu.
*)
  Abstract ('put');
END;

PROCEDURE eineStruktur.putActual (CONST Objekt: GLP);
BEGIN
  Abstract ('putActual');
END;

FUNCTION eineStruktur.FindNextObjekt (CONST Objekt: GLP): GLP;
(* Durchsucht die Liste nach einem Knoten, der auf Objekt
   deutet *)
BEGIN
  Abstract ('FindeNextObjekt'); FindNextObjekt := NIL;
END;

FUNCTION eineStruktur.FindPrevObjekt (CONST Objekt: GLP): GLP;
(* Durchsucht die Liste nach einem Knoten, der auf Objekt
   deutet *)
BEGIN
  Abstract ('FindPrevObjekt'); FindPrevObjekt := NIL;
END;

(* Die folgenden Operationen etablieren einen Knoten/Triple und
   setzen seinen Rechts-Pointer auf das bergebene Objekt *)

FUNCTION eineStruktur.InsertFirstElement (CONST Objekt: GLP): GLP;
(* Fgt am Anfang der Liste ein Element ein *)
BEGIN
  Abstract ('InsertFirstElement'); InsertFirstElement := NIL;
END;

FUNCTION eineStruktur.InsertBefore (CONST Objekt: GLP): GLP;
(* Fgt vor dem aktuellen Element ein neues ein *)
BEGIN
  Abstract ('InsertBefore'); InsertBefore := NIL;
END;

FUNCTION eineStruktur.InsertAfter (CONST Objekt: GLP): GLP;
(* Fgt nach dem aktuellen Element ein Element ein *)
BEGIN
  Abstract ('InsertAfter'); InsertAfter := NIL;
END;

FUNCTION eineStruktur.AppendElement (CONST Objekt: GLP): GLP;
(* Hngt ein Element am Ende der Liste an, bei einem
   Ring vor dem aktuellen Element *)
BEGIN
  Abstract ('AppendElement'); AppendElement := NIL;
END;

FUNCTION eineStruktur.AddElement (CONST Objekt: GLP): GLP;
(* Fgt ein Element in die Liste ein, falls es noch nicht vorhanen ist,
   gibt sonst das vorhandene zurck *)
BEGIN
  Abstract ('AddElement'); AddElement := NIL;
END (* AddElement *);

FUNCTION eineStruktur.SubElement (CONST Objekt: GLP): GLP;
(* Entfernt ein Element aus der Liste und gibt es zurck, oder
   NIL, wenn es nicht vorhanden ist *)
BEGIN
  Abstract ('SubElement'); SubElement := NIL;
END (* SubElement *);

FUNCTION eineStruktur.ToggleElement (CONST Objekt: GLP): GLP;
(* Funktioniert, je nachdem, ob das Element enthalten ist,
   wie Add oder Sub *)
BEGIN
  Abstract ('ToggleElement'); ToggleElement := NIL;
END (* ToggleElement *);

FUNCTION eineStruktur.ActualElement: GLP;
(* liefert aktuelles Element *)
BEGIN
  Abstract ('ActualElement'); ActualElement := NIL;
END;

FUNCTION eineStruktur.FirstElement: GLP;
(* impliziert Reset *)
BEGIN
{zeigMeldung ('eineStruktur (' + StringVonTyp + ').FirstElement', 'NICHT IMPLEMENTIERT');}
  Abstract ('FirstElement'); FirstElement := NIL;
END;

FUNCTION eineStruktur.NextElement: GLP;
(* verstellt aktuelles Element *)
BEGIN
  Abstract ('NextElement');
  NextElement := NIL;
END;

FUNCTION eineStruktur.PrevElement: GLP;
BEGIN
  Abstract ('PrevElement');
  PrevElement := NIL;
END; (* verstellt aktuelles Element *)

FUNCTION eineStruktur.NtesElement (CONST N: LONGINT): GLP;
(* stellt aktuelles auf das nte Element ein *)
BEGIN
  Abstract ('NtesElement');
  NtesElement := NIL;
END;

FUNCTION eineStruktur.NOrLastElement (CONST N: LONGINT): GLP;
BEGIN
  Abstract ('NOrLastElement');
  NOrLastElement := NIL;
END (* NOrLastElement *);

FUNCTION eineStruktur.LastElement: GLP;
(* verstellt aktuelles Element *)
BEGIN
  Abstract ('LastElement');
  LastElement := NIL;
END;

FUNCTION eineStruktur.FindElement (CONST Objekt: GLP): GLP;
(* Durchsucht die Liste nach dem Element und liefert
   seinen Knoten oder NIL *)
BEGIN
  Abstract ('FindElement');
  FindElement := NIL;
END;

FUNCTION eineStruktur.NtesObjekt (CONST Ind: LONGINT): GLP;
(* quivalent zu getDataPointer (NtesObjekt (Ind)) *)
VAR  elem: einStrukturPointer;
BEGIN
  GLP (elem) := NtesElement (Ind);
  IF elem <> NIL THEN
    NtesObjekt := elem^.getDataPointer
  ELSE NtesObjekt := NIL;
END (* NtesObjekt *);

FUNCTION eineStruktur.FirstElementIs: GLP;
(* impliziert Reset *)
BEGIN
  Abstract ('FirstElementIs');
  FirstElementIs := NIL;
END;

FUNCTION eineStruktur.NextElementIs: GLP;
(* verstellt aktuelles Element *)
BEGIN
  Abstract ('NextElementIs');
  NextElementIs := NIL;
END;

FUNCTION eineStruktur.PrevElementIs: GLP;
BEGIN
  Abstract ('PrevElementIs');
  PrevElementIs := NIL;
END;

FUNCTION eineStruktur.NtesElementIs (CONST N: LONGINT): GLP;
(* stellt aktuelles auf das nte Element ein *)
BEGIN
  Abstract ('NtesElementIs');
  NtesElementIs := NIL;
END;

FUNCTION eineStruktur.NOrLastElementIs (CONST N: LONGINT): GLP;
BEGIN
  Abstract ('NOrLastElementIs');
  NOrLastElementIs := NIL;
END (* NOrLastElement *);

FUNCTION eineStruktur.LastElementIs: GLP;
(* verstellt aktuelles Element *)
BEGIN
  Abstract ('LastElementIs');
  LastElementIs := NIL;
END;

FUNCTION eineStruktur.FindElementIs (CONST Objekt: GLP): GLP;
(* Durchsucht die Liste nach dem Element und liefert
   seinen Knoten oder NIL *)
BEGIN
  Abstract ('FindElementIs');
  FindElementIs := NIL;
END;

FUNCTION eineStruktur.NtesObjektIs (CONST Ind: LONGINT): GLP;
(* quivalent zu getDataPointer (NtesObjekt (Ind)) *)
VAR  elem: einStrukturPointer;
BEGIN
{
zeigMeldung ('eineStruktur (' + StringVonTyp + ')',
             'NtesObjektIs: ' + StringVonNum (Ind));
}
  GLP (elem) := NtesElementIs (Ind);
  IF elem <> NIL THEN
    NtesObjektIs := elem^.getDataPointer
  ELSE NtesObjektIs := NIL;
END (* NtesObjektIs *);
{
PROCEDURE eineStruktur.sende (CONST view: PBuffText; VAR ereignis: TEvent;
                              CONST Sig: LONGINT);
BEGIN
  INHERITED sende (view, ereignis, Sig);
END (* sende *);
}
FUNCTION eineStruktur.IndexVonObjekt (CONST Objekt: GLP): LONGINT;
BEGIN
  Abstract ('IndexVonObjekt');
  IndexVonObjekt := 0;
END (* IndexVonObjekt *);

FUNCTION eineStruktur.DeleteFirst: GLP;
(* Liefert das erste Element und entfernt es aus der Liste *)
BEGIN
  Abstract ('DeleteFirst');
  DeleteFirst := NIL;
END;

FUNCTION eineStruktur.DeleteLast: GLP;
(* Liefert das letzte Element und entfernt es *)
BEGIN
  Abstract ('DeleteLast');
  DeleteLast := NIL;
END;

FUNCTION eineStruktur.Delete: GLP;
(* Liefert das aktuelle Element und entfernt es *)
BEGIN
  Abstract ('Delete');
  Delete := NIL;
END;

FUNCTION eineStruktur.DeleteNext: GLP;
(* Liefert das nchste Element und entfernt es *)
BEGIN
  Abstract ('DeleteNext');
  DeleteNext := NIL;
END;

(************ Die folgenden Operationen arbeiten wie in LISP: *********)

FUNCTION eineStruktur.Rest: GLP;
(* Liefert zweiten Listenknoten oder NIL, ohne aktuelles
   Element zu beeinflussen. *)
BEGIN
  Abstract ('Rest');
  Rest := NIL;
END;

PROCEDURE eineStruktur.SetObject (CONST ziel, quelle: GLP);
(* setzt nicht die Pointer gleich, sondern den Inhalt,
   dh. kopiert die Verweispointer, also wie Set *)
BEGIN
  Abstract ('SetObject');
END;

PROCEDURE eineStruktur.setzAssoziation (CONST Assoz: GLP);
BEGIN
  Abstract ('setzAssoziation');
END;

FUNCTION eineStruktur.liesAssoziation: GLP;
BEGIN
  Abstract ('liesAssoziation');
  liesAssoziation := NIL;
END (* liesAssoziation *);

FUNCTION eineStruktur.getDataPointer: GLP;
BEGIN
  Abstract ('getDataPointer');
  getDataPointer := NIL;
END;

(*********************** spezielle Vektormethoden: *******************)

PROCEDURE eineStruktur.allocVektor (CONST anzahl);
BEGIN Abstract ('allocVektor');
END;
PROCEDURE eineStruktur.deallocVektor;
BEGIN Abstract ('deallocVektor');
END;
PROCEDURE eineStruktur.resizeVektor (CONST anzahl);
BEGIN Abstract ('resizeVektor');
END;
PROCEDURE eineStruktur.assertVekSize (CONST anzahl);
BEGIN Abstract ('assertVekSize');
END;
PROCEDURE eineStruktur.einfuege (CONST start, anzahl; CONST elem);
BEGIN Abstract ('einfuge');
END;
PROCEDURE eineStruktur.ausfuege (CONST start, anzahl);
BEGIN Abstract ('ausfuege');
END;
PROCEDURE eineStruktur.appendWert (CONST wert);
BEGIN Abstract ('appendWert');
END;
PROCEDURE eineStruktur.appendTyp (CONST Typ: einGenTyp);
BEGIN Abstract ('appendTyp');
END;
PROCEDURE eineStruktur.setze (CONST index; CONST elem);
BEGIN Abstract ('setze');
END;
(*PROCEDURE hole (VAR index; VAR elem);*)
(* hole ist in GLFBase fr GLF definiert *)
PROCEDURE eineStruktur.neuPointer (CONST index: LONGINT;
                                   CONST typ: einGenTyp; CONST xflags: einFlagSet);
BEGIN Abstract ('neuPointer');
END;
PROCEDURE eineStruktur.setPointer (CONST index: LONGINT; CONST p: GLP);
BEGIN Abstract ('setPointer');
END;
FUNCTION eineStruktur.getPointer (CONST index: LONGINT): GLP;
BEGIN Abstract ('getPointer');
END;
PROCEDURE eineStruktur.insertAtIndex (CONST index: LONGINT; CONST p: GLP);
BEGIN Abstract ('insertAtIndex');
END;
FUNCTION eineStruktur.deleteAtIndex (CONST index: LONGINT): GLP;
BEGIN Abstract ('deleteAtIndex');
END;

PROCEDURE eineStruktur.TextSpeichern;
BEGIN
  Abstract ('TextSpeichern');
END (* TextSpeichern *);

PROCEDURE eineStruktur.TextLaden (CONST psname: PString; CONST StartZeile: LONGINT;
                                  CONST Block: BOOLEAN);
BEGIN
  Abstract ('TextLaden');
END (* TextLaden *);

END (* GLFStructures *).
