UNIT GLFViews;
{$F+}
{$O+}

(* (C) 1990-1995 by Johannes Leckebusch
   Version 0.902
   Stand: 07. 02. 1995
*)

INTERFACE

USES Objects, Drivers, {My}Views, {My}Dialogs, {OutLine,} GLFTypes, GLFBase;

TYPE

    PTextGrafik = ^TTextGrafik;
    TTextGrafik = OBJECT (TBuffText)
                      derText: GLP;
                      CONSTRUCTOR Init (VAR Bounds: TRect; LimX, LimY: INTEGER;
                                        Offs: TPoint;
                                        AHScrollBar, AVScrollBar: PScrollBar; TextO: GLP);
                      PROCEDURE Draw; VIRTUAL;
                      PROCEDURE HandleEvent (VAR Event: TEvent); VIRTUAL;
                      PROCEDURE ScrollDraw; VIRTUAL;
                    END (* TTextGrafik *);

(********************* Inneres fr Listenwhl-Fenster *************)

    PListWaehler =  ^TListWaehler;
    TListWaehler =  OBJECT (TListViewer)
                      SpaltenBreite: INTEGER;
                      PROCEDURE Draw; VIRTUAL;
                      PROCEDURE ChangeBounds (VAR Bounds: TRect); VIRTUAL;
                      PROCEDURE HandleEvent (VAR Event: TEvent); VIRTUAL;
                      PROCEDURE BerechneSpalten; VIRTUAL;
                      FUNCTION GetPalette: PPalette; VIRTUAL;
                    END (* TListWaehler *);

    PModalWaehler =  ^TModalWaehler;
    TModalWaehler = OBJECT (TListWaehler)
                      PROCEDURE HandleEvent (VAR Event: TEvent); VIRTUAL;
                    END (* TListWaehler *);

(******************** Browser fr Windowliste ************************)

    PWindowBrowser = ^TWindowBrowser;
    TWindowBrowser = OBJECT (TModalWaehler)
                       dieFenster: PCollection;
                       dasFenster: PWindow;
                       CONSTRUCTOR Init (VAR Bounds: TRect; SPBreite: INTEGER;
                                         AHScrollBar, AVScrollBar: PScrollBar;
                                         StartWindow: PView;
                                         Start, PreSelect: INTEGER);
                       DESTRUCTOR Done; VIRTUAL;
                       PROCEDURE HandleEvent (VAR Event: TEvent); VIRTUAL;
                       FUNCTION GetText (Item: INTEGER; MaxLen: INTEGER): STRING;
                         VIRTUAL;
                       PROCEDURE SelectItem (Item: INTEGER); VIRTUAL;
                       FUNCTION GetTheWindow: PWindow; VIRTUAL;
                     END (* TWindowBrowser *);

IMPLEMENTATION

USES {My}App, EditStuff,
     GLFStructures, {GLFVektors,}
     GLFQuadruple, GLFLists,
     ObjStrings, TextBase, SimpelText;


(****************** Methoden zur Oberklasse TListWaehler ****************)

PROCEDURE TListWaehler.Draw;
(* Aus TV Views.PAS, modifiziert von Johannes Leckebusch
   11. 10. 1991 *)
VAR
  I, J, Item: Integer;
  NormalColor, SelectedColor, FocusedColor, Color: Word;
  ColWidth, CurCol, Indent: Integer;
  B: TDrawBuffer;
  Text: String;
  SCOff: Byte;
BEGIN
  if State and (sfSelected + sfActive) = (sfSelected + sfActive) then
  begin
    NormalColor := GetColor(1);
    FocusedColor := GetColor(3);
    SelectedColor := GetColor(4);
  end else
  begin
    NormalColor := GetColor(2);
    SelectedColor := GetColor(4);
  end;
  if HScrollBar <> nil then Indent := HScrollBar^.Value
  else Indent := 0;
  ColWidth := Size.X div NumCols + 1;
  (* for I := 0 to Size.Y - 1 do begin *)

  (* TListViewer baut die Darstellung zeilenweise auf und schreibt jeweils
     einen ganzen String mit allen Spalten in einer Zeile. Das ist zwar
     fr die Ausgabe sehr effizient, schafft aber fr zeigerverkettete
     Strukturen, die zum Rcksetzen auf ein I-ntes Item viel Zeit brauchen,
     schlechte Bedingungen. Daher hier nderung auf spaltenweise Ausgabe. *)
  FOR J := 0 TO NumCols-1 DO BEGIN
    FOR I := 0 TO Size.Y - 1 DO BEGIN
    (* for J := 0 to NumCols-1 do begin *)
      Item := J * Size.Y + I + TopItem;
      CurCol := J * ColWidth;
      if (State and (sfSelected + sfActive) = (sfSelected + sfActive)) and
        (Focused = Item) and (Range > 0) then
      begin
        Color := FocusedColor;
        SetCursor (CurCol+1,I);
        SCOff := 0;
      end
      else if (Item < Range) and IsSelected(Item) then
      begin
        Color := SelectedColor;
        SCOff := 2;
      end
      else
      begin
        Color := NormalColor;
        SCOff := 4;
      end;
      (* MoveChar (B [CurCol], ' ', Color, ColWidth); *)
      MoveChar (B [0], ' ', Color, ColWidth);
      if Item < Range then begin
        Text := GetText (Item, ColWidth + Indent);
        Text := Copy (Text, Indent, ColWidth);
        (* MoveStr (B [CurCol+1], Text, Color); *)
        MoveStr (B [1], Text, Color);
        if ShowMarkers then
        begin
          (*
          WordRec(B[CurCol]).Lo := Byte(SpecialChars[SCOff]);
          WordRec(B[CurCol+ColWidth-2]).Lo := Byte(SpecialChars[SCOff+1]);
          *)
          WordRec (B [0]).Lo := Byte (SpecialChars [SCOff]);
          WordRec (B [ColWidth-2]).Lo := Byte (SpecialChars [SCOff+1]);
        end;
      end;
      (* senkrechter Balken: *)
      (* MoveChar(B[CurCol+ColWidth-1], #179, GetColor(5), 1); *)
      MoveChar ( B [ColWidth-1], #179, GetColor(5), 1);
      WriteLine (CurCol, I, ColWidth, 1, B);
    end (* inneres FOR *);
    (* WriteLine(0, I, Size.X, 1, B); *)
  end (* ueres FOR *);
END (* Draw *);

FUNCTION TListWaehler.GetPalette: PPalette;
(* Ohne berschreiben dieser Funktion kommt Text in Hellwei blinkend!!! *)
CONST
  Farben = #6#1#7#2#6;
  FarbStr: STRING [LENGTH (Farben)] = Farben;
BEGIN
  GetPalette := @FarbStr;
END (* GetPalette *);

PROCEDURE TListWaehler.BerechneSpalten;
VAR Spalten:  INTEGER;
BEGIN
  Spalten := Size.X DIV SpaltenBreite;
  IF Spalten = 0 THEN INC (Spalten);
  IF numCols <> Spalten THEN BEGIN
    numCols := Spalten;
    Draw;
  END;
END (* BerechneSpalten *);

PROCEDURE TListWaehler.ChangeBounds (VAR Bounds: TRect);
BEGIN
  TListViewer.ChangeBounds (Bounds);
  BerechneSpalten;
END (* ChangeBounds *);

PROCEDURE TListWaehler.HandleEvent (VAR Event: TEvent);
VAR  Mauslokal:    TPoint;
     Balken:       TRect;
     aktSpalte:    INTEGER;
BEGIN
  (* Interaktives Anwhlen eines Listeneintrags: *)
  WITH Event DO BEGIN
    CASE What OF
      evMouseDown: IF Event.Buttons AND mbLeftButton <> 0 THEN BEGIN
                     aktSpalte := Size.X DIV numCols + 1;
                     MakeLocal (Event.Where, Mauslokal);
                     Balken.A.X := aktSpalte - 2; Balken.A.Y := 1;
                     Balken.B.X := aktSpalte;
                     Balken.B.Y := Size.Y-1;
                     IF Balken.Contains (Mauslokal) THEN BEGIN
                     (* Benutzer hat erste Spalte angeklickt,
                        Interaktives Einstellen der Spaltenbreite: *)
                       REPEAT
                         GetMouseEvent (Event);
                         IF Event.What = evMouseMove THEN BEGIN
                           MakeLocal (Event.Where, Mauslokal);
                           Spaltenbreite := Mauslokal.X;
                           IF Spaltenbreite < 3 THEN Spaltenbreite := 3;
                           BerechneSpalten;
                           Draw;
                         END (* IF *);
                       UNTIL Event.What = evMouseUp;
                       IF Spaltenbreite < 3 THEN Spaltenbreite := 3;
                   END (* IF *);
                 END (* MouseDown *);
      evKeyDown:  BEGIN
                    (*
                    Str (KeyCode, s);
                    zeigMeldung ('KeyCode', s);
                    *)
                    CASE KeyCode OF
                      (* Damit Auswahl mit Return statt Space erfolgt! *)
                      kbEnter: KeyCode := kbSpace;
                   END (* CASE KeyCode *);
                 END;
    END (* CASE *);
  END (* WITH EVENT *);

  TListViewer.HandleEvent (Event);
END (* HandleEvent *);

(********************* Modal betriebener Browser ***********************)

PROCEDURE TModalWaehler.HandleEvent (VAR Event: TEvent);
BEGIN
  (* Interaktives Anwhlen eines Listeneintrags: *)
  WITH Event DO BEGIN
    CASE What OF
      evCommand:  BEGIN
                    IF Command = cmClose THEN BEGIN
                      EndModal (cmCancel);
                      ClearEvent (Event);
                    END;
                  END;
      evKeyDown:  BEGIN
                    CASE KeyCode OF
                      (* Damit Auswahl mit Return statt Space erfolgt! *)
                      kbEsc: BEGIN
                               EndModal (cmCancel);
                               ClearEvent (Event);
                             END;
                   END (* CASE KeyCode *);
                 END;
    END (* CASE *);
  END (* WITH EVENT *);

  TListWaehler.HandleEvent (Event);
END (* HandleEvent *);

(******************** Browser fr Windowliste ************************)

CONSTRUCTOR TWindowBrowser.Init (VAR Bounds: TRect; SPBreite: INTEGER;
                  AHScrollBar, AVScrollBar: PScrollBar;
                  StartWindow: PView;
                  Start, PreSelect: INTEGER);
VAR  Spalten:   INTEGER;
     Zeilen:    INTEGER;
     win:       PWindow;
     AnzahlW:   INTEGER;
     i:         INTEGER;
     tit:       STRING;
     len:       INTEGER;
BEGIN
  TModalWaehler.Init (Bounds, Spalten, AHScrollBar, AVScrollBar);
  (* Objektvariable mssen NACH Aufruf des Constructors initialisiert
     werden!!! *)
  len := hatBreite (Bounds);

  (* Anzahl der Fenster ermitteln: *)
  win := PWindow (StartWindow); AnzahlW := 0;
  WHILE (win <> NIL) DO BEGIN
    INC (AnzahlW);
    win := PWindow (win^.NextView);
  END (* WHILE *);
  TopItem := Start;
  IF PreSelect + 1 >= AnzahlW THEN PreSelect := AnzahlW - 2;
  Focused := PreSelect;
  IF TopItem < Focused - Zeilen THEN TopItem := Focused - Zeilen;
  dieFenster := NIL;

  IF AnzahlW > 0 THEN DEC (AnzahlW);
  SetRange (AnzahlW);

  (* Einrichten der Collection, welche die WindowPointer speichert: *)

  IF AnzahlW > 0 THEN BEGIN
    dieFenster := NEW (PCollection, Init (AnzahlW, AnzahlW));
    win := PWindow (StartWindow);
    i := 0;
    WHILE (i < AnzahlW) {(win <> NIL)} DO BEGIN
      INC (i);
      tit := win^.GetTitle (len);
      IF LENGTH (tit) {+ 2} > len THEN len := LENGTH (tit) {+ 2};
      dieFenster^.Insert (win);
      win := PWindow (win^.NextView);
    END (* WHILE *);
  END (* IF AnzahlW > 0 *);
  SpaltenBreite := SPBreite;

  IF len > hatBreite (Bounds) THEN
    Bounds.B.X := Bounds.A.X + len;
  Spalten := hatBreite (Bounds) DIV SPBreite;
  IF Spalten = 0 THEN INC (Spalten);
  Zeilen :=  hatHoehe (Bounds) - 2;

  GrowMode := gfGrowHiX + gfGrowHiY;
  Options := Options OR ofFramed;
  BerechneSpalten;
  dasFenster := NIL;
END (* Init *);

DESTRUCTOR TWindowBrowser.Done;
BEGIN
  (* um die Pointer zu lschen, ohne die Fenster, auf die sie zeigen,
     zu entfernen *)
  IF dieFenster <> NIL THEN BEGIN
    dieFenster^.DeleteAll;
    (* Jetzt kann man die Collection lschen, ohne da die Objekte gelscht
       werden, auf die ihre Pointer zeigen! *)
    DISPOSE (dieFenster, Done);
  END (* IF dieFenster <> NIL *);
  TModalWaehler.Done;
END (* Done *);

PROCEDURE TWindowBrowser.HandleEvent (VAR Event: TEvent);
BEGIN
  TModalWaehler.HandleEvent (Event);
END (* HandleEvent *);

FUNCTION TWindowBrowser.GetText (Item: INTEGER; MaxLen: INTEGER): STRING;
VAR  PItem: PWindow;
BEGIN
  IF dieFenster <> NIL THEN BEGIN
    PItem := dieFenster^.At (Item);
    IF PItem <> NIL THEN
      GetText := PItem^.GetTitle (MaxLen)
    ELSE GetText := 'ERROR in dieFenster';
  END
  ELSE GetText := 'Kein Fenster vorhanden';
END (* GetText *);

PROCEDURE TWindowBrowser.SelectItem (Item: INTEGER);
BEGIN
  IF dieFenster <> NIL THEN BEGIN
    dasFenster := dieFenster^.At (Item);
  END (* IF *);
  EndModal (cmSelect);
END (* SelectItem *);

FUNCTION TWindowBrowser.GetTheWindow: PWindow;
BEGIN
  GetTheWindow := dasFenster;
END (* GetWindow *);

(************************ Methoden zu TTextGrafik **********************)

CONSTRUCTOR TTextGrafik.Init (VAR Bounds: TRect; LimX, LimY: INTEGER;
                              Offs: TPoint;
                              AHScrollBar, AVScrollBar: PScrollBar; TextO: GLP);
BEGIN
  INHERITED Init (Bounds, LimX, LimY, Offs, AHScrollBar, AVScrollBar);
  derText := TextO; ShowCursor;
END (* Init *);

PROCEDURE TTextGrafik.Draw;
BEGIN
  derText^.zeigFenster (@SELF, 0, 0, 0, 0, FALSE);
END (* Draw *);

PROCEDURE TTextGrafik.HandleEvent (VAR Event: TEvent);
VAR  Pos:     TPoint;
     Abmess:  TRect;
     dummyS:  INTEGER;
BEGIN
  CASE Event.What OF
    evMouseDown: BEGIN
                   MakeLocal (Event.Where, Event.Where);
                   IF derText <> NIL THEN
                     derText^.startNetzMessage (@SELF, Event);
                 END;
    evCommand:  CASE Event.Command OF
                  cmQuit:       BEGIN
                                    DISPOSE (einSimpleTextPointer (derText), Done);
                                    derText := NIL;
                                END;
                  cmClose:      BEGIN
                                  IF (einSimpleTextPointer (derText)^.hatInternFlag (istClipBoard)) OR
                                    (zeigFrage ('Textfenster schlieen',
                                     'Wollen Sie wirklich den Text schlieen?'))
                                  THEN BEGIN
{zeigMeldung ('cmClose', 'setzFensterInhalt (NIL)');}
                                    einSimpleTextPointer (derText)^.
                                      setzFensterInhalt (NIL);
{zeigMeldung ('cmClose', 'zurck von setzFensterInhalt (NIL)');}
                                    IF NOT (einSimpleTextPointer (derText)^.
                                      hatInternflag (istUndopuffer) OR
                                      einSimpleTextPointer (derText)^.
                                      hatInternflag (istProtokollpuffer) OR
                                      einSimpleTextPointer (derText)^.
                                      hatInternflag (istClipboard))
                                      THEN BEGIN
{zeigMeldung ('ist nicht Clipboard', 'derText^.LoeschPfad');}
                                        derText^.LoeschPfad (derText^.neuSig);
                                        DISPOSE (einSimpleTextPointer (derText), Done);
                                      END;
{zeigMeldung ('ist nicht Clipboard', 'derText := NIL');}
                                      derText := NIL;
                                      {INHERITED HandleEvent (Event);}
                                  END (* IF zeigFrage *)
                                  ELSE BEGIN
{zeigMeldung ('war Clipboard', 'ClearEvent');}
                                    ClearEvent (Event);
                                    EXIT;
                                  END;
                                END (* cmClose *);

                END (* CASE *);
   evBroadcast: CASE Event.Command OF
                  cmClose(*All*):   BEGIN
                                    einSimpleTextPointer (derText)^.
                                      setzFensterInhalt (NIL);
                                    IF NOT (einSimpleTextPointer (derText)^.
                                      hatInternflag (istUndopuffer) OR
                                      einSimpleTextPointer (derText)^.
                                      hatInternflag (istProtokollpuffer) OR
                                      einSimpleTextPointer (derText)^.
                                      hatInternflag (istClipboard))
                                      THEN BEGIN
                                        einSimpleTextPointer (derText)^.loescheInternflag (DateinameAbfragen);
                                        derText^.LoeschPfad (derText^.neuSig);
                                        DISPOSE (einSimpleTextPointer (derText), Done);
                                    END;
                                    derText := NIL;
                                    Event.What := evCommand;
(* Das ist ein sehr seltsamer Mechanismus, den ich selbst nicht
   verstehe, aber wenn ich einen anderen Event als cmClose whle,
   kommt dieser hier nicht an - daher evBroadcast zur Unterscheidung.
   Der darf aber dann wohl nicht mehr weitergegeben werden, sonst
   GPF. *)
                                END;{}
                END (* inneres CASE *);
  END (* CASE *);

  INHERITED HandleEvent (Event);
  IF (Event.What <> evNothing) AND (derText <> NIL) THEN BEGIN
     ShowCursor;
     einSimpleTextPointer (derText)^.schreibStatus;
     derText^.startNetzMessage (@SELF, Event);
  END (* <> evNothing *);
  (* ClearEvent (Event); *)
END (* HandleEvent *);

PROCEDURE TTextGrafik.ScrollDraw;
BEGIN
  INHERITED ScrollDraw;
  (* zeigMeldung ('ScrollDraw', 'Event ausgefhrt'); *)
  derText^.setOffset ({Long}Delta);
  einSimpleTextPointer (derText)^.schreibStatus;
END (* ScrollDraw *);

END (* GLFViews *).
