UNIT GLFWordBase;

{$F+}
{$O+}

(* (C) 1991, 1993 by Johannes Leckebusch
   Version 0.703
   Stand 23. 11. 1993
*)

(* Ableitung von GLFStructures berprfen, berarbeiten... *)

(* nderung ab 24. 9. 1991:
   Um das Problem der Gro-Kleinschreibung in den Griff zu bekommen, wird
   jeder Netzstring mit einer Bitmap versehen. Die Bedeutung der Bits:
   Index:  Zuordnung zum Zeichen unter demselben Index im String
   0:      Kleinschreibung
   1:      Groschreibung
   Alle Zeichen werden auf Kleinschreibung normalisiert. Begrndung:
   Grogeschriebene Zeichen sind umzuwandeln, diese Operation sollte
   mglichst selten ntig sein (Grobuchstaben sind seltener).
   Die Netzstrings selbst haben mit diesen Manamen nichts zu tun. Erzeugen
   der Bitmap und Normalisieren wird von einLexikon vorgenommen. Die
   Funktion StringVonBegriff nimmt eine Einstellung der Schreibung vor.

   Es knnen Begriffe mit demselben Wortlaut, aber unterschiedlicher
   Schreibung gespeichert werden. Daher mu die Bitmap jedes Begriffes Bits
   fr den ganzen Bezeichner, nicht nur fr die Teilkette.

   Beim Einspeichern eines Begriffes wird fr diesen die Bitmap generiert,
   dann wird er auf Kleinschreibung normalisiert. Der Pointer auf die Bitmap
   wird an den terminalen Netzstring fr den Begriff weitergegeben.

   Beim Auslesen wird er entsprechend zurckgewandelt.

   nderung ab 13. 10. 91: Um die Speicherung umfangreicher Indizes zu
   optimieren, werden Seitenvektoren nur noch in einer Mindestlnge ge-
   speichert. Die Mindestlnge betrgt immer 1, wenn alle Vektorelemente NIL
   sind oder wenn das Vektorelement 0 das einzige ist, das nicht-NIL ist.
   (Knnte ev. auch noch wegoptimiert werden... ???)
   Ansonsten wird der Vektor so angelegt, da das letzte Element, das
   nicht-NIL ist, noch erfat wird (es gibt ein neues Attribut "AnzahlSeiten").

   Dazu gehrt die nderung aller Abfragen von AnzahlElemente auf AnzahlSeiten,
   sofern sie sich auf den Seitenzeiger-Vektor (und nicht den Zeichenvektor)
   beziehen,
   sowie nderungen in resizeVektor und die neue Operation assertVekSize.
   resizeVektor versucht automatisch, den Seitenvektor zu verkrzen, wenn
   er am Ende nur NIL-Zeiger enthlt.

*)

INTERFACE

USES Drivers, GLFTypes, GLFBase, ObjStrings, Views;

TYPE

    StringRelation =    (kleiner, kleinergleich, gleich, groessergleich,
                         groesser, undefiniert);

    ParamP = ^BegrParam;
    BegrParam =  RECORD
                   wort:           StringPointer;
                   schreibExakt:   BOOLEAN;
                   neu:            BOOLEAN;
                   PSchreib:       PBitmap;
                 END (* BegrParam *);

  einVektorArray16Pointer =     ^einVektorArray16;
  einVektorArray16 =            ARRAY [SVekRange16] OF GLP;

    einNetzStringPointer = ^einNetzString;
    einNetzString =      OBJECT (einString8)
                           AnzahlSeiten:     BYTE;
                           SeitenVektor:     einVektorArray16Pointer;
                           (* Naechster:       GLP; --> SeitenVektor [0] *)
                           (* == SeitenVektor [0] *)
                           Rueck:            GLP;
                           Data:             GLP;
                           Offset:           BYTE;
                           Schreibung:       PBITMAP;
    (* Ich knnte auch einfach zwei Bits pro CHAR verwenden, um Umlautungen
       getrennt zu erfassen, ohne da dazu ein weiterer Pointer ntig wre.
       Noch kompakter wre es, die einzelnen BitSets an den String selbst
       anzuhngen! *)

    (* Allgemeine virtuelle Methoden *)

                           CONSTRUCTOR Init (CONST xflags: einFlagSet);
                           DESTRUCTOR Done; VIRTUAL;
                           DESTRUCTOR DoneCluster; VIRTUAL;
                           PROCEDURE LoeschPfad (CONST Sig: LONGINT); VIRTUAL;
                           PROCEDURE LoeschCluster (CONST Sig: LONGINT); VIRTUAL;
                           FUNCTION erzeugeMich (CONST xflags: einFlagSet): GLP; VIRTUAL;
                           FUNCTION isSubclass (CONST class: einGenTyp): BOOLEAN; VIRTUAL;

                           FUNCTION isTerminal: BOOLEAN; VIRTUAL;
                           PROCEDURE nulleNetzFlag (CONST flag: einFlag; CONST Sig: LONGINT);
                             VIRTUAL;
                           PROCEDURE setzeNetzFlag (CONST flag: einFlag; CONST Sig: LONGINT);
                             VIRTUAL;
                           PROCEDURE sende (CONST view: PBuffText; VAR ereignis: TEvent;
                                            CONST Sig: LONGINT);
                             VIRTUAL;
                           FUNCTION LaengeIst: LONGINT; VIRTUAL;

    (* berschriebene virtuelle Basis- und Vektormethoden *)

                           FUNCTION meineAttribute: GLP; VIRTUAL;
                             (* ergibt Objektpointer *)
                           FUNCTION habeAttribute: BOOLEAN; VIRTUAL;
                             (* ergibt TRUE *)
                           PROCEDURE setAttribute (CONST a: GLP); VIRTUAL;
                             (* setzt ObjektPointer *)
                           FUNCTION RueckIst: GLP; VIRTUAL;
                             (* ergibt RueckPointer *)
                           FUNCTION getPointer (CONST index: LONGINT): GLP;
                             VIRTUAL; (* ergibt Seitenpointer *)
                           FUNCTION getNamePointer: GLP; VIRTUAL;
                           FUNCTION getDataPointer: GLP; VIRTUAL;
                           FUNCTION getDataStr: STRING; VIRTUAL;
    (* berschriebene virtuelle Stringmethoden *)

                           FUNCTION gran (g: WORD): WORD; VIRTUAL;
                           PROCEDURE allocVektor (CONST anzahl); VIRTUAL;
                           PROCEDURE deallocVektor; VIRTUAL;
                           PROCEDURE resizeVektor (CONST anzahl); VIRTUAL;
                           PROCEDURE assertVekSize (CONST anzahl); VIRTUAL;
                           PROCEDURE holeString (VAR s); VIRTUAL;
                           FUNCTION Edit (CONST view: PBuffText;
                                          CONST x, y, b, h: WORD;
                                          CONST name: GLP): GLP;
                             VIRTUAL;
                             (* trgt neuen Begriff in das Netz ein *)

    (* Methoden nur von einNetzString *)

                           PROCEDURE normalisiereBezeichner (CONST wort: StringPointer;
                                                             VAR PSchreib: PBITMAP); VIRTUAL;
                          (* Wandelt alle Zeichen in dem String von begr
                             in Kleinbuchstaben um, erzeugt eine Bitmap fr
                             den String und setzt alle Bits fr Zeichen,
                             die ursprnglich grogeschrieben waren *)
                           PROCEDURE denormalisiereBezeichner (CONST wort: StringPointer;
                                   PSchreib: PBITMAP); VIRTUAL;
                          (* Wandelt alle Zeichen in dem String von begr,
                             deren zugehriges Schreibungsbit gesetzt ist,
                             in Grobuchstaben um. *)

                           FUNCTION groessergeschrieben (CONST Test, Referenz: PBITMAP;
                                                         CONST AnzZeichen: WORD): StringRelation;
                             VIRTUAL;
                           (* Liefert TRUE, wenn Test z. B. mit einem Grobuch-
                              staben beginnt, Referenz aber mit einem kleinen.
                              Entscheidet nach dem ersten Grobuchstaben gegen-
                              ber einem Kleinbuchstaben in (gleichlautenden)
                              Bezeichnern. *)

                           FUNCTION ZeichenVergleich (CONST a, b: CHAR;
                                      CONST vorher: StringRelation):
                                      StringRelation; VIRTUAL;

                           FUNCTION NewParam: ParamP;

                           FUNCTION sucheBegriff (CONST Param: ParamP;
                                                  CONST ind: WORD;
                                                  VAR absender: GLP;
                                                  VAR elemIndex: LONGINT): GLP; VIRTUAL;
                             (* suche String im Netz,
                                NIL, wenn unbekannt, sonst WortHandle *)

                           FUNCTION insertBegriff (CONST Param: ParamP;
                                                   CONST ind: WORD;
                                                   CONST absender,
                                                   rueckverweis: GLP;
                                                   CONST absind: WORD):
                                                  GLP; VIRTUAL;
                             (* String eintragen und Handle zurckgeben, NIL
                                wenn schon vorhanden UND neu=TRUE, sonst
                                vorhandenes Handle zurckgeben *)

                           FUNCTION doInsertkleiner (CONST Param: ParamP;
                                                   CONST ind: WORD;
                                                   CONST absender,
                                                   rueckverweis: GLP;
                                                   CONST absind: WORD;
                                                   CONST gleichbis: WORD):
                                                  GLP; VIRTUAL;

                           FUNCTION doInsertkleinergleich (CONST Param: ParamP;
                                                   CONST ind: WORD;
                                                   CONST absender,
                                                   rueckverweis: GLP;
                                                   CONST absind: WORD;
                                                   CONST gleichbis: WORD):
                                                  GLP; VIRTUAL;

                           FUNCTION doInsertgleich (CONST Param: ParamP;
                                                   CONST ind: WORD;
                                                   CONST absender,
                                                   rueckverweis: GLP;
                                                   CONST gleichbis: WORD):
                                                  GLP; VIRTUAL;

                           FUNCTION doInsertgroessergleich (CONST Param: ParamP;
                                                   CONST ind: WORD;
                                                   CONST gleichbis: WORD):
                                                  GLP; VIRTUAL;

                           FUNCTION doInsertgroesser (CONST Param: ParamP;
                                                   CONST ind: WORD;
                                                   CONST absender,
                                                         rueckverweis: GLP;
                                                   CONST gleichbis: WORD):
                                                  GLP; VIRTUAL;

                           FUNCTION loescheBegriff (CONST abs: GLP): GLP; VIRTUAL;
                             (* Begriff austragen *)
                           {}
                           FUNCTION indiziereBegriffe (VAR Index: LONGINT;
                                                           ind: WORD;
                                                       CONST stop: GLP;
                                                       VAR gefunden: BOOLEAN): GLP;
                             (* Durchindizieren bis stop oder NIL. Diese Prozedur
                                verfolgt keine Rckpointer, daher nur von der obersten
                                Ebene aus aufrufen! *)
                             VIRTUAL;
                          (* siehe Vorbemerkung - liefert Anzahl der Begriffe
                             und setzt diese. In jedem Begriffsobjekt wird
                             das Feld "meineReferenz" auf den Index gesetzt. "Stop" ist
                             der Begriff, bei dem gestoppt werden soll, NIL fr
                             Ende des Netzes. *)
                           {}
		           FUNCTION LastElement: GLP; VIRTUAL{ dLastElement}; (* liefert hier @SELF *)

                           FUNCTION letzterBegriff: GLP; VIRTUAL;
                             (* alphabetisch letzten Begriff - nur in Lexikon implementiert *)

                           FUNCTION naechsterBegriff (ind: WORD): GLP;
                             VIRTUAL;
                             (* alphabetisch nchsten Begriff nach dem
                                Begriff SELF, NIL wenn keiner
                                mehr vorhanden. *)

                           FUNCTION vorigerBegriff: GLP;
                             VIRTUAL;
                             (* alphabetisch voriger Begriff nach dem
                                Begriff SELF, NIL wenn keiner
                                mehr vorhanden, n: Sprung *)

                           FUNCTION aehnlichsterBegriff (CONST wort: StringPointer;
                                                         VAR verg: StringRelation;
                                                         VAR gleichbis: WORD;
                                                         CONST ind: WORD;
                                                         VAR abs: GLP): GLP;
                             VIRTUAL;
                             (* suche den alphabetisch unmittelbar folgenden
                                Begriff, wenn das eingesetzte Wort nicht
                                vorhanden ist, sonst wird dieses selbst
                                ausgegeben. Gibt man den Anfang eines bekannten
                                Begriffes ein, so wird dieser ermittelt (in
                                der krzesten Fassung, HAN-> HANS. HANSWURST
                                ist dann (einer) der nchste(n).
                                "exakt" gibt an, ob der gefundene Begriff
                                dem gesuchten genau gleich war. *)

                           FUNCTION StringVergleich (CONST s1, s2: STRING): StringRelation;
                             VIRTUAL;

                           FUNCTION Liste (CONST n: LONGINT; CONST Start: GLP): GLP;
                             VIRTUAL{ dListe};
                             (* Gibt die Liste der n Begriffe, die dem Begriff
                                Start alphabetisch folgen, einschlielich
                                von Start. *)
                           PROCEDURE setzAssoziation (CONST Assoz: GLP);
                             VIRTUAL{ dSetzAssoziation};
                             (* Verknpfe Wort mit einem GLF-Pointer.
                                Bemerkung: Wenn das Objekt ein NetzString ist,
                                entsteht eine lokale Wortbasis, die ber den
                                ersten Namen ansprechbar ist. *)
                           FUNCTION liesAssoziation: GLP;
                             VIRTUAL{ dLiesAssoziation};
                             (* Lies den mit einem Wort verknpften GLP-
                                Pointer (Objekt), kann NIL sein wenn nie
                                gesetzt. *)
                           PROCEDURE StringvonBegriff (VAR str: STRING);
                             VIRTUAL;
                           FUNCTION StringVonDaten: STRING; VIRTUAL;

                           PROCEDURE Monitor (CONST view: PBuffText); VIRTUAL;

                           FUNCTION insertSeitenkette (CONST Param: ParamP;
                                                       CONST naechster: GLP;
                                                       CONST absender,
                                                       rueckverweis: GLP;
                                                       CONST absind: WORD;
                                                       CONST ind: WORD):
                                                       GLP; VIRTUAL;
                           PROCEDURE wortZusammenlegen (CONST wort: StringPointer;
                                           CONST ind: WORD; CONST abs: GLP); VIRTUAL;
                           FUNCTION spalteWort (CONST Param: ParamP;
                                                CONST gleichbis: WORD;
                                                CONST absender, rueckverweis: GLP;
                                                CONST absind: WORD): GLP;
                             VIRTUAL;
                           PROCEDURE verbindeSeitenkette (CONST ind: WORD;
                                                          CONST ziel: GLP;
                                                          CONST ruecklink, weitergeben: BOOLEAN);
                             VIRTUAL;
                           PROCEDURE setzeRueck (CONST ziel: GLP; CONST weitergeben: BOOLEAN);
                             VIRTUAL;
                           FUNCTION vergleiche (CONST wort: StringPointer;
                                     CONST ind: WORD; VAR gleichbis: WORD):
                                     StringRelation; VIRTUAL;
                             (* Vergleiche String mit Kette des Objektes *)
                           FUNCTION insertReststring (CONST wort: StringPointer;
                                                      CONST ind: WORD): GLP; VIRTUAL;
                           PROCEDURE baueString (VAR ind: WORD;
                                                 CONST stop: WORD;
                                                 VAR str: STRING); VIRTUAL;
                           FUNCTION neuerNetzString: GLP; VIRTUAL;
                           FUNCTION insertStringConst (s: STRING; CONST eindeutig: BOOLEAN):
                             einNetzStringPointer;
                           FUNCTION insertString (CONST s: STRING; CONST eindeutig: BOOLEAN):
                             einNetzStringPointer; VIRTUAL;
                         END;

IMPLEMENTATION

USES Objects, EditStuff, GLFStructures, GLFQuadruple;

CONSTRUCTOR einNetzString.Init;
BEGIN
  INHERITED Init (xflags);
  {AnzahlSeiten := 0;}
  MeinTyp := NetzString;
  {
  SeitenVektor := NIL; (* bedeutet, da WordBase leer *)
  Rueck := NIL; Data := NIL;
  Offset := 0; Schreibung := NIL;
  }
END (* Init *);

DESTRUCTOR einNetzString.Done;
VAR i, mem: WORD;
BEGIN
{
zeigMeldung ('einNetzString.Done', StringVonTyp (meinTyp) + atString (@SELF));
}
  IF SeitenVektor <> NIL THEN BEGIN
    IF AnzahlSeiten > 0 THEN
      FOR i := 0 TO AnzahlSeiten - 1 DO BEGIN
        IF SeitenVektor^[i] <> NIL THEN BEGIN
          DISPOSE (SeitenVektor^[i], Done);
        END (* IF <> NIL *);
      END (* FOR *);
  END (* IF SeitenVektor *);

  (* Aus dem "IF SeiteVektor <> NIL" rausgezogen am 19. 1. 92: *)
  IF Schreibung <> NIL THEN BEGIN
{
zeigMeldung ('einNetzString.Done',
            'FreeMem (Schreibung):' + atString (Schreibung) + '; ' +
           StringVonNum ((derString^.Laenge + Offset - 1) DIV 8 + 1));
}
    IF (derString^.Laenge + Offset) > 0 THEN
      mem := (derString^.Laenge + Offset - 1) DIV 8 + 1
    ELSE mem := 1;
    FreeMem (Schreibung, mem);
  END (* IF Schreibung *);

  INHERITED Done;
{
zeigMeldung ('einString8.Done', 'Fertig');
}
  IF Rueck <> NIL THEN DISPOSE (Rueck, Done);
  IF Data <> NIL THEN DISPOSE (Data, Done);
END (* Done *);

DESTRUCTOR einNetzString.DoneCluster;
VAR i, mem: WORD;
BEGIN
{
zeigMeldung ('einNetzString.DoneCluster', StringVonTyp (meinTyp) + atString (@SELF));
}
  IF SeitenVektor <> NIL THEN BEGIN
    IF AnzahlSeiten > 0 THEN
      FOR i := 0 TO AnzahlSeiten - 1 DO BEGIN
        IF SeitenVektor^[i] <> NIL THEN BEGIN
          DISPOSE (SeitenVektor^[i], DoneCluster);
        END (* IF <> NIL *);
      END (* FOR *);
  END (* IF SeitenVektor *);

  (* Aus dem "IF SeiteVektor <> NIL" rausgezogen am 19. 1. 92: *)
  IF Schreibung <> NIL THEN BEGIN
{
zeigMeldung ('einNetzString.DoneCluster',
            'FreeMem (Schreibung):' + atString (Schreibung) + '; ' +
           StringVonNum ((derString^.Laenge + Offset - 1) DIV 8 + 1));
}
    IF (derString^.Laenge + Offset) > 0 THEN
      mem := (derString^.Laenge + Offset - 1) DIV 8 + 1
    ELSE mem := 1;
    FreeMem (Schreibung, mem);
  END (* IF Schreibung *);

  INHERITED DoneCluster;

  {
  IF Rueck <> NIL THEN DISPOSE (Rueck, Done);
  }
END (* DoneCluster *);

FUNCTION einNetzString.erzeugeMich (CONST xflags: einFlagSet): GLP;
BEGIN
  erzeugeMich := NEW (einNetzStringPointer, Init (xflags));
END (* erzeugeMich *);

FUNCTION einNetzString.isSubclass (CONST class: einGenTyp): BOOLEAN;
BEGIN
  isSubclass := (class = NetzString) OR INHERITED isSubClass (class);
END (* Member *);

FUNCTION einNetzString.isTerminal: BOOLEAN;
BEGIN
  isTerminal := FALSE;
END (* isTerminal *);

PROCEDURE einNetzString.setzeNetzFlag (CONST flag: einFlag; CONST Sig: LONGINT);
VAR   i:  WORD;
BEGIN
  IF (Sig <> meineSignatur) THEN BEGIN
    INHERITED setzeNetzFlag (flag, Sig);
    IF (SeitenVektor <> NIL) AND (AnzahlSeiten > 0) THEN BEGIN
      FOR i := 0 TO AnzahlSeiten - 1 DO BEGIN
        IF SeitenVektor^ [i] <> NIL THEN
          SeitenVektor^ [i]^.setzeNetzFlag (flag, Sig);
      END (* FOR *);
    END (* IF SeitenVektor <> NIL *);
    IF Rueck <> NIL THEN BEGIN
      Rueck^.setzeNetzFlag (flag, Sig);
    END;
    IF Data <> NIL THEN BEGIN
      Data^.setzeNetzFlag (flag, Sig);
    END;
  END (* IF NOT besucht *);
END (* setzeNetzFlag *);

PROCEDURE einNetzString.nulleNetzFlag (CONST flag: einFlag; CONST Sig: LONGINT);
VAR   i:  WORD;
BEGIN
  IF (Sig <> meineSignatur) THEN BEGIN
    INHERITED nulleNetzFlag (flag, Sig);
    IF (SeitenVektor <> NIL) AND (AnzahlSeiten > 0) THEN BEGIN
      FOR i := 0 TO AnzahlSeiten - 1 DO BEGIN
        IF SeitenVektor^ [i] <> NIL THEN
          SeitenVektor^ [i]^.nulleNetzFlag (flag, Sig);
      END (* FOR *);
    END (* IF SeitenVektor <> NIL *);
    IF Rueck <> NIL THEN BEGIN
      Rueck^.nulleNetzFlag (flag, Sig);
    END;
    IF Data <> NIL THEN BEGIN
      Data^.nulleNetzFlag (flag, Sig);
    END;
  END (* IF NOT besucht *);
END (* nulleNetzFlag *);

PROCEDURE einNetzString.loeschPfad (CONST Sig: LONGINT);
VAR i:  WORD;
BEGIN
  INHERITED LoeschPfad (Sig);

  IF (SeitenVektor <> NIL) AND (AnzahlSeiten > 0) THEN BEGIN
    FOR i := 0 TO AnzahlSeiten - 1 DO BEGIN
      IF SeitenVektor^[i] <> NIL THEN BEGIN
        IF Sig = SeitenVektor^[i]^.meineSignatur THEN
          SeitenVektor^[i] := NIL
        ELSE SeitenVektor^[i]^.LoeschPfad (Sig);
      END (* IF <> NIL *);
    END (* FOR *);
  END (* IF SeitenVektor *);

  IF Rueck <> NIL THEN BEGIN
    IF Sig = Rueck^.meineSignatur THEN Rueck := NIL
    ELSE Rueck^.LoeschPfad (Sig);
  END (* IF *);
  IF Data <> NIL THEN BEGIN
    IF Sig = Data^.meineSignatur THEN Data := NIL
    ELSE Data^.LoeschPfad (Sig);
  END (* IF *);
END (* loeschPfad *);

PROCEDURE einNetzString.loeschCluster (CONST Sig: LONGINT);
VAR i:  WORD;
BEGIN
{
zeigMeldung ('einNetzString.LoeschCluster', StringVonDaten + atString (@SELF));
}
  INHERITED LoeschCluster (Sig);

  IF (SeitenVektor <> NIL) AND (AnzahlSeiten > 0) THEN BEGIN
    FOR i := 0 TO AnzahlSeiten - 1 DO BEGIN
      IF SeitenVektor^[i] <> NIL THEN BEGIN
        IF Sig = SeitenVektor^[i]^.meineSignatur THEN
          SeitenVektor^[i] := NIL
        ELSE SeitenVektor^[i]^.LoeschCluster (Sig);
      END (* IF <> NIL *);
    END (* FOR *);
  END (* IF SeitenVektor *);
END (* loeschCluster *);

FUNCTION einNetzString.LaengeIst: LONGINT;
BEGIN
  (* VORSICHT - das hat vorher AnzahlElemente ausgegeben... !!! *)
  IF (SeitenVektor <> NIL) THEN LaengeIst := AnzahlSeiten
  ELSE LaengeIst := 0;
END (* LaengeIst *);

PROCEDURE einNetzString.sende (CONST view: PBuffText; VAR ereignis: TEvent;
                               CONST Sig: LONGINT);
VAR  dataObjekt:  GLP;
BEGIN
  dataObjekt := liesAssoziation;
  IF (dataObjekt <> NIL) AND (dataObjekt^.meineSignatur <> Sig) THEN BEGIN
    meineSignatur := Sig;
    dataObjekt^.sende (view, ereignis, Sig);
  END
  ELSE BEGIN
    IF (ereignis.what <> evNothing) THEN
      INHERITED sende (view, ereignis, Sig);
  END (* ELSE *);
END (* sende *);

(******** berschriebene virtuelle Basis- und Vektormethoden ******)

FUNCTION einNetzString.meineAttribute: GLP;
(* ergibt Objektpointer *)
BEGIN
  meineAttribute := Data;
END (* meineAttribute *);

FUNCTION einNetzString.habeAttribute: BOOLEAN;
(* ergibt TRUE *)
BEGIN habeAttribute := TRUE;
END (* habeAttribute *);

PROCEDURE einNetzString.setAttribute (CONST a: GLP);
(* setzt ObjektPointer *)
BEGIN Data := a;
END (* setAttribute *);

FUNCTION einNetzString.RueckIst: GLP;
(* ergibt RueckPointer *)
BEGIN
  RueckIst := Rueck;
END (* RueckIst *);

FUNCTION einNetzString.getPointer (CONST index: LONGINT): GLP;
(* ergibt Seitenpointer *)
BEGIN
  IF (SeitenVektor <> NIL) AND (index < AnzahlSeiten) THEN BEGIN
    getPointer := SeitenVektor^[index];
  END
  ELSE getPointer := NIL;
END (* getPointer *);

(***************** berschriebene virtuelle Stringmethoden ***********)

FUNCTION einNetzString.gran (g: WORD): WORD;
(* Wir setzen hier den minimalen Speicherbedarf in den Vordergrund,
   mit einer Reorganisation der Netzstrings ist eher selten zu rechnen.
   Sollte sich dies als falsch erweisen, kann die berschreibung von
   gran entfallen oder gendert werden (siehe String).
*)
BEGIN
  gran := g; (* gran := einString8.gran (g); *)
END (* gran *);

PROCEDURE einNetzString.allocVektor (CONST anzahl);
BEGIN
{
zeigTrace ('>>>>>>>>> einNetzString.allocVektor', StringVonNum (Byte (anzahl)));
}
  IF BYTE (anzahl) > 0 THEN BEGIN
    INHERITED allocVektor (anzahl);
      (* Schreibt effektive Anzahl in AnzahlElemente... *)
    AnzahlSeiten := 1; (* Nur Vektor [0] *)
    GetMem (SeitenVektor, SizeOf (SeitenVektor^[0]));
    SeitenVektor^[0] := NIL;
  END (* IF anzahl > 0 *);
END (* allocVektor *);

PROCEDURE einNetzString.deallocVektor;
BEGIN
  INHERITED deallocVektor;
  IF SeitenVektor <> NIL THEN
    FreeMem (SeitenVektor, SizeOf (SeitenVektor^[0]) * AnzahlSeiten);
  SeitenVektor := NIL;
END (* deallocVektor *);

PROCEDURE einNetzString.resizeVektor (CONST anzahl);
VAR  nanzahl:        BYTE;
     h:              einVektorArray16Pointer;
     i:              BYTE;
BEGIN
  INHERITED resizeVektor (anzahl);

  nanzahl := AnzahlSeiten; (* Alte Gre Seitenvektor! *)
  IF nanzahl > 0 THEN BEGIN
    DEC (nanzahl);
    WHILE (SeitenVektor^[nanzahl] = NIL) DO BEGIN
      DEC (nanzahl);
    END (* WHILE *);
    INC (nanzahl);
  END (* IF *);

  (* Revision 13. 10. 91: resizeVektor reduziert Seitenvektor automatisch
     so weit, da NIL-Pointer am Ende wegfallen. Siehe neue Methode
     "assertVeksize". Dh. der Seitenvektor wird hier gar nicht fr
     eine ev. grere neue Lnge angelegt!!! *)

  IF nanzahl <> AnzahlSeiten THEN BEGIN
    (* Es gab NIL-Pointer am Ende *)
    GetMem (h, WORD (nanzahl * SizeOf (h^[0])));
    IF SeitenVektor <> NIL THEN BEGIN
      IF nanzahl > AnzahlSeiten THEN BEGIN
        Move (SeitenVektor^, h^, WORD (AnzahlSeiten * SizeOf (h^[0])));
        FOR i := AnzahlSeiten TO nanzahl - 1 DO BEGIN
          h^ [i] := NIL;
        END (* FOR *);
      END
      ELSE BEGIN
        Move (SeitenVektor^, h^, WORD (nanzahl * SizeOf (h^[0])));
      END;
      FreeMem (SeitenVektor, AnzahlSeiten * SizeOf (SeitenVektor^[0]));
    END
    ELSE BEGIN
      FOR i := 0 TO nanzahl - 1 DO BEGIN
        h^ [i] := NIL;
      END (* FOR *);
    END (* IF SeitenVektor <> NIL *);
    SeitenVektor := h; AnzahlSeiten := nanzahl;
  END (* IF nanzahl <> anz *);
END (* resizeVektor *);

PROCEDURE einNetzString.assertVekSize (CONST anzahl);
VAR  h:       einVektorArray16Pointer;
     i:       BYTE;
BEGIN
  IF BYTE (anzahl) > AnzahlSeiten THEN BEGIN
    GetMem (h, WORD (BYTE (anzahl) * SizeOf (h^[0])));
    IF SeitenVektor <> NIL THEN BEGIN
      Move (SeitenVektor^, h^, WORD (AnzahlSeiten * SizeOf (h^[0])));
      FOR i := AnzahlSeiten TO BYTE (anzahl) - 1 DO BEGIN
        h^[i] := NIL;
      END (* FOR *);
      FreeMem (SeitenVektor, AnzahlSeiten * SizeOf (SeitenVektor^[0]));
    END (* IF SeitenVektor <> NIL *)
    ELSE BEGIN
      FOR i := 0 TO BYTE (anzahl) - 1 DO BEGIN
        h^[i] := NIL;
      END (* FOR *);
    END (* IF NIL *);
    SeitenVektor := h; AnzahlSeiten := BYTE (anzahl);
  END (* IF grer machen *);
END (* assertVekSize *);

PROCEDURE einNetzString.holeString (VAR s);
BEGIN
  StringVonBegriff (STRING (s));
END (* holeString *);

FUNCTION einNetzString.Edit (CONST view: PBuffText; CONST x, y, b, h: WORD;
                             CONST name: GLP): GLP;
VAR   s:    STRING;
      p:    GLP;
      strp: StringPointer;
BEGIN
  s := StringVonDaten;
  Edit := EditName (name, @SELF, x + 1, y + 1, b, s);
  strp := @s;
  (* p := insertBegriff (strp, 0, TRUE, NIL, NIL, 0); *)
END (* Edit *);

(***************** Eigentliche Methoden von NetzString ***************)

PROCEDURE einNetzString.normalisiereBezeichner (CONST wort: StringPointer;
                                                VAR PSchreib: PBITMAP);
                          (* Wandelt alle Zeichen in dem String von begr
                             in Kleinbuchstaben um, erzeugt eine Bitmap fr
                             den String und setzt alle Bits fr Zeichen,
                             die ursprnglich grogeschrieben waren *)
VAR  i:  WORD;
     l:  LONGINT;
BEGIN
  IF (wort <> NIL){ AND (wort^.Laenge > 0)} THEN BEGIN
    WITH wort^ DO BEGIN
      l := Laenge; IF l > 0 THEN DEC (l);
      GetMem (PSchreib, (l) DIV 8 + 1);
      FillChar (PSchreib^, (l) DIV 8 + 1, 0);
{
zeigMeldung ('normalisiereBezeichner: ' + STRING (wort^),
             'vor der FOR-Schleife, Laenge:' + StringVonNum (Laenge));
}
      IF Laenge > 0 THEN
      FOR i := 0 TO Laenge - 1 DO BEGIN
        (* ACHTUNG: kleinVon wandelt Grobuchstaben in KleinBuchstaben
           um und liefert TRUE, wenn das Argument grogeschrieben war! *)
(*
zeigMeldung ('FOR i: ' + StringVonNum (i), 'Str [i] = ' + Str [i]);
*)
        (* Hier knnte man eine effizientere Prozedur schreiben,
           die den ganzen String und das Bitset gleichzeitig
           bearbeitet!!! Auch bei "denormalisiereBezeichner"! *)
        IF kleinVon (Str [i]) THEN SetBit (PSchreib, i);
      END (* FOR *);

{
zeigMeldung ('normalisiereBezeichner' + STRING (wort^), 'nach der FOR-Schleife');
}
    END (* WITH *);
  END (* IF *)
  ELSE BEGIN
    PSchreib := NIL;
  END (* ELSE *);
END (* normalisiereBezeichner *);

PROCEDURE einNetzString.denormalisiereBezeichner (CONST wort: StringPointer; PSchreib: PBITMAP);
                          (* Wandelt alle Zeichen in dem String von begr,
                             deren zugehriges Schreibungsbit gesetzt ist,
                             in Grobuchstaben um. *)
VAR  i:   WORD;
BEGIN
  (* Warnung: Hier sollte ein Pointer auf eine Kopie des Originalstrings
     bergeben werden, sonst wird dieser denormalisiert!
     Diese Prozedur lt den als normalisiert angenommenen String
     unangetastet, wenn der Schreibungspointer NIL ist - im Falle von
     ausschlielich kleingeschriebenen Bezeichnern knnte die Bitmap
     also "wegoptimiert" werden. *)
  IF (wort <> NIL) AND (PSchreib <> NIL) THEN BEGIN
    WITH wort^ DO BEGIN
      IF Laenge > 0 THEN
      FOR i := 0 TO Laenge - 1 DO BEGIN
        IF IsBit (PSchreib, i) THEN BEGIN
          grossVon (Str [i]);
        END (* ELSE *);
      END (* FOR *);
    END (* WITH *);
  END (* IF *);
END (* denormalisiereBezeichner *);

FUNCTION einNetzString.groessergeschrieben (CONST Test, Referenz: PBITMAP;
                                            CONST AnzZeichen: WORD): StringRelation;
  (* Liefert groesser, wenn Test z. B. mit einem Grobuch-
     staben beginnt, Referenz aber mit einem kleinen.
     Entscheidet nach dem ersten Grobuchstaben gegen-
     ber einem Kleinbuchstaben in (gleichlautenden)
     Bezeichnern. *)
VAR  ind:          WORD;
     rel:          StringRelation;
     TestB, RefB:  BOOLEAN;
BEGIN
  rel := gleich;
  IF (Test <> NIL) AND (Referenz <> NIL) THEN BEGIN
    ind := 0;
    TestB := isBit (Test, 0);
    RefB := isBit (Referenz, 0);
    WHILE (ind < AnzZeichen) AND (TestB = RefB)
    DO BEGIN
      INC (ind);
      IF ind <= AnzZeichen THEN BEGIN
        TestB := isBit (Test, ind);
        RefB := isBit (Referenz, ind);
      END (* IF *);
    END (* WHILE *);
    IF (ind < AnzZeichen) THEN BEGIN
      IF (TestB) AND NOT (RefB)
      THEN rel := groessergleich
      ELSE IF NOT (TestB) AND (RefB)
           THEN rel := kleinergleich;
    END (* IF ind < AnzZeichen *);
  END (* IF *);
  groessergeschrieben := rel;
END (* groessergeschrieben *);

FUNCTION einNetzString.ZeichenVergleich (CONST a, b: CHAR; CONST vorher: StringRelation): StringRelation;
TYPE   codeType = WORD (* LONGINT *);
CONST  cfaktor = 2;
       umklein = 1;
(*       klein =   2;
       umgross = 1;
       cnfaktor = 256;
*)

VAR  verg:                StringRelation;
     acod, bcod:          codeType;

  FUNCTION code (CONST ch: CHAR): codeType;
  BEGIN
    CASE ch OF
      '':  code := ORD ('a') * cfaktor + umklein;
      '':  code := ORD ('o') * cfaktor + umklein;
      '':  code := ORD ('u') * cfaktor + umklein;
      '':  code := ORD ('s') * cfaktor + umklein;
      ELSE BEGIN
        code := ORD (ch) * cfaktor;
      END (* ELSE *);
    END (* CASE *);
  END (* code *);

BEGIN (* ZeichenVergleich *)
  acod := code (a); bcod := code (b);
  IF acod = bcod THEN verg := gleich
  ELSE IF acod < bcod THEN verg := kleiner
  ELSE verg := groesser;
  IF vorher = gleich THEN BEGIN
    IF verg = kleiner THEN verg := kleinergleich
    ELSE IF verg = groesser THEN verg := groessergleich;
  END (* IF vorher *);
  ZeichenVergleich := verg;
END (* ZeichenVergleich *);

FUNCTION einNetzString.vergleiche (CONST wort: StringPointer;
                                   CONST ind: WORD; VAR gleichbis: WORD):
                                    StringRelation;
(* Vergleiche den Rest von wort ab ind mit String des Objektes. Der Teil
   des Wortes vor dem Index wird ignoriert, das Zeichen ab ind wird mit
   dem ersten Zeichen des eigenen Strings verglichen. *)
VAR    kleinste_gemeins_laenge: INTEGER;
       wort_laenge:             StringRelation;

       vergl_ind:               WORD;
       verg:                    StringRelation;
BEGIN
  (* WriteLn ('vergleiche ind: ', ind, ', gleichbis: ', gleichbis); *)
  gleichbis := 0; wort_laenge := undefiniert;
  IF (derString = NIL) OR (wort = NIL) THEN BEGIN
    vergleiche := undefiniert;
  END
  ELSE BEGIN
    kleinste_gemeins_laenge := derString^.Laenge;
    IF (wort^.Laenge - ind) < kleinste_gemeins_laenge THEN BEGIN
      wort_laenge := kleiner;
      kleinste_gemeins_laenge := wort^.Laenge - ind;
    END
    ELSE BEGIN
      IF (wort^.Laenge - ind) = kleinste_gemeins_laenge THEN
        wort_laenge := gleich
      ELSE wort_laenge := groesser;
    END (* wer ist krzer *);
    vergl_ind := 0;

    IF kleinste_gemeins_laenge < 1 THEN BEGIN
      (* einer der beiden Strings wird als leerer String betrachtet *)
      verg := wort_laenge;
    END
    ELSE BEGIN
      (* Zunchst versuchen wir, zu entscheiden, ob einer der beiden Strings
         eindeutig alphabetisch vor dem anderen kommt *)

      verg := ZeichenVergleich (wort^.Str [vergl_ind + ind],
                               derString^.Str [vergl_ind], undefiniert);

      (* Solange sich keine Entscheidung treffen lt... *)

      gleichbis := 1;

      WHILE (vergl_ind + 1 < kleinste_gemeins_laenge) AND (verg = gleich) DO BEGIN
        INC (vergl_ind);

        verg := ZeichenVergleich (wort^.Str [vergl_ind + ind],
                                  derString^.Str [vergl_ind], verg);
        IF verg = gleich THEN gleichbis := vergl_ind + 1;
      END (* WHILE *);
      IF (verg = gleich) THEN BEGIN
        CASE wort_laenge OF
          groesser: verg := groessergleich;
          kleiner: verg := kleinergleich;
          ELSE;
        END (* CASE *);
      END (* IF verg = gleich *);
    END (* IF kleinste_gemeins... *);
  END (* IF NIL *);
  vergleiche := verg;
END (* vergleiche *);

FUNCTION einNetzString.NewParam: ParamP;
VAR p: ParamP;
BEGIN
  NEW (p);
  WITH p^ DO BEGIN
    wort := NIL; schreibExakt := TRUE; neu := TRUE;
    PSchreib := NIL;
  END (* WITH *);
  NewParam := p;
END (* NewParam *);

FUNCTION einNetzString.sucheBegriff (CONST Param: ParamP;
                                     CONST ind: WORD;
                                     VAR Absender: GLP;
                                     VAR ElemIndex: LONGINT): GLP;
(* NIL, wenn unbekannt, sonst WortHandle *)
VAR   verg:          StringRelation;
      gleichbis:     WORD;
BEGIN
{
zeigMeldung ('einNetzString (' + StringVonTyp + ').sucheBegriff',
             'in NetzString nicht implementiert!');
}
  sucheBegriff := NIL; INC (ElemIndex);
IF Absender = @SELF THEN BEGIN
zeigMeldung ('einNetzString.sucheBegriff', 'Absender = @SELF');
EXIT;
END;

{ Das folgende ist aus "insertBegriff" bernommen. Wenn die Schreibung
  (Gro/Klein) berprft werden soll, mu schreibExakt = TRUE sein}
  WITH Param^ DO BEGIN
    verg := vergleiche (wort, ind, gleichbis);

    IF (verg = gleich) AND schreibExakt THEN
      verg := groessergeschrieben (PSchreib, Schreibung, AnzahlElemente);

    CASE verg OF
      gleich: sucheBegriff := @SELF;
      groessergleich: (* Der Wortrest ist eine Seitenkette des eigenen Str *)
                   BEGIN
                     IF (SeitenVektor <> NIL) AND (AnzahlSeiten >= gleichbis)
                        AND (SeitenVektor^[gleichbis] <> NIL) THEN BEGIN
                       Absender := @SELF;
                       sucheBegriff :=
                         einNetzStringPointer (SeitenVektor^[gleichbis])^.
                           sucheBegriff (Param, ind, Absender, ElemIndex);
                     END;
                   END;
      groesser:      (* Der Auftrag wird weitergereicht *)
                   BEGIN
                     IF (SeitenVektor <> NIL) AND (SeitenVektor^[0] <> NIL) THEN BEGIN
                       Absender := @SELF;
                       sucheBegriff :=
                         einNetzStringPointer (SeitenVektor^[0])^.
                         sucheBegriff (Param, ind, Absender, ElemIndex);
                     END (* IF *);
                   END;
      ELSE         BEGIN zeigMeldung ('sucheBegriff', 'Fehler in vergleiche');
                   END;
    END (* CASE *);
  END (* WITH Param *);
END (* sucheBegriff *);

FUNCTION einNetzString.insertReststring (CONST wort: StringPointer; CONST ind: WORD): GLP;
(* Weist die Teilkette ab ind einfach dem String8 zu... *)
VAR      laen:      BYTE;
BEGIN
  laen := wort^.Laenge - ind;
  resizeVektor (laen);
  Move (wort^.Str[ind], derString^.Str[0], WORD (laen));
  derString^.Laenge := laen;
  insertRestString := @SELF;
END (* insertReststring *);

(************************* Funktionsgruppe InsertBegriff *****************)

FUNCTION einNetzString.doInsertkleiner (CONST Param: ParamP;
                                        CONST ind: WORD;
                                        CONST absender, rueckverweis: GLP;
                                        CONST absind: WORD;
                                        CONST gleichbis: WORD): GLP;
BEGIN
{
schreibProtokoll ('doInsertkleiner ' + StringVonDaten, 'Wort: ' + STRING (Param^.wort^) +
', Stack: ' + StringVonNum (SPtr));
}
  IF absind = 0 THEN BEGIN
    doInsertkleiner := einNetzStringPointer (absender)^.insertSeitenkette
                           (Param, @SELF, NIL, rueckverweis, absind, ind);
  END
  ELSE BEGIN
    doInsertkleiner := einNetzStringPointer (absender)^.insertSeitenkette
                        (Param, @SELF, absender, rueckverweis, absind,
                         {wort, PSchreib,} ind {,neu});
  END (* IF absind = 0 *);
END (* doInsertkleiner *);

FUNCTION einNetzString.doInsertkleinergleich (CONST Param: ParamP;
                                              CONST ind: WORD;
                                              CONST absender, rueckverweis: GLP;
                                              CONST absind: WORD;
                                              CONST gleichbis: WORD): GLP;
VAR  oldindex:    WORD;
     handle:      GLP;
BEGIN
{
schreibProtokoll ('doInsertkleinergleich ' + StringVonDaten, 'Wort: ' + STRING (Param^.wort^) +
', Stack: ' + StringVonNum (SPtr));
}
  oldindex := Offset;
  handle := spalteWort (Param, gleichbis, absender, rueckverweis, absind);
  Rueck := handle;
  doInsertkleinergleich := handle;
END (* doInsertkleiner *);

FUNCTION einNetzString.doInsertgleich (CONST Param: ParamP;
                                       CONST ind: WORD;
                                       CONST absender, rueckverweis: GLP;
                                       {CONST absind: WORD;}
                                       CONST gleichbis: WORD): GLP;
BEGIN
{
schreibProtokoll ('doInsertgleich ' + StringVonDaten, 'Wort: ' + STRING (Param^.wort^) +
', Stack: ' + StringVonNum (SPtr));
}
  IF Param^.neu THEN doInsertgleich := NIL
  ELSE doInsertgleich := @SELF;
END (* doInsertgleich *);

VAR  assertSize:   BYTE;
VAR   verg:          StringRelation;
      oldglch, glchbis,
      gleichbis:     WORD;
      id{, oldid}:   WORD;
      SkipObj:       einNetzStringPointer;
      OldSkip:       GLP;

FUNCTION einNetzString.doInsertgroessergleich (CONST Param: ParamP;
                                               CONST ind: WORD;
                                               CONST gleichbis: WORD): GLP;
{ - achtung, benutzt globale Variable, um Stack zu sparen -
VAR  assertSize:     BYTE;
VAR   verg:          StringRelation;
      oldglch,
      glchbis:       WORD;
      id:            WORD;
      SkipObj:       einNetzStringPointer;
      OldSkip:       GLP;
}
BEGIN
  (* Fr rueckverweis wird @SELF angegeben *)
{
schreibProtokoll ('doInsertgroessergleich ' + StringVonDaten, 'Wort: ' + STRING (Param^.wort^) +
', Stack: ' + StringVonNum (SPtr));
}
  assertSize := gleichbis + 1;
  assertVekSize (assertSize);

  IF SeitenVektor^[gleichbis] = NIL THEN BEGIN
    SeitenVektor^[gleichbis] := insertSeitenkette (Param,
                           SeitenVektor^[gleichbis],
                           @SELF, @SELF, gleichbis,
                           gleichbis + ind);
    doInsertgroessergleich := SeitenVektor^[gleichbis];
  END
  ELSE BEGIN
{
    doInsertgroessergleich := einNetzStringPointer (SeitenVektor^[gleichbis])^.
                       insertBegriff (Param, gleichbis + ind,
                       @SELF, @SELF, gleichbis);
}
{}
(*zeigMeldung ('doInsertgroessergleich', 'wort: ' + STRING (Param^.wort^));*)
    SkipObj := einNetzStringPointer (SeitenVektor^[gleichbis]);
    OldSkip := @SELF; id := ind + gleichbis;
    oldglch := gleichbis;
    WITH Param^ DO BEGIN
      verg := SkipObj^.vergleiche (wort, id, glchbis);
      IF verg = gleich THEN
        verg := SkipObj^.groessergeschrieben (PSchreib, SkipObj^.Schreibung, SkipObj^.AnzahlElemente);
      assertSize := glchbis + 1;
      SkipObj^.assertVekSize (assertSize);
{
schreibProtokoll ('doInsertgroessergleich vor WHILE ' + SkipObj^.StringVonDaten,
'verg: ' + StringVonNum (ORD (verg)) +
', ind: ' + StringVonNum (ind) + ', glchbis: ' + StringVonNum (glchbis));
}
      WHILE ((verg = groessergleich) OR (verg = gleich)) AND
            (SkipObj^.SeitenVektor <> NIL) AND
            (SkipObj^.SeitenVektor^[glchbis] <> NIL) DO BEGIN
(*
zeigMeldung ('WHILE, glchbis: ' + StringVonNum (glchbis) + ', id: ' + StringVonNum (id),
'wort: ' + STRING (wort^) + ', Obj: ' + SkipObj^.StringVonDaten);
*)
        OldSkip := SkipObj; id := id + glchbis;
        SkipObj := einNetzStringPointer (SkipObj^.SeitenVektor^[glchbis]);
        oldglch := glchbis;
        verg := SkipObj^.vergleiche (wort, id, glchbis);
        IF verg = gleich THEN
          verg := SkipObj^.groessergeschrieben (PSchreib, SkipObj^.Schreibung, SkipObj^.AnzahlElemente);
        assertSize := glchbis + 1;
        SkipObj^.assertVekSize (assertSize);
{
schreibProtokoll ('doInsertgroessergleich END WHILE ' + SkipObj^.StringVonDaten,
'verg: ' + StringVonNum (ORD (verg)) +
', ind: ' + StringVonNum (ind) + ', glchbis: ' + StringVonNum (glchbis));
}
      END (* WHILE *);
    END (* WITH *);
(*
      IF verg = gleich THEN
        verg := groessergeschrieben (PSchreib, Schreibung, AnzahlElemente);
*)
      CASE verg OF
        kleiner:       (* Nachricht an Absender zurck... *)
                     BEGIN
                       doinsertgroessergleich :=
                         SkipObj^.doInsertkleiner (Param, id,
                                   OldSkip, OldSkip, oldglch, glchbis);
                     END;
        kleinergleich: (* Das Netz mu reorganisiert, der eigene String
                          aufgespalten werden *)
                     BEGIN
(*
zeigMeldung ('kleinergleich, whiles: ' + StringVonNum (whiles),
'wort: ' + STRING (Param^.wort^));
*)
                       doinsertgroessergleich :=
                         SkipObj^.doInsertkleinergleich (Param, id,
                                   OldSkip, OldSkip, oldglch, glchbis);
                     END;
        ELSE
(*
zeigMeldung ('groesser/gleich, whiles: ' + StringVonNum (whiles),
'wort: ' + STRING (Param^.wort^));
*)
          IF (SkipObj^.SeitenVektor^[glchbis] = NIL) AND
           ((verg = groessergleich) {OR (verg = gleich)}) THEN BEGIN
            SkipObj^.SeitenVektor^[glchbis] :=
              SkipObj^.insertSeitenkette (Param,
                                   SkipObj^.SeitenVektor^[glchbis],
                                   SkipObj, SkipObj, glchbis,
                                   glchbis + id);
            doInsertgroessergleich := SkipObj^.SeitenVektor^[glchbis];
          END
          ELSE
            doInsertgroessergleich :=
               SkipObj^.insertBegriff (Param, id, OldSkip, OldSkip, glchbis);
    END (* CASE *);
{}
  END (* ELSE *);
END (* doInsertgroessergleich *);

CONST  assertEins:   BYTE = 1;

FUNCTION einNetzString.doInsertgroesser (CONST Param: ParamP;
                                         CONST ind: WORD;
                                         CONST absender, rueckverweis: GLP;
                                         CONST gleichbis: WORD): GLP;
{ - achtung, benutzt globale Variable, um Stack zu sparen -
CONST  assertEins:   BYTE = 1;
VAR   verg:          StringRelation;
      oldglch,
      glchbis:       WORD;
      id:            WORD;
      SkipObj:       einNetzStringPointer;
      OldSkip:       GLP;
}
BEGIN
{
schreibProtokoll ('doInsertgroesser ' + StringVonDaten, 'Wort: ' + STRING (Param^.wort^) +
', Stack: ' + StringVonNum (SPtr));
}
  assertVekSize (assertEins);

  IF SeitenVektor^[0] = NIL THEN BEGIN
    IF ind = 0 THEN BEGIN
      SeitenVektor^[0] := insertSeitenkette
                           (Param, NIL, NIL, rueckverweis, 0, ind);
    END
    ELSE BEGIN
      SeitenVektor^[0] := insertSeitenkette
                           (Param, NIL, absender, rueckverweis, 0, ind);
    END;
    doInsertgroesser := SeitenVektor^[0];
  END
  ELSE BEGIN
    IF ind = 0 THEN BEGIN
    (* Fr Rckverweis = NIL, denn bei ind = 0 gibt
       es keinen Rckverweis (Wortwurzel). *)
{
      doInsertgroesser := einNetzStringPointer (SeitenVektor^[0])^.
        insertBegriff (Param, ind, @SELF, NIL, 0);
    END
    ELSE BEGIN
      doInsertgroesser := einNetzStringPointer (SeitenVektor^[0])^.
        insertBegriff (Param, ind, @SELF, rueckverweis, 0);
    END (* ind <> 0 *);
 END (* IF SeitenVektor *);
}
{}
      SkipObj := einNetzStringPointer (SeitenVektor^[0]);
      OldSkip := @SELF;
      WITH Param^ DO BEGIN

        verg := SkipObj^.vergleiche (wort, ind, glchbis);
{
schreibProtokoll ('doInsertgroesser vor WHILE ' + SkipObj^.StringVonDaten,
'verg: ' + StringVonNum (ORD (verg)) +
', ind: ' + StringVonNum (ind) + ', glchbis: ' + StringVonNum (glchbis));
}
        WHILE (verg = groesser) AND (SkipObj^.SeitenVektor <> NIL) AND
              (SkipObj^.SeitenVektor^[0] <> NIL) DO BEGIN
(*
zeigMeldung ('doInsertgroesser', 'Wort: ' + STRING (wort^) + ', SELF: ' + StringVonDaten +
', Seitenvektor^[0] = ' + atString (SkipObj^.SeitenVektor^[0]) +
', "' + SkipObj^.SeitenVektor^[0]^.StringVonDaten + '"');
*)
          OldSkip := SkipObj;
          SkipObj := einNetzStringPointer (SkipObj^.SeitenVektor^[0]);
          verg := SkipObj^.vergleiche (wort, ind, glchbis);
{
schreibProtokoll ('doInsertgroesser END WHILE ' + SkipObj^.StringVonDaten,
'verg: ' + StringVonNum (ORD (verg)) +
', ind: ' + StringVonNum (ind) + ', glchbis: ' + StringVonNum (glchbis));
}
        END (* WHILE *);

      END (* WITH *);

      IF (SkipObj^.SeitenVektor <> NIL) AND
         (SkipObj^.SeitenVektor^[0] = NIL) AND (verg = groesser) THEN BEGIN
        SkipObj^.SeitenVektor^[0] := SkipObj^.insertSeitenkette (Param, NIL, NIL,
          rueckverweis, 0, ind);
        doInsertgroesser := SkipObj^.SeitenVektor^[0];
      END
      ELSE
        doInsertgroesser := SkipObj^.insertBegriff (Param, ind, OldSkip, NIL, 0);
      END
    ELSE BEGIN

      SkipObj := einNetzStringPointer (SeitenVektor^[0]);
      OldSkip := @SELF;
      WITH Param^ DO BEGIN
        verg := SkipObj^.vergleiche (wort, ind, glchbis);
{
schreibProtokoll ('doInsertgroesser vor WHILE ' + SkipObj^.StringVonDaten,
'verg: ' + StringVonNum (ORD (verg)) +
', ind: ' + StringVonNum (ind) + ', glchbis: ' + StringVonNum (glchbis));
}
        WHILE (verg = groesser) AND (SkipObj^.SeitenVektor <> NIL) AND
              (SkipObj^.SeitenVektor^[0] <> NIL) DO BEGIN
          OldSkip := SkipObj;
          SkipObj := einNetzStringPointer (SkipObj^.SeitenVektor^[0]);
          verg := SkipObj^.vergleiche (wort, ind, glchbis);
{
schreibProtokoll ('doInsertgroesser END WHILE ' + SkipObj^.StringVonDaten, 'verg: ' + StringVonNum (ORD (verg)) +
', ind: ' + StringVonNum (ind) + ', glchbis: ' + StringVonNum (glchbis));
}
        END (* WHILE *);
      END (* WITH *);

      IF (SkipObj^.SeitenVektor <> NIL) AND
         (SkipObj^.SeitenVektor^[0] = NIL) AND (verg = groesser) THEN BEGIN
        SkipObj^.SeitenVektor^[0] := SkipObj^.insertSeitenkette (Param, NIL, OldSkip,
          rueckverweis, 0, ind);
        doInsertgroesser := SkipObj^.SeitenVektor^[0];
      END
      ELSE
      doInsertgroesser := SkipObj^.insertBegriff (Param, ind, OldSkip,
                                                  rueckverweis, 0);

    END (* if ind = 0 *);
  END (* IF SeitenVektor *);
{}
END (* doInsertgroesser *);

FUNCTION einNetzString.insertBegriff (CONST Param: ParamP;
                                      CONST ind: WORD;
                                      CONST absender, rueckverweis: GLP;
                                      CONST absind: WORD): GLP;
(* String eintragen und Handle zurckgeben, NIL
   wenn schon vorhanden UND neu=TRUE, sonst
   vorhandenes Handle zurckgeben *)
{VAR   verg:          StringRelation;
      gleichbis:     WORD;}
BEGIN (* insertBegriff *)
{
schreibProtokoll ('insertBegriff ' + StringVonDaten, 'Wort: ' + STRING (Param^.wort^) +
', Stack: ' + StringVonNum (SPtr));
}
  (* Wenn schon eine Wortbasis vorhanden ist, mu zunchst die richtige
     Einfgestelle gefunden werden.
     Wenn das aktuelle Objekt einen String besitzt, der alphabetisch vor
     dem neuen String liegt, ist die Nachricht weiterzureichen.
     Prfe, ob Anfang des Strings vorhanden. Wenn nicht, ob es Seiten-
     pointer gibt, dann Auftrag an diesen weiterreichen, wenn alphabetisch
     noch vor dem String, sonst neues Objekt erzeugen und diesem den
     Auftrag geben.
     Wenn Objekt selbst neu ist, String alloziieren.
  *)
  WITH Param^ DO BEGIN
    IF derString = NIL THEN BEGIN
      (* Rest oder ganzer String ab Index mu neu eingetragen werden *)
      insertBegriff := insertReststring (wort, ind);
      Offset := ind; Rueck := rueckverweis;
      (* zeigMeldung ('insertBegriff', 'Schreibung := PSchreib'); *)
      Schreibung := PSchreib;
    END
    ELSE BEGIN
    (*
      Wir mssen feststellen, ob der einzutragende (Teil-)String lexikalisch
      nach oder vor dem eigenen String steht oder ob er teilweise mit
      diesem gleich ist, dh. sein Rest eine Teilkette bildet.
    *)
      verg := vergleiche (wort, ind, gleichbis);

    (* Wenn der Begriff gleichlautend mit einem schon eingetragenen ist,
       mssen wir prfen, ob er sich in der Gro-/Kleinschreibung unter-
       scheidet. Solche Begriffe werden als "kleinergleich" eingetragen,
       wenn sie kleingeschrieben sind und der vorhandene gro, bzw. als
       "groessergleich" im umgekehrten Fall. *)

      IF verg = gleich THEN
        verg := groessergeschrieben (PSchreib, Schreibung, AnzahlElemente);

      CASE verg OF
        kleiner:       (* Nachricht an Absender zurck... *)
                     BEGIN
                       insertBegriff :=
                         doInsertkleiner (Param, ind,
                                   absender, rueckverweis, absind, gleichbis);
                     END;
        kleinergleich: (* Das Netz mu reorganisiert, der eigene String
                          aufgespalten werden *)
                     BEGIN
                       insertBegriff :=
                         doInsertkleinergleich (Param, ind,
                                   absender, rueckverweis, absind, gleichbis);
                     END;
        gleich: (* Das Wort ist schon eingetragen.
                   Hier mu aber nun auch der Fall behandelt werden, da
                   zwar der buchstabenmige Wortlaut des Begriffes mit dem
                   vorhandenen bereinstimmt, aber die Gro-Kleinschreibung
                   abweicht.  *)
                     BEGIN
                       insertBegriff :=
                         doInsertgleich (Param, ind, absender, rueckverweis,
                                         gleichbis);
                     END;
        groessergleich: (* Der Wortrest ist eine Seitenkette des eigenen Str *)
                     BEGIN
                       insertBegriff :=
                         doInsertgroessergleich (Param, ind, {absind,} gleichbis);
                     END;
        groesser:      (* Der Auftrag wird weitergereicht *)
                     BEGIN
                       insertBegriff :=
                         doInsertgroesser (Param, ind,
                                    absender, rueckverweis, gleichbis);
                     END;
        ELSE         BEGIN zeigMeldung ('insertBegriff', 'Fehler in vergleiche');
                     END;
      END (* CASE *);
    END (* IF derString = NIL *);
  END (* WITH Param *);
END (* insertBegriff *);

FUNCTION einNetzString.insertSeitenkette (CONST Param: ParamP;
                                          CONST naechster: GLP;
                                          CONST absender,
                                          rueckverweis: GLP;
                                          CONST absind: WORD;
                                          CONST ind: WORD): GLP;
(* Diese Methode bildet den unmittelbaren Auftrag, ein neues Objekt am
   Anfang der Seitenkette einzufgen (oder eine solche neu anzufangen).
   Sie wird entweder von insertBegriff an das Objekt selbst erteilt, wenn
   der Seitenpointer NIL ist, oder er kommt vom ersten/nchsten Objekt
   der Seitenkette an den Absender von insertBegriff zurck, wenn der
   Empfnger feststellt, da die einzufgende Seitenkette alphabetisch
   vor seiner eigenen Kette liegt. *)
VAR  assertSize:  BYTE;
BEGIN
{
schreibProtokoll ('insertSeitenkette ' + StringVonDaten, 'Wort: ' + STRING (Param^.wort^) +
', Stack: ' + StringVonNum (SPtr));
}
  assertSize := absind + 1;
  assertVekSize (assertSize);

  SeitenVektor^[absind] := neuerNetzString;
  insertSeitenkette :=
    einNetzStringPointer (SeitenVektor^[absind])^.insertBegriff
      (Param, ind, absender, rueckverweis, 0);

  assertSize := 1;
  einNetzStringPointer (SeitenVektor^[absind])^.assertVekSize (assertSize);

  einNetzStringPointer (SeitenVektor^[absind])^.SeitenVektor^ [0] :=
    naechster;
END (* insertSeitenkette *);

FUNCTION einNetzString.spalteWort (CONST Param: ParamP;
                                   CONST gleichbis: WORD;
                                   CONST absender, rueckverweis: GLP;
                                   CONST absind: WORD): GLP;
(* Diese Methode behandelt den Sonderfall, da das neue Wort mit dem
   eigenen Wortstamm bereinstimmt, sein Rest aber alphabetisch vor dem
   eigenen Restwort einzuordnen ist. Dann mu das Wortnetz reorganisert
   werden, nachdem das ursprngliche Wort aufgespalten worden ist.
   Da der Pointer auf den bereits gespeicherten Begriff gltig bleiben
   mu, legen wir rckwrts ein neues Objekt mit dem gemeinsamen
   Stamm an. *)
VAR   {dasneueWort:      StringPointer;}
      neueLaenge:       BYTE;
      nanzahl:          BYTE;
      verkuerzung:      BYTE;
      fori:             BYTE;
      handle:           GLP;
      i:                BYTE;
      h:                einVektorArray16Pointer;
      NeuParam:         ParamP;
      (* stri:             STRING; *)
BEGIN (* spalteWort *)
{
schreibProtokoll ('spalteWort ' + StringVonDaten, 'Wort: ' + STRING (Param^.wort^) +
', Stack: ' + StringVonNum (SPtr));
}
  (* Zunchst mssen wir den Absender auffordern, ein neues Wort mit dem
     Wortstamm einzufgen, dieses Wort wird zunchst zusammengesetzt: *)
  NeuParam := NewParam;
  NeuParam^ := Param^;
  WITH NeuParam^ DO BEGIN
    neueLaenge := wort^.Laenge - Offset;
    nanzahl := gran (neueLaenge + SizeOf (wort^.Laenge));
{
zeigMeldung ('spalteWort', 'neueLaenge: ' + StringVonNum (neueLaenge) +
                           ', nanzahl: ' + StringVonNum (nanzahl) +
                           ', gleichbis: ' + StringVonNum (gleichbis));
}
    GetMem (wort {dasneueWort}, WORD (nanzahl));
    {dasneue}Wort^.Laenge := neueLaenge;
    Move (derString^.Str [0], {dasneue}Wort^.Str [0], WORD (gleichbis));
{
zeigMeldung ('Move Laenge - Offset - gleichbis',
             StringVonNum (WORD (Param^.wort^.Laenge - Offset - gleichbis)));
}
    Move (Param^.wort^.Str [gleichbis + Offset], {dasneue}Wort^.Str [gleichbis],
          WORD (Param^.wort^.Laenge - Offset - gleichbis));

    IF absind = 0 THEN BEGIN
      handle := einNetzStringPointer (absender)^.insertSeitenkette (NeuParam,
                  @SELF, NIL, rueckverweis, absind, {dasneueWort, PSchreib,}
                  0{, neu});
    END
    ELSE BEGIN
      handle := einNetzStringPointer (absender)^.insertSeitenkette (NeuParam,
                 @SELF, absender, rueckverweis, absind, {dasneueWort, PSchreib,}
                 0{, neu});
    END (* IF absind = 0 *);
    einNetzStringPointer (handle)^.assertVekSize (nanzahl);

    FreeMem ({dasneue}Wort, WORD (nanzahl));

      (* Bei aufgespaltenem Begriff ist ind immer Null, Offset mu daher
         korrigiert werden: *)
    einNetzStringPointer (handle)^.Offset := (*absind*) Offset;

    (* Nun sind die Seitenzeiger zu bertragen... *)

    (* ... dies sind die Pointer auf Nebenketten in dem Wortstamm des ur-
       sprnglichen Wortes, der jetzt von dem neuen Objekt handle gebildet
       wird. In der Schleife sollen, auer i = 0, auch Rcklinks umgesetzt
       und weitergegeben werden. *)

    IF (AnzahlSeiten > 0) THEN BEGIN
      IF gleichbis >= AnzahlSeiten THEN fori := AnzahlSeiten - 1
      ELSE fori := gleichbis;

      FOR i := 0 TO fori DO BEGIN
  (*
  Str (i, stri);
  zeigMeldung ('spalteWort', 'verbindeSeitenkette ' + stri);
  *)
        einNetzStringPointer (handle)^.verbindeSeitenkette (i, SeitenVektor^[i],
       (i <> 0), TRUE);
      END (* FOR *);
    END (* IF AnzahlSeiten > 0 *)
    ELSE BEGIN
      einNetzStringPointer (handle)^.verbindeSeitenkette (0, NIL, FALSE, TRUE);
    END;

  (*
  zeigMeldung ('verbindeSeitenkette', 'Ende');
  *)
    (* Damit ist das Netz reorganisiert, es mu noch die Umhngung des
       eigenen Objektes erfolgen. Das heit, der letzte Pointer im
       Seitenvektor des neu angelegten Wortstammes wird mit dem alten
       (Rest)-Wort verbunden. Das Rcklink soll gesetzt, aber nicht
       weitergegeben werden! *)
    einNetzStringPointer (handle)^.verbindeSeitenkette (gleichbis, @SELF,
                                                        TRUE, FALSE);

    (* und zuletzt die Verkrzung des eigenen Strings auf den verbleibenden
       Rest. Zuerst konstruieren wir den neuen String: *)
    neueLaenge := derString^.Laenge - gleichbis;
    nanzahl := gran (neueLaenge + SizeOf (derString^.Laenge));
    GetMem ({dasneue}Wort, WORD (nanzahl));
    {dasneue}Wort^.Laenge := neueLaenge;
    Move (derString^.Str[gleichbis], {dasneue}Wort^.Str[0], WORD (neueLaenge));

    (* ...dann mu dieser gesetzt werden und der Rest des Seitenvektors
       mu ebenfalls verkrzt kopiert werden *)

    FreeMem (derString, AnzahlElemente);
    derString := {dasneue}Wort;
{
zeigMeldung ('spalteWort',
STRING (derString^));
}
    AnzahlElemente := nanzahl; (* StringElemente!!!!! *)
    (*
    ursprngliches Wort:
    |H|a|n|s|d|a|m|p|f|

    neues Wort:
    |H|a|n|s|   (gleichbis = 4)

    verbleibender Rest:
            |d|a|m|p|f|
    *)

    IF AnzahlSeiten > gleichbis THEN BEGIN
      nanzahl := AnzahlSeiten - gleichbis;
      GetMem (h, WORD (nanzahl * SizeOf (h^[0])));

      Move (SeitenVektor^[gleichbis], h^[0],
          WORD ((nanzahl) * SizeOf (h^[0])));
    END
    ELSE BEGIN
      h := NIL; nanzahl := 0;
    END;

    FreeMem (SeitenVektor, AnzahlSeiten * SizeOf (SeitenVektor^[0]));
    SeitenVektor := h; AnzahlSeiten := nanzahl;
    Offset := Offset + gleichbis;
    spalteWort := handle;
  END (* WITH Param^ *);
{
zeigMeldung ('spalteWort',
STRING (derString^));
zeigMeldung ('spalteWort', StringVonDaten);
}
END (* spalteWort *);

PROCEDURE einNetzString.verbindeSeitenkette (CONST ind: WORD;
                                             CONST ziel: GLP;
                                             CONST ruecklink, weitergeben: BOOLEAN);
VAR  assertSize:  BYTE;
BEGIN
  IF (SeitenVektor <> NIL) AND (ind < AnzahlSeiten) THEN BEGIN
    assertSize := ind + 1;
    assertVekSize (assertSize);
    SeitenVektor^[ind] := ziel;
    IF ruecklink AND (ziel <> NIL) THEN BEGIN
      einNetzStringPointer (SeitenVektor^[ind])^.setzeRueck (@SELF, weitergeben);
    END (* IF ziel *);
  END (* IF *);
END (* verbindeSeitenkette *);

PROCEDURE einNetzString.setzeRueck (CONST ziel: GLP; CONST weitergeben: BOOLEAN);
BEGIN
  Rueck := ziel;
  IF (weitergeben) AND (SeitenVektor <> NIL) AND (SeitenVektor^[0] <> NIL) THEN
    einNetzStringPointer (SeitenVektor^[0])^.setzeRueck (ziel, TRUE);
END (* setzeRueck *);

FUNCTION einNetzString.loescheBegriff (CONST abs: GLP): GLP;
(* Begriff austragen und ev. Data zurckliefern *)
(* Diese Prozedur soll das Netz so reorganisieren, da das Objekt gelscht
   werden kann. Dazu mssen die Seitenzeiger berprft werden. Gibt es andere
   Begriffe, die mit diesem gemeinsame Wortstmme aufweisen, mu derjenige
   Begriff ermittelt werden, der diesen ersetzen kann. *)
VAR  i, ii:      WORD;
     (* vorgaenger: einNetzStringPointer; *)
     naechster:  einNetzStringPointer;
     absoff:     BYTE;
     assertSize: BYTE;
     dummyg:     GLP;
BEGIN
(*
zeigMeldung ('loescheBegriff', 'Vorgnger: ' + abs^.StringVonDaten +
             ', Begr: ' + StringVonDaten);
*)
  absoff := einNetzStringPointer (abs)^.Offset;

  IF SeitenVektor <> NIL THEN BEGIN
    (* Stelle fest, ob es Seitenketten zu dem Begriff gibt und ermittle
       denjenigen, der lexikalisch der nchste ist (und damit auch den lngsten
       gemeinsamen Wortstamm hat) *)
    IF (AnzahlSeiten > 1) THEN BEGIN
      i := AnzahlSeiten - 1;
      WHILE (i > 1) AND (SeitenVektor^[i] = NIL) DO BEGIN
        DEC (i);
      END (* WHILE *);
(*
zeigMeldung ('loescheBegriff', 'Seitenketten gesucht');
*)
      GLP (naechster) := SeitenVektor^[i];
    END
    ELSE naechster := NIL;

    IF naechster <> NIL THEN BEGIN
    (* Es gibt einen Begriff, der diesen ersetzen mu. Zunchst mssen in dem
       Begriff "naechster" String und Seitenvektor um den fehlenden gemeinsamen
       Wortstamm ergnzt werden. *)
      naechster^.wortZusammenlegen (derString, i, @SELF);
(*
zeigMeldung ('loescheBegriff', 'Wort zusammengelegt');
*)
      einNetzStringPointer (abs)^.SeitenVektor^[Offset - absoff] := naechster;
    END (* naechster <> NIL *)
    ELSE BEGIN
(*
zeigMeldung ('loescheBegriff', 'Begriff aushngen:');
*)
  (* Nun mu der Begriff selbst ausgehngt werden, dazu wird der Vorgnger
     bentigt, der in "abs" steht: *)
       assertSize := Offset - absoff + 1;
       einNetzStringPointer (abs)^.assertVekSize (assertSize);
       einNetzStringPointer (abs)^.SeitenVektor^[Offset - absoff] :=
         SeitenVektor^[0];
    END (* ELSE *);
  END (* IF SeitenVektor <> NIL *)
  ELSE BEGIN
(*
zeigMeldung ('loescheBegriff', 'Begriff aushngen:');
*)
  (* Nun mu der Begriff selbst ausgehngt werden, dazu wird der Vorgnger
     bentigt, der in "abs" steht: *)
    assertSize := Offset - absoff + 1;
    einNetzStringPointer (abs)^.assertVekSize (assertSize);
    einNetzStringPointer (abs)^.SeitenVektor^[Offset - absoff] := NIL;
  END (* ELSE *);

  (* Ich glaube, das ist Vorbereitung auf Disposen - damit keine
     Seiteneffekte durch Weitergabe von Done entstehen, werden alle
     Verkettungen gelscht: *)
  IF AnzahlSeiten > 0 THEN
    FOR i := 0 TO AnzahlSeiten - 1 DO BEGIN
      SeitenVektor^[i] := NIL;
    END (* FOR *);
  Rueck := NIL;

  loescheBegriff := @SELF;
END (* loescheBegriff *);

PROCEDURE einNetzString.wortZusammenlegen (CONST wort: StringPointer;
                                           CONST ind: WORD; CONST abs: GLP);
VAR dasneueWort:     StringPointer;
    neueLaenge:      BYTE;
    nanzahl:         BYTE;
    nAnzahlSeiten:   BYTE;
    i:               WORD;
    h:               einVektorArray16Pointer;
BEGIN
(*
zeigMeldung ('wortZusammenlegen', STRING (wort^));
*)
  (* Zunchst werden der Wortstamm des Absenders und der eigene String
     zum neuem String "dasneueWort" zusammengelegt *)
  neueLaenge := StringLaenge + ind;
  nanzahl := gran (neueLaenge + SizeOf (wort^.Laenge));
  GetMem (dasneueWort, WORD (nanzahl));
  dasneueWort^.Laenge := neueLaenge;

  Move (wort^.Str [0], dasneueWort^.Str [0], ind);
  Move (derString^.Str [0], dasneueWort^.Str [ind], derString^.Laenge);

  FreeMem (derString, AnzahlElemente);
  derString := dasneueWort;

  (*******************************************************************)
  (* Jetzt ist ein neuer Seitenvektor anzulegen und die Werte aus den
     beiden alten zu kopieren *)

  nAnzahlSeiten := einNetzStringPointer (abs)^.AnzahlElemente + AnzahlSeiten;

  GetMem (h, WORD (nAnzahlSeiten * SizeOf (h^[0])));

  Move (einNetzStringPointer (abs)^.SeitenVektor^[0], h^[0],
        WORD (ind * SizeOf (h^[0])));
  Move (SeitenVektor^[0], h^[ind], WORD (AnzahlSeiten * SizeOf (h^[0])));

  IF ind > 0 THEN
    FOR i := 1 TO ind - 1 DO BEGIN
      IF h^[i] <> NIL THEN
        einNetzStringPointer (h^[i])^.setzeRueck (@SELF, TRUE);
    END (* FOR *);

  FreeMem (SeitenVektor, AnzahlSeiten * SizeOf (SeitenVektor^[0]));
  SeitenVektor := h; AnzahlElemente := nanzahl;
  (* AnzahlSeiten := nAnzahlSeiten; <-- hier offenbar falsch, warum??? *)
  Offset := einNetzStringPointer (abs)^.Offset;
  Rueck := einNetzStringPointer (abs)^.Rueck;

  (* Eventuell durch resizeVektor SeitenVektor schrumpfen... ??? *)
  (* resizeVektor (AnzahlElemente); *)

END (* wortZusammenlegen *);


FUNCTION einNetzString.LastElement: GLP;
(* verstellt aktuelles Element *)
BEGIN
  LastElement := NIL {@SELF};
END;


FUNCTION einNetzString.letzterBegriff: GLP;
(* alphabetisch letzten Begriff *)
BEGIN
  letzterBegriff := NIL; (* Ist das gut??? *)
END;

{}
FUNCTION einNetzString.indiziereBegriffe (VAR Index: LONGINT; ind: WORD;
                                          CONST stop: GLP;
                                          VAR gefunden: BOOLEAN): GLP;
(* siehe Vorbemerkung - liefert Anzahl der Begriffe
   und setzt diese. In jedem Begriffsobjekt wird
   das Feld "meineReferenz" auf den Index gesetzt. "Stop" ist
   der Begriff, bei dem gestoppt werden soll, NIL fr
   Ende des Netzes. Da diese Prozedur keine Rckzeiger verfolgt,
   nur vom Anfang des Labyrinthes (Lexikon) aus aufrufen! *)
VAR beg: GLP;
BEGIN
  (* Wir whlen den uersten Index: *)
  INC (Index); meineReferenz := Index;
  (* gefunden wird in einLexikon.indiziereBegriffe
     vor dem Start auf FALSE gesetzt *)
  IF (SeitenVektor = NIL) OR (AnzahlSeiten = 0) THEN BEGIN
    indiziereBegriffe := @SELF; Exit;
  END (* IF *);
  IF @SELF <> stop THEN BEGIN
    beg := @SELF; (* Falls es im Seitenvektor keine Begriffe mehr gibt,
                     bin ich der letzte *)
    IF ind > AnzahlSeiten THEN ind := AnzahlSeiten;
    REPEAT
      IF ind > 0 THEN DEC (ind);
      IF (SeitenVektor^[ind] <> NIL) THEN BEGIN
        (*indiziereBegriffe :=*)
        beg :=
         einNetzStringPointer (SeitenVektor^[ind])^.indiziereBegriffe
             (Index, einNetzStringPointer (SeitenVektor^[ind])^.AnzahlSeiten,
              stop, gefunden);
        indiziereBegriffe := beg;
      END (* IF *)
      ELSE indiziereBegriffe := beg;
    UNTIL gefunden OR (ind = 0);
  END (* SELF <> stop *)
  ELSE BEGIN
    gefunden := TRUE; indiziereBegriffe := @SELF;
  END (* ELSE *);
END (* indiziereBegriffe *);
{}

FUNCTION einNetzString.naechsterBegriff (ind: WORD): GLP;
(* alphabetisch nchsten Begriff nach dem
   Begriff SELF, NIL wenn keiner
   mehr vorhanden. *)
VAR dummys:  STRING;
    inds:    STRING [10];
BEGIN
{
IF istProtokollAn THEN schreibProtokoll ('Aufruf von naechsterBegriff in',
atString (@SELF) + ', Offset: ' + StringVonNum (Offset) + ', "' + StringVonDaten
+ '"');
}
  IF (SeitenVektor <> NIL) AND (AnzahlSeiten > 0) THEN BEGIN
    IF ind > AnzahlSeiten THEN ind := AnzahlSeiten + 1;
    IF ind > 0 THEN DEC (ind);
    WHILE (ind > 0) AND (SeitenVektor^[ind] = NIL) DO BEGIN
      DEC (ind);
    END (* WHILE *);

    IF (SeitenVektor^[ind] = NIL) AND (Rueck <> NIL) THEN BEGIN
{
IF istProtokollAn THEN schreibProtokoll ('naechsterBegriff', '(2), ind: '
+ StringVonNum (ind));
}
      naechsterBegriff :=
        einNetzStringPointer (Rueck)^.naechsterBegriff
          (Offset - einNetzStringPointer (Rueck)^.Offset);
    END
    ELSE BEGIN
{
IF istProtokollAn THEN schreibProtokoll ('naechsterBegriff', '(3), ind: '
+ StringVonNum (ind) + ', ' + atString (SeitenVektor^[ind]));
}
      naechsterBegriff := SeitenVektor^[ind];
    END (* ELSE *)
  END
  ELSE IF (Rueck <> NIL) THEN BEGIN
{
IF istProtokollAn THEN schreibProtokoll ('naechsterBegriff', '(4), ind: '
+ StringVonNum (ind));
}
    naechsterBegriff :=
      einNetzStringPointer (Rueck)^.naechsterBegriff
        (Offset - einNetzStringPointer (Rueck)^.Offset);
  END
  ELSE BEGIN
{
IF istProtokollAn THEN schreibProtokoll ('naechsterBegriff', '(5)');
}
    naechsterBegriff := NIL;
  END (* ELSE *);
END (* naechsterBegriff *);

FUNCTION einNetzString.vorigerBegriff: GLP;
(* alphabetisch voriger Begriff nach dem
   Begriff SELF, NIL wenn keiner
   mehr vorhanden, n: Sprung - siehe einLexikon *)
BEGIN
  Abstract ('vorigerBegriff');
  vorigerBegriff := NIL;
END;

FUNCTION einNetzString.aehnlichsterBegriff (CONST wort: StringPointer;
                                            VAR verg: StringRelation;
                                            VAR gleichbis: WORD;
                                            CONST ind: WORD;
                                            VAR abs: GLP): GLP;
(*  Diese Prozedur sucht den lexikalisch hnlichsten Begriff. hnlichkeit
    wird als Lnge gleichlautender Wortstmme definiert.
    Der Suchprozess versucht, den Suchbegriff auf den Baum der gespeicherten
    Begriffe abzubilden.
    Endet dies mit einer vollstndigen bereinstimmung, so ist der hnlichste
    Begriff gefunden - es ist der identische.
    Andernfalls ist eine Unterentscheidung zu treffen.
    Ist das Suchwort nicht vorhanden, aber Teil eines vorhandenen Begriffes,
    so wird dieser ausgegeben: Han fhrt zu Hans.

    Die Prozedur "vergleiche" fhrt einen lokalen Vergleich durch, und zwar
    vergleicht sie die gespeicherte Teilkette mit dem Such-"wort" ab ind.
    Der Teil des Suchwortes davor wird ignoriert, da er ja schon als gleich
    mit dem bisherigen Suchpfad erkannt wurde.

   "verg" liefert die Relation des gefundenen Begriffes im Vergleich zum
     gesuchten.
   "gleichbis" ist der Index, bis zu dem die Begriffe gleich sind.
   "abs" enthlt NIL oder den vorigen Begriff. *)

VAR   lokalgleichbis:  WORD;

BEGIN (* aehnlichsterBegriff *)
{
zeigTrace ('NetzString.aehnlichsterBegriff ("' + STRING (wort^) + '")',
           StringVonTyp + atString (@SELF) +
           '; Str: ' + StringVonDaten);
}
  verg := vergleiche (wort, ind, lokalgleichbis);
  (* zeigMeldung ('NetzString', 'aehnlichsterBegriff'); *)
  CASE verg OF
    (* Der triviale Fall der Identitt. Der Suchvorgang terminiert. *)
    gleich:         BEGIN
                      zeigTrace (StringVonDaten, 'gleich');
                      gleichbis := lokalgleichbis + Offset;
                      aehnlichsterBegriff := @SELF;
                    END;

    groesser:       BEGIN
                      zeigTrace (StringVonDaten, 'groesser');

    (* Wenn der Suchbegriff grer ist, mu entschieden werden, ob die
       Suche fortgesetzt werden kann: *)

                      IF (SeitenVektor <> NIL) AND
                         (SeitenVektor^[0] <> NIL) THEN BEGIN
                        abs := @SELF;
                        aehnlichsterBegriff := einNetzStringPointer (
                        SeitenVektor^[0])^.aehnlichsterBegriff (
                         wort, verg, lokalgleichbis, Offset, abs);
                      END (* IF weitersuchen *)

                      ELSE BEGIN
     (* Wenn nicht, stellt der bisherige Begriff die grtmgliche
        Annherung dar, denn sonst wre - aufgrund der lexikalischen
        Sortierung - bereits ein anderer Pfad durchlaufen worden *)
                        gleichbis := lokalgleichbis + Offset;
                        aehnlichsterBegriff := @SELF;
                      END (* ELSE *);
                    END;

    groessergleich: BEGIN
                      zeigTrace (StringVonDaten, 'groessergleich');

    (* Auch hier gibt es den Fall, da die Suche fortzustzen ist: *)

                      IF (lokalgleichbis < AnzahlSeiten) AND
                         (SeitenVektor^[lokalgleichbis] <> NIL) THEN BEGIN
                        abs := @SELF;
                        aehnlichsterBegriff := einNetzStringPointer (
                          SeitenVektor^[lokalgleichbis])^.aehnlichsterBegriff (
                  wort, verg, lokalgleichbis, lokalgleichbis + Offset, abs);
                      END
                      ELSE BEGIN

    (* Es gibt keinen weiteren Suchpfad, also auch keinen hnlicheren
       Begriff *)
                        gleichbis := lokalgleichbis + Offset;
                        aehnlichsterBegriff := @SELF;
                      END (* ELSE *);
                    END;


    kleinergleich:  BEGIN
    (* Der Suchbegriff ist teilweise mit diesem Pfad gleich und mu daher
       diesem am hnlichsten sein, da es keine weiteren hnlicheren
       Begriffe mehr gibt. *)

                      zeigTrace (StringVonDaten, 'kleinergleich');
                      gleichbis := lokalgleichbis + Offset;
                      aehnlichsterBegriff := @SELF;
                    END;

    kleiner:        BEGIN
    (* Dies ist der einzige Fall, der unklar ist. Daher mu entschieden
       werden, ob der letzvorige Begriff hnlicher war als der jetzige *)

                      zeigTrace (StringVonDaten, 'kleiner');

                      IF (gleichbis + Offset >= lokalgleichbis + Offset) THEN
                        aehnlichsterBegriff := abs
                      ELSE aehnlichsterBegriff := @SELF;
                    END;
    ELSE            BEGIN
                      zeigMeldung ('aehnlichster Begriff', 'INTERNER FEHLER');
                      aehnlichsterBegriff := NIL;
                    END;

  END (* verg *);
END (* aehnlichsterBegriff *);

FUNCTION einNetzString.StringVergleich (CONST s1, s2: STRING): StringRelation;
BEGIN
END (* StringVergleich *);

FUNCTION einNetzString.Liste (CONST n: LONGINT; CONST Start: GLP): GLP;
(* Gibt die Liste der n Begriffe, die dem Begriff
   Start alphabetisch folgen, einschlielich
   von Start. *)
BEGIN
END;

PROCEDURE einNetzString.setzAssoziation (CONST Assoz: GLP);
(* Verknpfe Wort mit einem GLF-Pointer.
   Bemerkung: Wenn das Objekt ein NetzString ist,
   entsteht eine lokale Wortbasis, die ber den
   ersten Namen ansprechbar ist. *)
BEGIN
{zeigMeldung ('einNetzString', 'setzAssoziation');}
  Data := Assoz;
END (* setzAssoziation *);

FUNCTION einNetzString.liesAssoziation: GLP;
(* Lies den mit einem Wort verknpften GLP-
   Pointer (Data), kann NIL sein wenn nie
   gesetzt. *)
BEGIN
  liesAssoziation := Data;
END;

PROCEDURE einNetzString.baueString (VAR ind: WORD;
                                    CONST stop: WORD;
                                    VAR str: STRING);
VAR  rekind: WORD;
     i:      WORD;
BEGIN
  rekind := ind;
  IF Rueck <> NIL THEN BEGIN
    einNetzStringPointer (Rueck)^.baueString (rekind, Offset, str);
  END;
  (* Write ('rekind: ', rekind, ', stop: ', stop); ReadLn; *)
  IF stop > 0 THEN BEGIN
    FOR i := rekind TO rekind + stop - 1 DO BEGIN
      str [i + 1] := derString^.Str [i - rekind];
    END (* FOR *);
  END (* IF stop *);
  ind := rekind + stop - Offset;
END (* baueString *);

FUNCTION einNetzString.neuerNetzString: GLP;
BEGIN
  neuerNetzString := neuesObjekt (NetzString, []);
END (* neuerNetzString *);

PROCEDURE einNetzString.StringvonBegriff (VAR str: STRING);
VAR   rekind:   WORD;
      i:        WORD;
      strp:     StringPointer;
BEGIN
  IF derString <> NIL THEN BEGIN
    rekind := 0;
    IF Rueck <> NIL THEN BEGIN
      einNetzStringPointer (Rueck)^.baueString (rekind, Offset, str);
    END;
    {
    FOR i := rekind TO rekind + derString^.Laenge DO BEGIN
      str [i + 1] := derString^.Str [i - rekind];
    END (* FOR *);
    }
{zeigTrace ('NetzString.StringVonBegriff', 'Move');}
    MOVE (derString^.Str [0], str [rekind + 1], derString^.Laenge);
    SSTRING (str).Laenge := rekind + derString^.Laenge;
    strp := @str;
{zeigTrace ('NetzString.StringVonBegriff', 'denormalisiere');}
    denormalisiereBezeichner (strp, Schreibung);
{zeigTrace ('NetzString.StringVonBegriff', 'END');}
  END
  ELSE str := '';
END (* StringvonBegriff *);

FUNCTION einNetzString.StringVonDaten: STRING;
VAR s: STRING;
BEGIN
  StringVonBegriff (s);
{zeigMeldung ('einNetzString.StringvonDaten', s);}
  StringVonDaten := s;
END (* StringVonDaten *);

FUNCTION einNetzString.getNamePointer: GLP;
BEGIN
  getNamePointer := @SELF;
END (* getNamePointer *);

FUNCTION einNetzString.getDataPointer: GLP;
BEGIN
  getDataPointer := liesAssoziation;
END (* GLP *);

FUNCTION einNetzString.getDataStr: STRING;
VAR  datap: GLP;
BEGIN
  datap := liesAssoziation;
  IF datap <> NIL THEN
    getDataStr := datap^.StringVonDaten
  ELSE getDataStr := StringVonTyp;
END (* getDataStr *);

FUNCTION einNetzString.insertStringConst (s: STRING;
                                     CONST eindeutig: BOOLEAN): einNetzStringPointer;
BEGIN
  insertStringConst := insertString (s, eindeutig);
END (* insertStringConst *);

(* Zur Vererbung an ein Lexikon - abstrakt *)
FUNCTION einNetzString.insertString (CONST s: STRING;
                                     CONST eindeutig: BOOLEAN): einNetzStringPointer;
BEGIN
  Abstract ('insertString');
END (* insertString *);

PROCEDURE einNetzString.Monitor;
VAR  i:   WORD;
     ds:  STRING;
     s:   STRING;
BEGIN
  INHERITED Monitor (view);
  ds := StringVonDaten;
  view^.WriteString ('Begriff:          ', 0, 0); view^.WriteString (ds, 0, 0);
    view^.WriteLn;
  view^.WriteString ('Schreibung:       ', 0, 0); view^.WritePointer (Schreibung); view^.WriteLn;
  view^.WriteString ('BitMap:           ', 0, 0);
  IF (Schreibung <> NIL) AND (derString^.Laenge + Offset - 1 > 0) THEN BEGIN
    FOR i := 0 TO derString^.Laenge + Offset - 1 DO BEGIN
      IF IsBit (Schreibung, i) THEN view^.WriteChr ('1')
        ELSE view^.WriteChr ('0');
    END (* FOR *);
  END
  ELSE BEGIN view^.WriteString ('NIL', 0, 0);
  END;
    view^.WriteLn;

  view^.WriteString ('SeitenVektor:     ', 0, 0); view^.WritePointer (SeitenVektor);
    view^.WriteLn;
  view^.WriteString ('AnzahlSeiten:   ', 0, 0); view^.WriteInt (AnzahlSeiten);
    view^.WriteLn;
  IF AnzahlSeiten > 0 THEN BEGIN
    FOR i := 0 TO AnzahlSeiten - 1 DO BEGIN
      view^.WriteString ('[', 0, 0);
      view^.WriteIntF (i, 3);
      view^.WriteString     (']             ', 0, 0);
      view^.WriteGLFPointer (SeitenVektor^[i]);
      IF SeitenVektor^[i] <> NIL THEN BEGIN
        s := SeitenVektor^[i]^.StringVonDaten; view^.WriteString (' "', 0, 0);
        view^.WriteString (s, 0, 0); view^.WriteChr ('"');
      END;
      view^.WriteLn;
    END (* FOR *);
  END (* IF *);
  view^.WriteString ('Rueck:            ', 0, 0); view^.WriteGLFPointer (Rueck); view^.WriteLn;
  view^.WriteString ('Data:             ', 0, 0); view^.WriteGLFPointer (data); view^.WriteLn;
  view^.WriteString ('Offset:           ', 0, 0); view^.WriteInt (Offset); view^.WriteLn;
END (* Monitor *);

END (* GLFWordBase *).